--[[
		Recap - Hawksy (on Elune) (modified for WoW 2)
			  - original development by Gello (versions 3.32 and earlier)

		An AddOn for WoW to track and summarize all damage and healing done around the user.
]]

Recap_Version = 3.67
Recap_Sync_Version = 361 -- used to determine compatible versions of synchronization (integer for compact transmission) (only update if Recap sync format changes to be incompatible)


--[[ Saved Variables ]]

recap = {} -- options and current data (saved to SavedVariables)
recap_set = {} -- fight data sets (saved to SavedVariables)


--[[ Ace2 setup ]]

-- mixin AceComm for sync purposes
Recap = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceComm-2.0")

-- other Ace2 setup
Recap:SetCommPrefix("Recap")


--[[ local tables ]]

-- colors for the gauges
local gaugecolor = { ["DmgIn"] = { r=.66,g=.25,b=.25 }, ["DmgOut"] = { r=.25,g=.66,b=.35 }, ["Heal"] = { r=.25,g=.5,b=.85 },
					 ["Time"] = { r=.75, g=.75, b=.75 }, ["Kills"] = { r=.75, g=.75, b=.75 }, ["MaxHit"] = { r=.75, g=.75, b=.75 },
					 ["DPS"] = { r=.25, g=.66, b=.35 }, ["Over"] = { r=.25, g=.5, b=.85 } }

-- .Opt.x and its buttons
local recapbuttons = { ["Minimized"] = "RecapMinimizeButton", ["Pinned"] = "RecapPinButton",
					   ["Paused"] = "RecapPauseButton", ["SelfView"] = "RecapSelfViewButton" }


--[[ Callback functions ]]

function Recap:OnInitialize()
	-- slash commands
	SlashCmdList["RecapCOMMAND"] = Recap_SlashHandler
	SLASH_RecapCOMMAND1 = "/Recap"
end

function Recap:OnEnable()

	-- called on login and on reloadui
	-- recap_temp cleared, so Recap_Initialize will be called and its interior code will be executed

	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end

	-- register some events
	Recap:RegisterEvent("PLAYER_ENTERING_WORLD")
	Recap:RegisterEvent("UNIT_NAME_UPDATE")
	Recap:RegisterEvent("PLAYER_REGEN_DISABLED")
	Recap:RegisterEvent("PLAYER_REGEN_ENABLED")
	Recap:RegisterEvent("PARTY_MEMBERS_CHANGED")
	Recap:RegisterEvent("RAID_ROSTER_UPDATE")
end

function Recap:OnDisable()
	-- nothing right now
end

function Recap:PLAYER_ENTERING_WORLD()

	-- called on login and on reloadui

	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end
end

function Recap:UNIT_NAME_UPDATE()
	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end
end

function Recap:PLAYER_REGEN_DISABLED()
	if recap_temp.Loaded then -- protect from premature execution
		if recap.Opt.State.value=="Idle" then
			Recap_StartFight()
		end
	end
end

function Recap:PLAYER_REGEN_ENABLED()
	if recap_temp.Loaded then -- protect from premature execution
		-- player leaving combat, start the end of fight delay timer
		-- fight will not end until at least this amount of time has elapsed, longer if combat continues
		recap_temp.EndFightDelayTimer = 0
		RecapEndFightDelayFrame:Show() -- start end fight delay OnUpdates
		-- we leave the idle timer on, if it is on (and may the best timer win)
	end
end

function Recap:PARTY_MEMBERS_CHANGED()
	if recap_temp.Loaded then -- protect from premature execution
		Recap_MakeFriends()
	end
end

function Recap:RAID_ROSTER_UPDATE()
	if recap_temp.Loaded then -- protect from premature execution
		Recap_MakeFriends()
	end
end

function Recap:CHAT_MSG_CHANNEL_NOTICE()

	if arg4 and string.find(arg4,"recaplog"..string.lower((UnitName("player")))) and arg1=="YOU_JOINED" then
		-- we have successfully joined our private log channel

		local i, chatnum, chatname
		local logchat = "recaplog"..string.lower((UnitName("player")))

		chatnum = false
		for i=1,10 do
			j,chatname = GetChannelName(i)
			if chatname and string.lower(chatname)==logchat then
				chatnum=j
				break
			end
		end
		if chatnum then
			recap_temp.LogChatnum = chatnum
			-- begin logging to WOWChatLog.txt
			if LoggingChat() then
				-- already on
			else
				-- toggle on
				LoggingChat(true)
			end
			-- allow Report and Close buttons
			RecapOpenLogButton:SetText(RECAP_CHANNEL_JOINED)
			RecapOpenLogButton:Disable()
			RecapReportToLogButton:Enable()
			RecapCloseLogButton:Enable()
		else
			-- oops, didn't work after all
			DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.NoChannelNumber)
			RecapOpenLogButton:SetText(RECAP_OPEN_WOWCHATLOG_TXT)
			RecapOpenLogButton:Enable()
		end
		RecapFrame:UnregisterEvent("CHAT_MSG_CHANNEL_NOTICE")
	end
end

function Recap_Idle_OnUpdate(arg1)

	if not recap_temp.Loaded then
		return
	end

	-- timer to end idle fights if option checked
	if recap_temp.IdleTimer~=-1 and recap.Opt.IdleFight.value then
		recap_temp.IdleTimer = recap_temp.IdleTimer + arg1
		if recap_temp.IdleTimer > recap.Opt.IdleFightTimer.value then
			recap_temp.IdleTimer = -1
			Recap_EndFight(false)
			Recap_CheckJoinSync()
			Recap_CheckSendSummary()
			Recap_CheckProcessSyncData()
		end
	end

	-- if fighting and a plugin loaded or we're minimized, update min dps every half second
	--   (at some times this will be checking while Recap_OnUpdate is making the same check)
	if recap_temp.UpdateTimer~=-1 and recap.Opt.State.value=="Active" and
	  ((IB_Recap_Update or TitanPanelRecap_Update) or recap.Opt.Minimized.value) then
		recap_temp.UpdateTimer = recap_temp.UpdateTimer + arg1
		if recap_temp.UpdateTimer>.5 then
			recap_temp.UpdateTimer = 0
			Recap_UpdateMinimizedDPS()
		end
	end
end

function Recap_EndFightDelay_OnUpdate(arg1)

	if not recap_temp.Loaded then
		return
	end

	-- timer to wait after an end-of-fight is triggered before actually ending the fight
	-- prevents a fight being ended prematurely when it's just a short gap
	if recap_temp.EndFightDelayTimer~=-1 then
		recap_temp.EndFightDelayTimer = recap_temp.EndFightDelayTimer + arg1
		if recap_temp.EndFightDelayTimer > recap.Opt.EndFightDelay.value then
			recap_temp.EndFightDelayTimer = -1
			Recap_EndFight(false)
			Recap_CheckJoinSync()
			Recap_CheckSendSummary()
			Recap_CheckProcessSyncData()
		end
	end

	-- if fighting and a plugin loaded or we're minimized, update min dps every half second
	--   (at some times this will be checking while Recap_OnUpdate is making the same check)
	if recap_temp.UpdateTimer~=-1 and recap.Opt.State.value=="Active" and
	  ((IB_Recap_Update or TitanPanelRecap_Update) or recap.Opt.Minimized.value) then
		recap_temp.UpdateTimer = recap_temp.UpdateTimer + arg1
		if recap_temp.UpdateTimer>.5 then
			recap_temp.UpdateTimer = 0
			Recap_UpdateMinimizedDPS()
		end
	end
end

function Recap_OnUpdate(arg1)

	-- this function should get called continually, unless the entire main panel is hidden

	if not recap_temp.Loaded then
		return
	end

	-- update fade timer if option checked
	if recap.Opt.AutoFade.value and not recap.Opt.Minimized.value and MouseIsOver(RecapFrame) and recap_temp.FadeTimer<recap.Opt.AutoFadeTimer.value then
		recap_temp.FadeTimer = 0
	elseif recap_temp.FadeTimer~=-1 and recap.Opt.AutoFade.value and not recap.Opt.Minimized.value then
		recap_temp.FadeTimer = recap_temp.FadeTimer + arg1
		if recap_temp.FadeTimer > recap.Opt.AutoFadeTimer.value then
			RecapFrame:SetAlpha(max(1-min(2*(recap_temp.FadeTimer-recap.Opt.AutoFadeTimer.value),1),.1))
			if recap_temp.FadeTimer > (recap.Opt.AutoFadeTimer.value+.5) then
				RecapFrame_Hide()
			end
		end
	end

	-- global update timer (every half second if not in 'Stopped' state)
	if recap_temp.UpdateTimer~=-1 and recap.Opt.State.value~="Stopped" then
		recap_temp.UpdateTimer = recap_temp.UpdateTimer + arg1
		if recap_temp.UpdateTimer>0.5 then
			recap_temp.UpdateTimer = 0
			-- if fighting and a plugin loaded or we're minimized, update min dps
			if recap.Opt.State.value=="Active" and ((IB_Recap_Update or TitanPanelRecap_Update) or recap.Opt.Minimized.value) then
				Recap_UpdateMinimizedDPS()
			end
			-- possibly blink the sync status light (0 set to default value, 1 turn off, 2 turn on)
			if recap_temp.SyncStatusLightBlink == 0 then
				if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
					RecapMinSyncStatusTexture:SetVertexColor(0,0,1)
				else
					RecapMinSyncStatusTexture:SetVertexColor(.5,.5,.5)
				end
			elseif recap_temp.SyncStatusLightBlink == 1 then
				recap_temp.SyncStatusLightBlink = 2
				RecapMinSyncStatusTexture:SetVertexColor(.5,.5,.5)
			elseif recap_temp.SyncStatusLightBlink == 2 then
				recap_temp.SyncStatusLightBlink = 0
				RecapMinSyncStatusTexture:SetVertexColor(0,0,1)
			end
			-- check group status
			Recap_JoinLeaveGroup()
		end
	end

	if recap.Opt.AutoMinimize.value and recap_temp.Selected==0 then
		if not recap.Opt.Minimized.value and not MouseIsOver(RecapFrame) and not IsShiftKeyDown() then
			Recap_Minimize()
		elseif recap.Opt.Minimized.value and MouseIsOver(RecapFrame) and not IsShiftKeyDown() then
			Recap_Maximize()
		end
	end
end

function Recap_JoinLeaveGroup()
	-- If option is on, remind player on joining and leaving a group
	-- GetNumPartyMembers returns number of *other* people, not counting you
	-- GetNumRaidMembers returns number of people, including you
	local count = math.max(1+GetNumPartyMembers(),GetNumRaidMembers())
	if (recap.Opt.NumPartyMembers.value == 1) and (count > 1) and recap.Opt.RemindGroupStatus.value then
		StaticPopupDialogs["recap_temp.REMINDGROUP"] = {
			text = TEXT(recap_temp.Localize.RemindJoinGroup),
			button1 = TEXT(OKAY),
			OnAccept = function()
			end,
			timeout = 0,
			showAlert = 1,
			whileDead = 1
		}
		StaticPopup_Show("recap_temp.REMINDGROUP")
	elseif (recap.Opt.NumPartyMembers.value > 1) and (count == 1) and recap.Opt.RemindGroupStatus.value then
		StaticPopupDialogs["recap_temp.REMINDGROUP"] = {
			text = TEXT(recap_temp.Localize.RemindLeaveGroup),
			button1 = TEXT(OKAY),
			OnAccept = function()
			end,
			timeout = 0,
			showAlert = 1,
			whileDead = 1
		}
		StaticPopup_Show("recap_temp.REMINDGROUP")
	end
	recap.Opt.NumPartyMembers.value = count
end

function Recap_SlashHandler(arg1)

	local i

	if not Recap_Initialize() then
		return false
	end

	if arg1 and arg1=="reset" then
		RecapFrame:ClearAllPoints()
		RecapFrame:SetPoint("CENTER","UIParent","CENTER")
		RecapOptFrame:ClearAllPoints()
		RecapOptFrame:SetPoint("CENTER","UIParent","CENTER")
		Recap_SetView()

	elseif arg1=="debug" then
		if recap.debug then
			-- TODO: these puppies aren't localized
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug is ON. To turn it off: /recap debug off")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug is OFF. To turn it on: /recap debug on")
		end
	elseif arg1=="debug on" then
		if recap.debug then
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was on, and is still ON.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was off, and is now ON.")
		end
		recap.debug = true
	elseif arg1=="debug off" then
		if recap.debug then
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was on, and is now OFF.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was off, and is still OFF.")
		end
		recap.debug = false

	-- Hawksy: temporary debug setting, to automatically screenshot
	elseif arg1=="snap" then
		if recap_temp.snap then
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap is ON. To turn it off: /recap snap off")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap is OFF. To turn it on: /recap snap on")
		end
	elseif arg1=="snap on" then
		if recap_temp.snap then
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap was on, and is still ON.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap was off, and is now ON.")
		end
		recap_temp.snap = true
	elseif arg1=="snap off" then
		if recap_temp.snap then
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap was on, and is now OFF.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap was off, and is still OFF.")
		end
		recap_temp.snap = false

	-- Hawksy: another debug setting, to dump ParserLib events to WOWChatLog.txt
	elseif arg1=="dump" then
		if recap.dump then
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump is ON. To turn it off: /recap dump off")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump is OFF. To turn it on: /recap dump on")
		end
	elseif arg1=="dump on" then
		if recap_temp.LogChatnum then
			-- have a log channel, WOWChatLog should be open with logging turned on
			if recap.dump then
				DEFAULT_CHAT_FRAME:AddMessage("recap.dump was on, and is still ON.")
			else
				DEFAULT_CHAT_FRAME:AddMessage("recap.dump was off, and is now ON.")
			end
			SendChatMessage(string.format(recap_temp.Localize.LogHeader,(UnitName("player")),recap_temp.LastAll[recap.Opt.View.value],date(),GetRealZoneText()), "CHANNEL", nil, recap_temp.LogChatnum)
			recap.dump = true
			recap.dumplines = 0
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump logging not on, please go to Options/Records and Open the WOWChatLog.txt file.")
		end
	elseif arg1=="dump off" then
		if recap.dump then
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump was on, and is now OFF.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump was off, and is still OFF.")
		end
		recap.dump = false

	elseif arg1 and string.find(arg1,"opt") then
		if RecapOptFrame:IsShown() then
			RecapOptFrame:Hide()
		else
			RecapOptFrame:Show()
			Recap_SetSyncDisplay()
		end

	elseif RecapFrame:IsShown() then
		RecapFrame_Hide()
		RecapOptFrame:Hide()

	else
		RecapFrame_Show()
		Recap_CheckWindowBounds()
		if recap.Opt.State.value == "Stopped" then
			Recap_ClearAsIfEndOfFight()
			Recap_SetState("Idle")
			recap.Opt.Paused.value = false
			Recap_SetButtons()
		end
	end
end

--[[ Window appearance functions ]]

function Recap_SetState(arg1)
	if arg1 then
		recap.Opt.State.value = arg1
	end
	if recap.Opt.State.value=="Stopped" then
		RecapStatusTexture:SetVertexColor(1,0,0)
		Recap_Unregister_CombatEvents()
	elseif recap.Opt.State.value=="Active" then
		RecapStatusTexture:SetVertexColor(0,1,0)
		Recap_Register_CombatEvents()
	elseif recap.Opt.State.value=="Idle" then
		RecapStatusTexture:SetVertexColor(.5,.5,.5)
		Recap_Register_CombatEvents()
	else
		-- should not occur
	end
	if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
		RecapMinSyncStatusTexture:SetVertexColor(0,0,1)
	else
		RecapMinSyncStatusTexture:SetVertexColor(.5,.5,.5)
	end
	Recap_SetButtons()
	Recap_UpdatePlugins()
end

function RecapFrame_Hide()

	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end
	recap_temp.FadeTimer = -1
	RecapFrame:SetAlpha(1)
	recap.Opt.Visible.value = false
	RecapFrame:Hide()
end

function RecapFrame_Show()

	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end
	recap.Opt.Visible.value = true
	RecapFrame:Show()
end

-- Hawksy: catch Escape when maximized, and do a minimize instead; and genuinely hide otherwise
function RecapFrame_HideOrMinimize()

	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end
	if recap.Opt.Visible.value then
		-- other code wants it visible, so this must be an Escape, so minimize and show
		-- Minimize will disable the special status of RecapFrame so that it lets the Escape through
		-- Maximize will enable the special status of RecapFrame so that it captures the Escape
		if recap.Opt.MinimizeOnEscape.value then
			-- but only minimize if the option is set
			Recap_Minimize()
		end
		RecapFrame:Show()
	else
		-- other code wants it hidden, so let it happen (called from OnHide, so don't recurse)
	end
end

function RecapOptFrame_Hide()
	RecapOptFrame:Hide()
end

function RecapFrame_Toggle()
	if RecapFrame:IsShown() then
		RecapFrame_Hide()
		RecapOptFrame:Hide()
	else
		RecapFrame_Show()
	end
end

function Recap_Shutdown()

	recap.Opt.Paused.value = true
	recap_temp.HoldFight = "Normal"
	Recap_EndFight(true)
	Recap_SetState("Stopped")
	Recap_SetButtons()
	RecapOptFrame:Hide()
	RecapFrame_Hide()

	-- Hawksy: cleanup for 3.59 and later
	local thisCombatant
	for thisCombatant in pairs(recap.Combatant) do
		if recap.Combatant[thisCombatant].Incoming then
			recap.Combatant[thisCombatant].Incoming = nil -- keep as nil
		end
		if recap.Combatant[thisCombatant].Detail then
			recap.Combatant[thisCombatant].Detail = nil -- keep as nil
		end
	end
end

function Recap_ToggleView()

	if recap.Opt.View.value=="All" then
		Recap_SetView("Last")
	else
		Recap_SetView("All")
	end
	Recap_SetColumns()
end

-- changes button textures depending on their state
function Recap_SetButtons()

	local i

	for i in pairs(recapbuttons) do
		if recap.Opt[i].value then
			getglobal(recapbuttons[i]):SetNormalTexture("Interface\\AddOns\\Recap\\RecapButtons-Down")
			getglobal(recapbuttons[i]):SetPushedTexture("Interface\\AddOns\\Recap\\RecapButtons-Up")
		else
			getglobal(recapbuttons[i]):SetNormalTexture("Interface\\AddOns\\Recap\\RecapButtons-Up")
			getglobal(recapbuttons[i]):SetPushedTexture("Interface\\AddOns\\Recap\\RecapButtons-Down")
		end
	end

	if recap.Opt.View.value=="Last" then
		RecapShowAllButton:SetNormalTexture("Interface\\AddOns\\Recap\\RecapButtons-Up")
		RecapShowAllButton:SetPushedTexture("Interface\\AddOns\\Recap\\RecapButtons-Down")
		if recap_temp.HoldFight == "HoldNext" then
			RecapHoldButton:SetText(RECAP_WAITING)
			RecapHoldButton:Disable()
		elseif recap_temp.HoldFight == "Holding" then
			RecapHoldButton:SetText(RECAP_RESUME)
			RecapHoldButton:Enable()
		else
			recap_temp.HoldFight = "Normal"
			RecapHoldButton:SetText(RECAP_HOLD)
			RecapHoldButton:Enable()
		end
		if recap.Opt.Minimized.value then
			RecapHoldButton:Hide()
		else
			RecapHoldButton:Show()
		end
	else
		RecapShowAllButton:SetNormalTexture("Interface\\AddOns\\Recap\\RecapButtons-Down")
		RecapShowAllButton:SetPushedTexture("Interface\\AddOns\\Recap\\RecapButtons-Up")
		RecapHoldButton:Hide()
	end

	if recap.Opt.SelfView.value then
		RecapHoldButton:Hide()
	end
end

-- this resizes the window width to the columns defined in options
function Recap_SetColumns()

	local i, j, cx, item, subitem

	-- Hawksy: name width was 120 changed to 144 to allow for longer names, especially
	--   under Spells and Abilities on the "Personal Details" panel.
	-- Matching changes made in Recap.xml for parent_EName, parent_Name, RecapHeader_Name, and
	--   RecapHeader_EName.
	cx = recap_temp.ScrollAndEdgeWidth + recap_temp.NameWidth

	if recap.Opt.SelfView.value then
		-- personal view
		for i in pairs(recap.Opt) do
			if recap_temp.DefaultOpt[i].ewidth then
				if recap.Opt[i].value then
					getglobal("RecapHeader_"..i):SetWidth(recap_temp.DefaultOpt[i].ewidth)
					getglobal("RecapHeader_"..i):Show()
					for j=1,15 do
						getglobal("RecapSelf"..j.."_"..i):SetWidth(recap_temp.DefaultOpt[i].ewidth)
						getglobal("RecapSelf"..j.."_"..i):Show()
					end
					cx = cx + recap_temp.DefaultOpt[i].ewidth
				else
					getglobal("RecapHeader_"..i):SetWidth(1)
					getglobal("RecapHeader_"..i):Hide()
					for j=1,15 do
						getglobal("RecapSelf"..j.."_"..i):SetWidth(1)
						getglobal("RecapSelf"..j.."_"..i):Hide()
					end
					cx = cx + 1
				end
			elseif recap_temp.DefaultOpt[i].width then
				getglobal("RecapHeader_"..i):Hide()
			end
		end

		RecapTotals:Hide()
		RecapHeader_Name:Hide()
		RecapHeader_EName:Show()

		for i=1,15 do
			getglobal("RecapList"..i):Hide()
			getglobal("RecapSelf"..i):Show()
			getglobal("RecapSelf"..i):SetWidth(cx - recap_temp.ScrollAndEdgeWidth)
		end

	else
		-- general view
		for i in pairs(recap.Opt) do
			if recap_temp.DefaultOpt[i] and recap_temp.DefaultOpt[i].width then
				if recap.Opt[i].value then
					-- column will appear
					item = getglobal("RecapHeader_"..i)
					item:SetWidth(recap_temp.DefaultOpt[i].width)
					item:Show()
					item = getglobal("RecapTotals_"..i)
					item:SetWidth(recap_temp.DefaultOpt[i].width)
					item:Show()
					-- Hawksy: Faction and Level always show and hide together
					if i == "Faction" then
						-- Hawksy: split the width (hardcoded numbers)
						item:SetWidth(14)
						subitem = getglobal("RecapTotals_Level")
						subitem:SetWidth(18)
						subitem:Show()
					end
					for j=1,15 do
						item = getglobal("RecapList"..j.."_"..i)
						item:SetWidth(recap_temp.DefaultOpt[i].width)
						item:Show()
						-- Hawksy: Faction and Level always show and hide together
						if i == "Faction" then
							-- Hawksy: split the width (hardcoded numbers)
							item:SetWidth(14)
							subitem = getglobal("RecapList"..j.."_Level")
							subitem:SetWidth(18)
							subitem:Show()
						end
					end
					cx = cx + recap_temp.DefaultOpt[i].width
				else
					-- column will not appear
					item = getglobal("RecapHeader_"..i)
					item:SetWidth(1)
					item:Hide()
					-- Hawksy: Ranks, Faction, Class, and Level column fix (see below)
					if (i == "Ranks") or (i == "Faction") or (i == "Class") then
						item:Show()
					end
					-- there is no header for Level
					item = getglobal("RecapTotals_"..i)
					item:SetWidth(1)
					item:Hide()
					-- Hawksy: Ranks, Faction, Class, and Level column fix (see below)
					if (i == "Ranks") or (i == "Faction") or (i == "Class") then
						item:Show()
					end
					-- Hawksy: Faction and Level always show together
					if i == "Faction" then
						subitem = getglobal("RecapTotals_Level")
						subitem:SetWidth(1)
						subitem:Show()
						cx = cx + 1 -- two columns here, not just one
					end
					for j=1,15 do
						item = getglobal("RecapList"..j.."_"..i)
						item:SetWidth(1)
						item:Hide()
						-- Hawksy: Ranks, Faction, Class, and Level column fix
						--   To fix a display bug these columns always appear, even if
						--   only blank (or transparent black) and one column wide.
						--   See also RecapScrollBar_Update and Recap_ConstructList.
						if (i == "Ranks") or (i == "Faction") or (i == "Class") then
							item:Show()
						end
						-- Hawksy: Faction and Level always show together
						if i == "Faction" then
							subitem = getglobal("RecapList"..j.."_Level")
							subitem:SetWidth(1)
							subitem:Show()
						end
					end
					cx = cx + 1
				end
			elseif recap_temp.DefaultOpt[i] and recap_temp.DefaultOpt[i].ewidth then
				getglobal("RecapHeader_"..i):Hide()
			end
		end

		for i=1,15 do
			getglobal("RecapSelf"..i):Hide()
			getglobal("RecapList"..i):Show()
			getglobal("RecapList"..i):SetWidth(cx - recap_temp.ScrollAndEdgeWidth)
		end

		RecapTotals:Show()
		RecapHeader_Name:Show()
		RecapHeader_EName:Hide()

	end

	-- Hawksy: the name slot on the Total line is much narrower
	-- see also the offset in Recap.xml
	RecapTotals_Name:SetWidth(recap_temp.NameWidth-142)
	RecapTotals:SetWidth(cx-142)

	if recap.Opt.GrowLeftwards.value and RecapFrame then
		i = RecapFrame:GetRight()
		j = RecapFrame:GetTop()
		if i and j then
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",i-cx,j) -- *** i not defined sometimes :( **
		else
			-- default behaviour if any undefined
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("CENTER","UIParent","CENTER")
		end
	end

	RecapTopBar:SetWidth(cx-8)
	RecapBottomBar:SetWidth(cx-16)
	RecapFrame:SetWidth(cx)
	RecapScrollBar:SetWidth(cx-16)

	recap_temp.GaugeWidth = cx - recap_temp.ScrollAndEdgeWidth - 3 -
							( (not recap.Opt.SelfView.value) and
							   ((((recap.Opt.Faction.value and recap.Opt.Faction.width) or 1) +
								 ((recap.Opt.Class.value and recap.Opt.Class.width) or 1) +
								 ((recap.Opt.Ranks.value and recap.Opt.Ranks.width) or 1)) or 0) or 0)

	Recap_CheckWindowBounds()
	Recap_SetView()
	RecapScrollBar_Update()

end

-- changes view mode
function Recap_SetView(arg1)

	local text

	if arg1 then
		recap.Opt.View.value = arg1
	end

	Recap_SetButtons()
	Recap_ConstructList()

	cx = RecapFrame:GetWidth()

	-- change titlebar
	if cx and cx>300 and recap.Opt.SelfView.value then
		text = recap_temp.Player..": "..recap_temp.Localize.Details
	elseif cx and cx>230 and recap.Opt.SelfView.value then
		text = recap_temp.Player
	elseif cx and cx<230 or recap.Opt.Minimized.value then
		text = ""
		if cx>180 and not recap.Opt.SelfView.value then
			text = recap.Opt.View.value
		end
	elseif cx and cx<260 then
		text = recap_temp.LastAll[recap.Opt.View.value]
	else
		text = recap_temp.Recap.." "..recap_temp.Localize.Of.." "..recap_temp.LastAll[recap.Opt.View.value]
	end

	RecapTitle:SetText(text)
	RecapHeader_Name:SetText((recap_temp.ListSize-1).." "..RECAP_COMBATANTS)
	Recap_ChangeBack()
end

function Recap_SetColors()

	local i

	if recap.Opt.UseColor.value then
		recap_temp.ColorDmgIn = recap_temp.ColorRed
		recap_temp.ColorDmgOut = recap_temp.ColorGreen
		recap_temp.ColorHeal = recap_temp.ColorBlue
		recap_temp.ColorHits = recap_temp.ColorCyan
		recap_temp.ColorTicks = recap_temp.ColorYellow
		recap_temp.ColorCrits = recap_temp.ColorLime
		recap_temp.ColorMiss = recap_temp.ColorPink
	else
		recap_temp.ColorDmgIn = recap_temp.ColorWhite
		recap_temp.ColorDmgOut = recap_temp.ColorNone
		recap_temp.ColorHeal = recap_temp.ColorWhite
		recap_temp.ColorHits = recap_temp.ColorWhite
		recap_temp.ColorTicks = recap_temp.ColorWhite
		recap_temp.ColorCrits = recap_temp.ColorWhite
		recap_temp.ColorMiss = recap_temp.ColorWhite
	end

	RecapTotals_DmgIn:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	RecapTotals_DmgOut:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	RecapTotals_HPS:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	RecapTotals_DPSIn:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	RecapTotals_DPS:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	RecapTotals_DPSvsAll:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	RecapTotals_Heal:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	RecapTotals_Over:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	RecapMinDPSIn_Text:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	RecapMinDPSOut_Text:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	for i=1,15 do
		getglobal("RecapList"..i.."_HealP"):SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
		getglobal("RecapList"..i.."_Over"):SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
		getglobal("RecapList"..i.."_DmgInP"):SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
		getglobal("RecapList"..i.."_DmgOutP"):SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
		getglobal("RecapSelf"..i.."_EHits"):SetTextColor(recap_temp.ColorHits.r,recap_temp.ColorHits.g,recap_temp.ColorHits.b)
		getglobal("RecapSelf"..i.."_EHitsAvg"):SetTextColor(recap_temp.ColorHits.r,recap_temp.ColorHits.g,recap_temp.ColorHits.b)
		getglobal("RecapSelf"..i.."_EHitsMax"):SetTextColor(recap_temp.ColorHits.r,recap_temp.ColorHits.g,recap_temp.ColorHits.b)
		getglobal("RecapSelf"..i.."_ETicks"):SetTextColor(recap_temp.ColorTicks.r,recap_temp.ColorTicks.g,recap_temp.ColorTicks.b)
		getglobal("RecapSelf"..i.."_ETicksAvg"):SetTextColor(recap_temp.ColorTicks.r,recap_temp.ColorTicks.g,recap_temp.ColorTicks.b)
		getglobal("RecapSelf"..i.."_ETicksMax"):SetTextColor(recap_temp.ColorTicks.r,recap_temp.ColorTicks.g,recap_temp.ColorTicks.b)
		getglobal("RecapSelf"..i.."_ECrits"):SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		getglobal("RecapSelf"..i.."_ECritsAvg"):SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		getglobal("RecapSelf"..i.."_ECritsMax"):SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		getglobal("RecapSelf"..i.."_ECritsP"):SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		getglobal("RecapSelf"..i.."_EMiss"):SetTextColor(recap_temp.ColorMiss.r,recap_temp.ColorMiss.g,recap_temp.ColorMiss.b)
		getglobal("RecapSelf"..i.."_EMissP"):SetTextColor(recap_temp.ColorMiss.r,recap_temp.ColorMiss.g,recap_temp.ColorMiss.b)
	end
	Recap_ChangeBack()
end

function Recap_ChangeBack()

	if recap.Opt.UseColor.value and (recap.Opt.MinBack.value or not recap.Opt.Minimized.value) then

		if recap.Opt.SelfView.value and not recap.Opt.Minimized.value then
			RecapTitleBack:SetVertexColor(0,0,.8)
		elseif recap.Opt.View.value=="Last" then
			RecapTitleBack:SetVertexColor(0,.66,0)
		else
			RecapTitleBack:SetVertexColor(.66,0,0)
		end
		RecapTitleBack:Show()
	else
		RecapTitleBack:Hide()
	end
end


--[[ Initialization ]]

-- Can be called often, will immediately dump out if it ran through successfully
function Recap_Initialize()

	local i, x
	local init_time

	if not recap_temp.Loaded then

		if not Recap_IntegrityCheck() then
			return false
		end

		if (UnitName("player")) and (UnitName("player"))~=UNKNOWNOBJECT and RecapFrame and RecapOptFrame then

			init_time = GetTime()

			recap_temp.Player = (UnitName("player"))
			recap_temp.Server = GetCVar("realmName")

			-- recap index for this player
			recap_temp.p = recap_temp.Player..recap_temp.Server -- use for stuff that can be made global
			recap_temp.s = recap_temp.Player..recap_temp.Server -- use for stuff that should never be global

			if recap and recap.UseOneSettings and recap.User then
				recap_temp.p = recap.User
			end

			-- Hawksy: force the dump flags to false, otherwise we could wake up spamming a channel other than our proper log channel
			recap.dump = false
			recap_temp.LogChatnum = false

			-- Ensure that this state variable starts properly
			recap_temp.HoldFight = "Normal"

			-- Hawksy: force snap on for me every time
			if (recap_temp.p == "HawksyElune") or (recap_temp.p == "GalassaElune") or (recap_temp.p == "MetrognomeElune") then
				recap_temp.snap = true
			end

			-- temporary until WoW 2.1, a more sensible garbage collection parameter
			collectgarbage("setpause", 110)

			Recap_InitializeData()

			if recap.Opt.RecentData.value then
				Recap_CreateRecent()
			else
				recap_temp.Recent = {}
			end

			if not recap[recap_temp.p].WindowTop and RecapFrame then
				RecapFrame:ClearAllPoints()
				RecapFrame:SetPoint("CENTER","UIParent","CENTER")
				RecapOptFrame:ClearAllPoints()
				RecapOptFrame:SetPoint("CENTER", "UIParent", "CENTER")
			elseif recap[recap_temp.p].WindowTop and RecapFrame then
				RecapFrame:ClearAllPoints()
				RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",recap[recap_temp.p].WindowLeft,recap[recap_temp.p].WindowTop)
				RecapFrame:SetWidth(recap[recap_temp.p].WindowWidth)
				RecapFrame:SetHeight(recap[recap_temp.p].WindowHeight)
			end

			-- Hawksy: silent repair
			if not recap.Combatant then
				recap.Combatant = {}
			end
			for i in pairs(recap.Combatant) do
				recap.Combatant[i].WasInLast = false
				recap.Combatant[i].WasInCurrent = false
			end
			recap[recap_temp.p].LastDuration = 0
			recap[recap_temp.p].LastDurationIn = 0
			recap[recap_temp.p].LastDurationHeal = 0
			-- guard code for upgrade for 3.59 and later
			if not recap[recap_temp.p].TotalDurationIn then
				recap[recap_temp.p].TotalDurationIn = recap[recap_temp.p].TotalDuration
			end
			if not recap[recap_temp.p].TotalDurationHeal then
				recap[recap_temp.p].TotalDurationHeal = recap[recap_temp.p].TotalDuration
			end
			-- Hawksy: silent repair
			if not recap.Self[recap_temp.s] then
				recap.Self[recap_temp.s] = {}
			end
			-- not at the moment being done for the player's pets

			Recap_SetColors()

			-- if we loaded in any state except 'Stopped' (i.e. perhaps logged out in mid-fight), force into idle state
			if recap.Opt.State.value ~= "Stopped" then
				Recap_ClearAsIfEndOfFight()
				recap.Opt.State.value = "Idle"
			end
			Recap_SetState(false)

			-- Hawksy: was 82 now 144
			RecapTotals_Name:SetWidth(recap_temp.NameWidth)

			Recap_InitDataSets()

			if recap.Opt.LightData.value then
				recap.Opt.SelfView.value = false
				RecapSelfViewButton:Hide()
			end

			if recap.Opt.Minimized.value then
				Recap_SetButtons()
				Recap_ConstructList()
				Recap_Minimize()
			else
				Recap_Maximize()
			end

			Recap_Register_CombatEvents()

			if recap.Opt.MaxEventRange.Value then
				Recap_SetEventRange(true)
			end

			recap_temp.Loaded = true

			Recap_InitializeOptions()
			Recap_MoveMinimize()
			Recap_SetRecapFrameBackdrop()

			-- initialize panel
			recap_temp.Selected = 0
			if recap.Opt.LightData.value then
				recap.Opt.PanelView.value = 3
				RecapPanelTab1:SetAlpha(.5)
				RecapPanelTab2:SetAlpha(.5)
			end
			RecapPanel_SwitchPanels(recap.Opt.PanelView.value)
			if recap.Opt.PanelAnchor.value then
				RecapPanel:ClearAllPoints()
				RecapPanel:SetPoint(recap.Opt.PanelAnchor.Panel,"RecapFrame",recap.Opt.PanelAnchor.Main,Recap_PanelOffset("x"),Recap_PanelOffset("y"))
			end

			-- clear any Sync values dependent on time, since we can't get times that are good across reboots
			if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
				recap.Opt.SyncUpdateWhen.value = math.floor(1000*GetTime()) -- integer thousandths
				recap_temp.SyncData = {}
				recap_temp.SyncInviteTimestamp = false
				recap_temp.SyncInviteLeader = false
				recap_temp.SyncInviteMergePets = false
				recap_temp.SyncInviteWhen = false
				recap_temp.OnCommReceive = false
			end

			-- Hawksy: removed the Alpha reduction for Faction icons

			if recap.Opt.Visible.value then
				RecapFrame_Show()
			else
				RecapFrame_Hide()
			end

			if recap.Opt.Paused.value then
				RecapFrame_Hide()
			end

			-- set frames that need to see the Escape key
			table.insert(UISpecialFrames,"RecapOptFrame")
			table.insert(UISpecialFrames,"RecapPanel")
			-- Hawksy: added main frame so that we can trap Escape and minimize instead
			if recap.Opt.Minimized.value or not recap.Opt.MinimizeOnEscape.value then
				-- a dummy if we are minimized or have Minimize on Escape turned off, so that we don't capture Escape when minimized
				table.insert(UISpecialFrames,"RecapFrameDummy")
			else
				table.insert(UISpecialFrames,"RecapFrame")
			end

			if recap.debug then
				DEFAULT_CHAT_FRAME:AddMessage("(recap.debug) Time to initialize: "..string.format("%.4f",GetTime()-init_time).." seconds.")
			end

			-- warn if user has a lot of data (over 500 combatants)
			x = 0
			for i in pairs(recap.Combatant) do
				x = x + 1
			end
			if x>500 and recap.Opt.WarnData.value then
				StaticPopupDialogs["recap_temp.Localize.LotsOfDataWarning"] = {
					text = TEXT(string.format(recap_temp.Localize.LotsOfDataWarning,x)),
					button1 = TEXT(OKAY),
					timeout = 0,
					showAlert = 1,
					whileDead = 1
				}
				StaticPopup_Show("recap_temp.Localize.LotsOfDataWarning")
			end
		end
	end

	return true
end

function Recap_IntegrityCheck()
	-- Hawksy: this statement, and the corresponding declarations, need to be updated for every version
	--   (an attempt at a simple package integrity check, we'll see if I can live with it)
	if not Recap_lua_367 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (Recap.lua)")
		Recap_Shutdown()
		return false
	end
	if not RecapCombat_lua_367 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapCombat.lua)")
		Recap_Shutdown()
		return false
	end
	if not RecapSync_lua_367 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapSync.lua)")
		Recap_Shutdown()
		return false
	end
	if not RecapAux_lua_367 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapAux.lua)")
		Recap_Shutdown()
		return false
	end
	if not RecapOptions_lua_367 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapOptions.lua)")
		Recap_Shutdown()
		return false
	end
	if not RecapPanel_lua_367 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapPanel.lua)")
		Recap_Shutdown()
		return false
	end
	if not Recap_xml_367 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (Recap.xml)")
		Recap_Shutdown()
		return false
	end
	if not RecapOptions_xml_367 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapOptions.xml)")
		Recap_Shutdown()
		return false
	end
	if not RecapPanel_xml_367 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapPanel.xml)")
		Recap_Shutdown()
		return false
	end
	if not localization_lua_367 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (localization.lua)")
		Recap_Shutdown()
		return false
	end
	if not localization_de_lua_367 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (localization.de.lua)")
		Recap_Shutdown()
		return false
	end
	return true
end

-- arg1==1 to hide buttons, 16 to show
function Recap_ShowButtons(arg1)

	RecapCloseButton:SetWidth(arg1)
	RecapPinButton:SetWidth(arg1)
	RecapPauseButton:SetWidth(arg1)
	RecapOptionsButton:SetWidth(arg1)
	RecapShowAllButton:SetWidth(arg1)

	if arg1==16 then
		RecapCloseButton:Show()
		RecapPinButton:Show()
		RecapPauseButton:Show()
		RecapOptionsButton:Show()
		RecapShowAllButton:Show()
	else
		RecapCloseButton:Hide()
		RecapPinButton:Hide()
		RecapPauseButton:Hide()
		RecapOptionsButton:Hide()
		RecapShowAllButton:Hide()
	end
end

function Recap_Minimize()

	local i, j
	local cx = 20 -- 8(edge)+4(space right of status)+8

	recap.Opt.Minimized.value = true

	-- Hawksy: to support the desired Escape behaviour, obscure the special status of RecapFrame so that it doesn't capture Escape when minimized
	local k, m
	for k,m in ipairs(UISpecialFrames) do
		if m == "RecapFrame" then
			UISpecialFrames[k] = "RecapFrameDummy"
		end
	end

	for i in pairs(recap.Opt) do
		if recap_temp.DefaultOpt[i].minwidth then
			if recap.Opt[i].value then
				item = getglobal("Recap"..i)
				item:SetWidth(recap_temp.DefaultOpt[i].minwidth)
				item:Show()
				cx = cx + recap_temp.DefaultOpt[i].minwidth
			else
				item = getglobal("Recap"..i)
				item:SetWidth(1)
				item:Hide()
				cx = cx + 1
			end
		elseif recap_temp.DefaultOpt[i].width or recap_temp.DefaultOpt[i].ewidth then
			getglobal("RecapHeader_"..i):Hide()
		end
	end

	if recap.Opt.MinButtons.value then
		cx = cx + 108
		Recap_ShowButtons(16)
	else
		Recap_ShowButtons(1)
		cx = cx + 23 -- 5 + 18
	end

	RecapTitle:SetText(recap.Opt.View.value)

	RecapTopBar:Hide()
	RecapBottomBar:Hide()
	RecapResetButton:Hide()
	RecapHoldButton:Hide()

	for i=1,15 do
		getglobal("RecapList"..i):Hide()
		getglobal("RecapSelf"..i):Hide()
	end
	RecapTotals:Hide()
	RecapHeader_Name:Hide()
	RecapHeader_EName:Hide()
	RecapScrollBar:Hide()

	if recap.Opt.AutoMinimize.value then
		cx = cx-15
	end

	if recap.Opt.GrowLeftwards.value and RecapFrame then
		i = RecapFrame:GetRight()
		j = RecapFrame:GetTop()
		if i and j then
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",i-cx,j)
		else
			-- default behaviour if any undefined
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("CENTER","UIParent","CENTER")
		end
	end
	RecapFrame:SetWidth(cx)
	Recap_SetHeight(28)

	if recap.Opt.MinBack.value then
		RecapFrame:SetBackdropColor(1,1,1,1)
		RecapFrame:SetBackdropBorderColor(.5,.5,.5,1)
	else
		RecapFrame:SetBackdropColor(0,0,0,0)
		RecapFrame:SetBackdropBorderColor(0,0,0,0)
	end

	RecapSelfViewButton:Hide()
	RecapPanel_Hide(1)
	Recap_MoveMinimize()
	Recap_ChangeBack()

	RecapMinView:GetLeft()

end

function Recap_Maximize()

	local i

	recap.Opt.Minimized.value = false

	Recap_SetColumns()

	for i in pairs(recap.Opt) do
		if recap_temp.DefaultOpt[i].minwidth then
			item = getglobal("Recap"..i)
			item:SetWidth(recap_temp.DefaultOpt[i].minwidth)
			item:Hide()
		end
	end
	RecapMinStatus:Show()
	RecapMinSyncStatus:Show()
	RecapMinView:Show()
	Recap_ShowButtons(16)

	RecapTopBar:Show()
	RecapBottomBar:Show()
	RecapResetButton:Show()
	if recap.Opt.View.value=="Last" then
		if recap_temp.HoldFight == "HoldNext" then
			RecapHoldButton:SetText(RECAP_WAITING)
			RecapHoldButton:Disable()
		elseif recap_temp.HoldFight == "Holding" then
			RecapHoldButton:SetText(RECAP_RESUME)
			RecapHoldButton:Enable()
		else
			recap_temp.HoldFight = "Normal"
			RecapHoldButton:SetText(RECAP_HOLD)
			RecapHoldButton:Enable()
		end
		RecapHoldButton:Show()
	else
		RecapHoldButton:Hide()
	end

	RecapScrollBar:Show()

	if recap.Opt.SelfView.value then
		RecapTotals:Hide()
		RecapHeader_Name:Hide()
		RecapHeader_EName:Show()
	else
		RecapTotals:Show()
		RecapHeader_Name:Show()
		RecapHeader_EName:Hide()
	end

	RecapFrame:SetBackdropColor(1,1,1,1)
	RecapFrame:SetBackdropBorderColor(.5,.5,.5,1)
	if not recap.Opt.LightData.value then
		RecapSelfViewButton:Show()
	end
	Recap_MoveMinimize()

	-- Hawksy: to support the desired Escape behaviour, make RecapFrame special so that it captures Escape when maximized (but only when we have Minimize on Escape turned on)
	if recap.Opt.MinimizeOnEscape.value then
		local k, m
		for k,m in ipairs(UISpecialFrames) do
			if m == "RecapFrameDummy" then
				UISpecialFrames[k] = "RecapFrame"
			end
		end
	end

end

function Recap_MoveMinimize()

	RecapCloseButton:ClearAllPoints()
	RecapCloseButton:SetPoint("TOPRIGHT", "RecapFrame", "TOPRIGHT", -6, -6)
	RecapMinStatus:ClearAllPoints()
	RecapMinStatus:SetPoint("TOPLEFT", "RecapFrame", "TOPLEFT", 6, -7)
	RecapResetButton:ClearAllPoints()
	RecapResetButton:SetPoint("BOTTOMLEFT", "RecapFrame", "BOTTOMLEFT", 5, 4)
	RecapMinimizeButton:ClearAllPoints()

	if recap.Opt.GrowLeftwards.value and not recap.Opt.GrowUpwards.value then -- topright
		RecapMinimizeButton:SetPoint("TOPRIGHT", "RecapFrame", "TOPRIGHT", -6, -6)
		RecapCloseButton:SetPoint("TOPRIGHT", "RecapMinimizeButton", "TOPLEFT", -2, 0)
	elseif recap.Opt.GrowLeftwards.value and recap.Opt.GrowUpwards.value then -- bottomright
		RecapMinimizeButton:SetPoint("BOTTOMRIGHT", "RecapFrame", "BOTTOMRIGHT", -6, 6)
		if recap.Opt.Minimized.value then
			RecapCloseButton:SetPoint("TOPRIGHT", "RecapMinimizeButton", "TOPLEFT", -2, 0)
		end
	elseif not recap.Opt.GrowLeftwards.value and not recap.Opt.GrowUpwards.value then -- topleft
		RecapMinimizeButton:SetPoint("TOPLEFT", "RecapFrame", "TOPLEFT", 6, -6)
		RecapMinStatus:SetPoint("TOPLEFT", "RecapMinimizeButton", "TOPRIGHT", 2, -1)
	else -- bottomleft
		RecapMinimizeButton:SetPoint("BOTTOMLEFT", "RecapFrame", "BOTTOMLEFT", 6, 6)
		RecapResetButton:SetPoint("BOTTOMLEFT", "RecapFrame", "BOTTOMLEFT", 28, 4)
	end

	if recap.Opt.Minimized.value and not recap.Opt.GrowLeftwards.value then
		RecapMinStatus:SetPoint("TOPLEFT", "RecapMinimizeButton", "TOPRIGHT", 2, -1)
	end

	if recap.Opt.AutoMinimize.value then
		RecapMinimizeButton:SetWidth(1)
		RecapMinimizeButton:Hide()
	else
		RecapMinimizeButton:SetWidth(16)
		RecapMinimizeButton:Show()
	end

end

function Recap_SetHeight(newcy)

	local i,j

	if recap.Opt.GrowUpwards.value and RecapFrame then
		i = RecapFrame:GetBottom()
		j = RecapFrame:GetLeft()
		if i and j then
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",j,i+newcy)
		else
			-- default behaviour if any undefined
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("CENTER","UIParent","CENTER")
		end
	end

	RecapFrame:SetHeight(newcy)
	Recap_CheckWindowBounds()
end

function RecapScrollBar_Update()

	local i, j, index, item
	local listsize = recap.Opt.SelfView.value and recap_temp.SelfListSize or recap_temp.ListSize
	local thisCombatant

	if not recap.Opt.Minimized.value then

		i = 72 + ( math.max( math.min(listsize-1,recap.Opt.MaxRows.value),1 )*14 )
		Recap_SetHeight(i)
		RecapScrollBar:SetHeight(i-63)
		FauxScrollFrame_Update(RecapScrollBar,listsize-1,recap.Opt.MaxRows.value,10)

		for i=1,recap.Opt.MaxRows.value do
			index = i + FauxScrollFrame_GetOffset(RecapScrollBar)

			if not recap.Opt.SelfView.value then
				-- general view

				if index < listsize then
					-- Hawksy: guard code
					thisCombatant = recap_temp.List[index].Name
					if thisCombatant and recap.Combatant[thisCombatant] then

						-- Hawksy: there is a display problem when switching from Personal Details
						--   to All Fights or Last Fight if the fights display begins with hidden
						--   columns.  The workaround appears to be to force the first four columns
						--   (Ranks, Faction, Class, and Level) to always appear (though perhaps
						--   with blank content).
						--   See more code in Recap_SetColumns and Recap_ConstructList,
						--   and a texture addition to RecapHeader_Faction.
						if recap.Opt["Ranks"].value then
							-- Ranks will appear
							getglobal("RecapList"..i.."_Ranks"):SetText(index..".")
						else
							-- Ranks needs to be set to blank
							getglobal("RecapList"..i.."_Ranks"):SetText("")
						end
						if recap.Opt["Faction"].value then
							-- Faction and Level will appear
							Recap_SetFactionIcon(i,recap.Combatant[thisCombatant].Faction)
							if recap.Combatant[thisCombatant].Level==-1 then
								getglobal("RecapList"..i.."_Level"):SetText("??")
							elseif recap.Combatant[thisCombatant].Level and tonumber(recap.Combatant[thisCombatant].Level)>0 then
								getglobal("RecapList"..i.."_Level"):SetText(recap.Combatant[thisCombatant].Level)
							end
						else
							-- Faction icon needs to be set to black, and Level to blank
							Recap_SetFactionIcon(i,nil)
							getglobal("RecapList"..i.."_Level"):SetText("")
						end
						if recap.Opt["Class"].value then
							-- Class will appear
							Recap_SetClassIcon(i,recap.Combatant[thisCombatant].Class)
						else
							-- Class icon needs to be set to black
							Recap_SetClassIcon(i,nil)
						end

						item = getglobal("RecapList"..i.."_Name")
						item:SetText(thisCombatant)
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						item = getglobal("RecapList"..i.."_Gauge")
						if recap.Opt.ShowGauges.value and recap_temp.GaugeMax>0 and recap.Combatant[thisCombatant].Friend and recap_temp.GaugeBy then
							item:SetWidth(1+recap_temp.GaugeWidth*recap_temp.List[index][recap_temp.GaugeBy]/recap_temp.GaugeMax)
							item:Show()
						else
							item:Hide()
						end

						getglobal("RecapList"..i.."_Kills"):SetText(recap_temp.List[index].Kills)

						getglobal("RecapList"..i.."_Time"):SetText(Recap_FormatTime(recap_temp.List[index].Time))
						getglobal("RecapList"..i.."_TimeIn"):SetText(Recap_FormatTime(recap_temp.List[index].TimeIn))
						getglobal("RecapList"..i.."_TimeHeal"):SetText(Recap_FormatTime(recap_temp.List[index].TimeHeal))

						item = getglobal("RecapList"..i.."_Heal")
						item:SetText(recap_temp.List[index].RawHeal - recap_temp.List[index].OverHeal)
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						getglobal("RecapList"..i.."_HealP"):SetText(recap.Combatant[thisCombatant].Friend and recap_temp.List[index].HealP.."%" or "")
						getglobal("RecapList"..i.."_Over"):SetText(recap.Combatant[thisCombatant].Friend and recap_temp.List[index].Over.."%" or "")

						getglobal("RecapList"..i.."_MaxHit"):SetText(recap_temp.List[index].MaxHit)

						item = getglobal("RecapList"..i.."_DmgIn")
						item:SetText(recap_temp.List[index].DmgIn)
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						getglobal("RecapList"..i.."_DmgInP"):SetText(recap.Combatant[thisCombatant].Friend and recap_temp.List[index].DmgInP.."%" or "")

						item = getglobal("RecapList"..i.."_DmgOut")
						item:SetText(recap_temp.List[index].DmgOut)
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						getglobal("RecapList"..i.."_DmgOutP"):SetText(recap.Combatant[thisCombatant].Friend and recap_temp.List[index].DmgOutP.."%" or "")

						item = getglobal("RecapList"..i.."_HPS")
						if recap_temp.List[index].HPS<1000 then
							item:SetText(string.format("%.1f",recap_temp.List[index].HPS))
						else
							item:SetText(string.format("%d",recap_temp.List[index].HPS))
						end
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end
						item = getglobal("RecapList"..i.."_DPSIn")
						if recap_temp.List[index].DPSIn<1000 then
							item:SetText(string.format("%.1f",recap_temp.List[index].DPSIn))
						else
							item:SetText(string.format("%d",recap_temp.List[index].DPSIn))
						end
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end
						item = getglobal("RecapList"..i.."_DPS")
						if recap_temp.List[index].DPS<1000 then
							item:SetText(string.format("%.1f",recap_temp.List[index].DPS))
						else
							item:SetText(string.format("%d",recap_temp.List[index].DPS))
						end
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						item = getglobal("RecapList"..i.."_DPSvsAll")
						if recap_temp.List[index].DPSvsAll<1000 then
							item:SetText(string.format("%.1f",recap_temp.List[index].DPSvsAll))
						else
							item:SetText(string.format("%d",recap_temp.List[index].DPSvsAll))
						end
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						item = getglobal("RecapList"..i)
						item:Show()
						if recap_temp.Selected == index then
							item:LockHighlight()
						else
							item:UnlockHighlight()
						end
					end
				else
					getglobal("RecapList"..i):Hide()
					getglobal("RecapList"..i):UnlockHighlight()
				end

				for i=recap.Opt.MaxRows.value+1,15 do
					getglobal("RecapList"..i):Hide()
				end

			elseif recap.Opt.SelfView.value then
				-- personal view

				if index < listsize then

					getglobal("RecapSelf"..i.."_EName"):SetText(string.sub(recap_temp.SelfList[index].EName,2))
					getglobal("RecapSelf"..i.."_EElement"):SetText(recap_temp.SelfList[index].EElement)
					getglobal("RecapSelf"..i.."_ETotal"):SetText(recap_temp.SelfList[index].ETotal)
					getglobal("RecapSelf"..i.."_EHits"):SetText(recap_temp.SelfList[index].EHits)
					getglobal("RecapSelf"..i.."_ETicks"):SetText(recap_temp.SelfList[index].ETicks)
					getglobal("RecapSelf"..i.."_ECrits"):SetText(recap_temp.SelfList[index].ECrits)
					getglobal("RecapSelf"..i.."_EHitsAvg"):SetText(recap_temp.SelfList[index].EHitsAvg)
					getglobal("RecapSelf"..i.."_ETicksAvg"):SetText(recap_temp.SelfList[index].ETicksAvg)
					getglobal("RecapSelf"..i.."_ECritsAvg"):SetText(recap_temp.SelfList[index].ECritsAvg)
					getglobal("RecapSelf"..i.."_EHitsMax"):SetText(recap_temp.SelfList[index].EHitsMax)
					getglobal("RecapSelf"..i.."_ETicksMax"):SetText(recap_temp.SelfList[index].ETicksMax)
					getglobal("RecapSelf"..i.."_ECritsMax"):SetText(recap_temp.SelfList[index].ECritsMax)
					getglobal("RecapSelf"..i.."_ECritsP"):SetText(recap_temp.SelfList[index].ECritsP)
					getglobal("RecapSelf"..i.."_EMiss"):SetText(recap_temp.SelfList[index].EMiss)
					getglobal("RecapSelf"..i.."_EMissP"):SetText(recap_temp.SelfList[index].EMissP)
					getglobal("RecapSelf"..i.."_EMaxAll"):SetText(recap_temp.SelfList[index].EMaxAll)
					getglobal("RecapSelf"..i.."_ETotalP"):SetText(recap_temp.SelfList[index].ETotalP)
					if string.sub(recap_temp.SelfList[index].EName,1,1)<"3" then
						getglobal("RecapSelf"..i.."_EName"):SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						getglobal("RecapSelf"..i.."_EElement"):SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						getglobal("RecapSelf"..i.."_ETotal"):SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						getglobal("RecapSelf"..i.."_ETotalP"):SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
					else
						getglobal("RecapSelf"..i.."_EName"):SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
						getglobal("RecapSelf"..i.."_EElement"):SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
						getglobal("RecapSelf"..i.."_ETotal"):SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
						getglobal("RecapSelf"..i.."_ETotalP"):SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
					end
					getglobal("RecapSelf"..i):Show()
				else
					getglobal("RecapSelf"..i):Hide()
				end

				for i=recap.Opt.MaxRows.value+1,15 do
					getglobal("RecapSelf"..i):Hide()
				end

			end

		end

	end
end


-- returns the current DPS of combatant named arg1 if it exists. UpdateDuration/DamageOut reused to save allocations
function Recap_GetCurrentDPS(arg1)

	if arg1 and recap_temp.Last[arg1] and recap_temp.Last[arg1].DmgOut>0 then
		recap_temp.UpdateDuration = recap_temp.Last[arg1].EndOut - recap_temp.Last[arg1].StartOut
		if recap.Opt.View.value=="All" then
			recap_temp.UpdateDuration = recap_temp.UpdateDuration + recap.Combatant[arg1].TotalTime
		end
		recap_temp.UpdateDamageOut = recap_temp.Last[arg1].DmgOut
		if recap.Opt.View.value=="All" then
			recap_temp.UpdateDamageOut = recap_temp.UpdateDamageOut + recap.Combatant[arg1].TotalDmgOut
		end

		if recap_temp.UpdateDuration > recap_temp.MinTime then
			return (recap_temp.UpdateDamageOut / recap_temp.UpdateDuration)
		end
	end

	return 0
end

function Recap_UpdateMinimizedDPS()

	local i

	if recap.Opt.State.value=="Active" then

		if recap_temp.Last[recap_temp.Player] and recap_temp.Last[recap_temp.Player].DmgOut>0 then
			i = Recap_GetCurrentDPS(recap_temp.Player)+Recap_GetCurrentDPS((UnitName("pet")))
			RecapMinYourDPS_Text:SetText(string.format("%.1f",i))
		end

		-- calculate running total DPS In and Out
		recap_temp.UpdateDuration = 0
		recap_temp.UpdateDurationIn = 0
		if recap_temp.FightEnd>0 and recap_temp.FightStart>0 then
			recap_temp.UpdateDuration = recap_temp.FightEnd - recap_temp.FightStart
		end
		if recap_temp.FightEndIn>0 and recap_temp.FightStartIn>0 then
			recap_temp.UpdateDurationIn = recap_temp.FightEndIn - recap_temp.FightStartIn
		end
		if recap.Opt.View.value=="All" then
			recap_temp.UpdateDuration = recap_temp.UpdateDuration + recap[recap_temp.p].TotalDuration
			recap_temp.UpdateDurationIn = recap_temp.UpdateDurationIn + recap[recap_temp.p].TotalDurationIn
		end

		if recap_temp.UpdateDuration>recap_temp.MinTime or recap_temp.UpdateDurationIn>recap_temp.MinTime then

			recap_temp.UpdateDmgIn = 0
			recap_temp.UpdateDmgOut = 0
			for i in pairs(recap_temp.Last) do
				if recap.Combatant[i].Friend then
					recap_temp.UpdateDmgIn = recap_temp.UpdateDmgIn + recap_temp.Last[i].DmgIn
					recap_temp.UpdateDmgOut = recap_temp.UpdateDmgOut + recap_temp.Last[i].DmgOut
				end
			end

			if recap.Opt.View.value=="All" then
				for i in pairs(recap.Combatant) do
					if recap.Combatant[i].Friend then
						recap_temp.UpdateDmgIn = recap_temp.UpdateDmgIn + recap.Combatant[i].TotalDmgIn
						recap_temp.UpdateDmgOut = recap_temp.UpdateDmgOut + recap.Combatant[i].TotalDmgOut
					end
				end
			end

			if recap_temp.UpdateDuration>0 then
				RecapMinDPSOut_Text:SetText(string.format("%.1f",recap_temp.UpdateDmgOut/recap_temp.UpdateDuration))
			else
				RecapMinDPSOut_Text:SetText("--")
			end
			if recap_temp.UpdateDurationIn>0 then
				RecapMinDPSIn_Text:SetText(string.format("%.1f",recap_temp.UpdateDmgIn/recap_temp.UpdateDurationIn))
			else
				RecapMinDPSIn_Text:SetText("--")
			end

		end

		Recap_UpdatePlugins()
	end
end

function Recap_ConstructSelf()

	local damage, heal, miss, i

	-- Hawksy: clear the 'SelfList' table entirely first (may cause more garbage collection, but
	--   appears to be necessary for 'table.sort' to work properly and to avoid display anomalies).
	recap_temp.SelfListSize = 1
	recap_temp.SelfList = {}

	-- Add player, and all of player's pets
	recap_temp.SelfListSize, damage, heal, miss = Recap_AddSelfListPlayerAndPets()

	if recap_temp.SelfListSize>1 then
		for i=1,recap_temp.SelfListSize-1 do
			if string.sub(recap_temp.SelfList[i].EName,1,1)<"3" and damage>0 then
				recap_temp.SelfList[i].ETotalP = string.format("%d%%",((100*recap_temp.SelfList[i].ETotal/damage)+0.5))
			elseif string.sub(recap_temp.SelfList[i].EName,1,1)>"2" and heal>0 then
				recap_temp.SelfList[i].ETotalP = string.format("%d%%",((100*recap_temp.SelfList[i].ETotal/heal)+0.5))
			else
				recap_temp.SelfList[i].ETotalP = "--"
			end
		end
	end

	-- Hawksy: now sort the personal details by any column
	Recap_SortSelfList()

end

function Recap_AddSelfListPlayerAndPets()

	local i, j, k
	local found, petName
	local idx, damage, heal, miss
	idx = 1
	damage = 0
	heal = 0
	miss = 0

	for i in pairs(recap.Self) do
		if string.find(i, recap_temp.s) then
			-- this is the player or one of the player's pets
			for j in pairs(recap.Self[i]) do
				if not recap_temp.SelfList[idx] then
					recap_temp.SelfList[idx] = {}
				end
				-- Hawksy: skip effects that have zero damage
				local total
				total = (recap.Self[i][j].HitsDmg or 0) + (recap.Self[i][j].CritsDmg or 0) + (recap.Self[i][j].TicksDmg or 0)
				if total == 0 then
					recap.Self[i][j] = nil -- keep as nil
				else
					found,_,petName = string.find(i, "^.+:(.+)$")
					if found then
						-- one of the pets
						if string.sub(j,1,1)=="1" then
							recap_temp.SelfList[idx].EName = "2"..petName..": "..string.sub(j,2)
						else
							recap_temp.SelfList[idx].EName = "4"..petName..": "..string.sub(j,2)
						end
					else
						-- the player
						recap_temp.SelfList[idx].EName = j
					end
					recap_temp.SelfList[idx].EElement = recap.Self[i][j].Element or "?"
					recap_temp.SelfList[idx].ETotal = total
					recap_temp.SelfList[idx].EHits = recap.Self[i][j].Hits or 0
					recap_temp.SelfList[idx].ECrits = recap.Self[i][j].Crits or 0
					recap_temp.SelfList[idx].ETicks = recap.Self[i][j].Ticks or 0
					recap_temp.SelfList[idx].EHitsMax = recap.Self[i][j].HitsMax or 0
					recap_temp.SelfList[idx].ETicksMax = recap.Self[i][j].TicksMax or 0
					recap_temp.SelfList[idx].ECritsMax = recap.Self[i][j].CritsMax or 0
					-- Hawksy: make the averages into numbers (not strings) to allow for sorting
					recap_temp.SelfList[idx].EHitsAvg = tonumber(string.format("%d",((recap_temp.SelfList[idx].EHits>0) and (((recap.Self[i][j].HitsDmg or 0)/recap_temp.SelfList[idx].EHits)+0.5) or 0)))
					recap_temp.SelfList[idx].ETicksAvg = tonumber(string.format("%d",((recap_temp.SelfList[idx].ETicks>0) and (((recap.Self[i][j].TicksDmg or 0)/recap_temp.SelfList[idx].ETicks)+0.5) or 0)))
					recap_temp.SelfList[idx].ECritsAvg = tonumber(string.format("%d",((recap_temp.SelfList[idx].ECrits>0) and (((recap.Self[i][j].CritsDmg or 0)/recap_temp.SelfList[idx].ECrits)+0.5) or 0)))
					recap_temp.SelfList[idx].EMaxAll = math.max( recap_temp.SelfList[idx].EHitsMax, math.max(recap_temp.SelfList[idx].ETicksMax,recap_temp.SelfList[idx].ECritsMax) )
					if tonumber(string.sub(j,1,1))<3 then -- damage event
						miss = (recap.Self[i][j].Missed or 0) + (recap.Self[i][j].Dodged or 0) + (recap.Self[i][j].Parried or 0) + (recap.Self[i][j].Blocked or 0) +
								(recap.Self[i][j].Deflected or 0) + (recap.Self[i][j].Resisted or 0) + (recap.Self[i][j].Reflected or 0) + (recap.Self[i][j].Absorbed or 0) +
								(recap.Self[i][j].Immune or 0) + (recap.Self[i][j].Evaded or 0)
						recap_temp.SelfList[idx].EMiss = miss
						k = miss + recap_temp.SelfList[idx].EHits + recap_temp.SelfList[idx].ECrits
						recap_temp.SelfList[idx].ECritsP = string.format("%.1f%%",(k>0) and (100*recap_temp.SelfList[idx].ECrits/k) or 0)
						recap_temp.SelfList[idx].EMissP = string.format("%.1f%%",(k>0) and (100*recap_temp.SelfList[idx].EMiss/k) or 0)
						damage = damage + recap_temp.SelfList[idx].ETotal
					else
						k = recap_temp.SelfList[idx].EHits + recap_temp.SelfList[idx].ECrits
						recap_temp.SelfList[idx].ECritsP = string.format("%.1f%%",(k>0) and (100*recap_temp.SelfList[idx].ECrits/k) or 0)
						recap_temp.SelfList[idx].EMiss = "--"
						recap_temp.SelfList[idx].EMissP = "--"
						heal = heal + recap_temp.SelfList[idx].ETotal
					end

					if recap_temp.SelfList[idx].ETicks==0 then
						recap_temp.SelfList[idx].ETicks = "--"
						recap_temp.SelfList[idx].ETicksAvg = "--"
						recap_temp.SelfList[idx].ETicksMax = "--"
					end
					if not recap.Self[i][j].CritsEvents then
						recap_temp.SelfList[idx].ECrits = "--"
						recap_temp.SelfList[idx].ECritsAvg = "--"
						recap_temp.SelfList[idx].ECritsMax = "--"
						recap_temp.SelfList[idx].ECritsP = "--"
						recap_temp.SelfList[idx].EHits = "--"
						recap_temp.SelfList[idx].EHitsAvg = "--"
						recap_temp.SelfList[idx].EHitsMax = "--"
					end

					idx = idx + 1
				end
			end
		end
	end

	return idx, damage, heal, miss
end

function Recap_ResetSelfPlayerAndPets()

	local i

	-- reset owner and pets
	for i in pairs(recap.Self) do
		if string.find(i, recap_temp.s) then
			recap.Self[i] = {}
		end
	end
end


function Recap_ConstructList()

	local i, j, dpsin, dpsout, entry, overallDuration, tempname, thisCombatant
	local maxhit = 0
	local dmgin = 0
	local dmgout = 0
	local kills = 0
	local rawheal = 0
	local overheal = 0
	local duration = 0
	local durationIn = 0
	local durationHeal = 0

	if recap.Opt.SelfView.value then
		Recap_ConstructSelf()
	end

	if recap_temp.Selected~=0 then
		tempname = recap_temp.List[recap_temp.Selected].Name
	end

	if recap.Opt.View.value=="Last" then
		overallDuration = recap[recap_temp.p].LastDuration
	else
		overallDuration = recap[recap_temp.p].TotalDuration
	end

	-- Hawksy: clear the 'List' table entirely first (may cause more garbage collection, but
	--   appears to be necessary for 'table.sort' to work properly and to avoid display anomalies).
	recap_temp.ListSize = 1
	recap_temp.List = {}

	for i in pairs(recap.Combatant) do
		if not recap_temp.List[recap_temp.ListSize] then
			recap_temp.List[recap_temp.ListSize] = {}
		end
		entry = false

		-- Hawksy: added heals, so that healers show up from the beginning, rather than
		--   only after they have received or given damage.
		if (recap.Combatant[i].LastDmgIn==0 and recap.Combatant[i].LastDmgOut==0 and
			recap.Combatant[i].TotalDmgOut==0 and recap.Combatant[i].TotalDmgIn==0 and
			recap.Combatant[i].LastRawHeal==0 and recap.Combatant[i].TotalRawHeal==0) or
		   (recap.Opt.SelfData.value and not Recap_CheckForSelf(i, recap.Combatant[i].OwnedBy)) or
		   (recap.Opt.PartyData.value and not Recap_CheckForParty(i, recap.Combatant[i].OwnedBy)) or
		   (recap.Opt.HideFoe.value and not recap.Combatant[i].Friend) or
		   (recap.Opt.HideYardTrash.value and (not recap.Combatant[i].Friend and recap.Combatant[i].TotalKills>1)) or
		   (recap.Combatant[i].Ignore) then
			entry = false -- strip out unwanted combatants

		elseif recap.Opt.View.value=="Last" and recap.Combatant[i].WasInLast then
			-- Hawksy: relax time restriction on "Last Fight" stats to TotalTime rather than LastTime
			-- Hawksy: relax further, for TotalTimeIn or TotalTimeHeal also
			if not recap.Opt.HideZero.value or ((recap.Combatant[i].TotalTime>recap_temp.MinTime) or (recap.Combatant[i].TotalTimeIn>recap_temp.MinTime) or (recap.Combatant[i].TotalTimeHeal>recap_temp.MinTime)) then
				recap_temp.List[recap_temp.ListSize].Name = i
				recap_temp.List[recap_temp.ListSize].Time = recap.Combatant[i].LastTime
				recap_temp.List[recap_temp.ListSize].TimeIn = recap.Combatant[i].LastTimeIn
				recap_temp.List[recap_temp.ListSize].TimeHeal = recap.Combatant[i].LastTimeHeal
				recap_temp.List[recap_temp.ListSize].MaxHit = recap.Combatant[i].LastMaxHit
				recap_temp.List[recap_temp.ListSize].DmgIn = recap.Combatant[i].LastDmgIn
				recap_temp.List[recap_temp.ListSize].DmgOut = recap.Combatant[i].LastDmgOut
				recap_temp.List[recap_temp.ListSize].HPS = recap.Combatant[i].LastHPS
				recap_temp.List[recap_temp.ListSize].DPSIn = recap.Combatant[i].LastDPSIn
				recap_temp.List[recap_temp.ListSize].DPS = recap.Combatant[i].LastDPS
				recap_temp.List[recap_temp.ListSize].DPSvsAll = overallDuration>recap_temp.MinTime and recap.Combatant[i].LastDmgOut/overallDuration or 0
				recap_temp.List[recap_temp.ListSize].Kills = recap.Combatant[i].LastKills
				-- two hidden variables
				recap_temp.List[recap_temp.ListSize].RawHeal = recap.Combatant[i].LastRawHeal
				recap_temp.List[recap_temp.ListSize].OverHeal = recap.Combatant[i].LastOverHeal
				-- calculated rather than assigned
				recap_temp.List[recap_temp.ListSize].Heal = recap.Combatant[i].LastRawHeal - recap.Combatant[i].LastOverHeal
				if recap_temp.List[recap_temp.ListSize].RawHeal > 0 then
					recap_temp.List[recap_temp.ListSize].Over = math.floor(((100*recap_temp.List[recap_temp.ListSize].OverHeal)/recap_temp.List[recap_temp.ListSize].RawHeal)+.5)
				else
					recap_temp.List[recap_temp.ListSize].Over = 0
				end
				entry = true
			end

		elseif recap.Opt.View.value=="All" then
			-- Hawksy: relax further, for TotalTimeIn or TotalTimeHeal also
			if not recap.Opt.HideZero.value or ((recap.Combatant[i].TotalTime>recap_temp.MinTime) or (recap.Combatant[i].TotalTimeIn>recap_temp.MinTime) or (recap.Combatant[i].TotalTimeHeal>recap_temp.MinTime)) then
				recap_temp.List[recap_temp.ListSize].Name = i
				recap_temp.List[recap_temp.ListSize].Time = recap.Combatant[i].TotalTime
				recap_temp.List[recap_temp.ListSize].TimeIn = recap.Combatant[i].TotalTimeIn
				recap_temp.List[recap_temp.ListSize].TimeHeal = recap.Combatant[i].TotalTimeHeal
				recap_temp.List[recap_temp.ListSize].MaxHit = recap.Combatant[i].TotalMaxHit
				recap_temp.List[recap_temp.ListSize].DmgIn = recap.Combatant[i].TotalDmgIn
				recap_temp.List[recap_temp.ListSize].DmgOut = recap.Combatant[i].TotalDmgOut
				recap_temp.List[recap_temp.ListSize].HPS = recap.Combatant[i].TotalHPS
				recap_temp.List[recap_temp.ListSize].DPSIn = recap.Combatant[i].TotalDPSIn
				recap_temp.List[recap_temp.ListSize].DPS = recap.Combatant[i].TotalDPS
				recap_temp.List[recap_temp.ListSize].DPSvsAll = overallDuration>recap_temp.MinTime and recap.Combatant[i].TotalDmgOut/overallDuration or 0
				recap_temp.List[recap_temp.ListSize].Kills = recap.Combatant[i].TotalKills
				-- two hidden variables
				recap_temp.List[recap_temp.ListSize].RawHeal = recap.Combatant[i].TotalRawHeal
				recap_temp.List[recap_temp.ListSize].OverHeal = recap.Combatant[i].TotalOverHeal
				-- calculated rather than assigned
				recap_temp.List[recap_temp.ListSize].Heal = recap.Combatant[i].TotalRawHeal - recap.Combatant[i].TotalOverHeal
				if recap_temp.List[recap_temp.ListSize].RawHeal > 0 then
					recap_temp.List[recap_temp.ListSize].Over = math.floor(((100*recap_temp.List[recap_temp.ListSize].OverHeal)/recap_temp.List[recap_temp.ListSize].RawHeal)+.5)
				else
					recap_temp.List[recap_temp.ListSize].Over = 0
				end
				entry = true
			end
		end

		if entry then
			-- Hawksy: guard code
			thisCombatant = recap_temp.List[recap_temp.ListSize].Name
			if thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
				duration = duration + recap_temp.List[recap_temp.ListSize].Time
				durationIn = durationIn + recap_temp.List[recap_temp.ListSize].TimeIn
				durationHeal = durationHeal + recap_temp.List[recap_temp.ListSize].TimeHeal
				if recap_temp.List[recap_temp.ListSize].MaxHit > maxhit then
					maxhit = recap_temp.List[recap_temp.ListSize].MaxHit
				end
				dmgin = dmgin + recap_temp.List[recap_temp.ListSize].DmgIn
				dmgout = dmgout + recap_temp.List[recap_temp.ListSize].DmgOut
				kills = kills + recap_temp.List[recap_temp.ListSize].Kills
				-- two hidden variables
				rawheal = rawheal + recap_temp.List[recap_temp.ListSize].RawHeal
				overheal = overheal + recap_temp.List[recap_temp.ListSize].OverHeal
			end
			recap_temp.ListSize = recap_temp.ListSize + 1
		end
	end

	if not recap_temp.List[recap_temp.ListSize] then
		recap_temp.List[recap_temp.ListSize] = {}
	end

	-- the final entry is the totals
	recap_temp.List[recap_temp.ListSize].Name = recap_temp.Localize.Totals
	recap_temp.List[recap_temp.ListSize].Time = duration
	RecapTotals_Time:SetText(Recap_FormatTime(duration))
	recap_temp.List[recap_temp.ListSize].TimeIn = durationIn
	RecapTotals_TimeIn:SetText(Recap_FormatTime(durationIn))
	recap_temp.List[recap_temp.ListSize].TimeHeal = durationHeal
	RecapTotals_TimeHeal:SetText(Recap_FormatTime(durationHeal))
	recap_temp.List[recap_temp.ListSize].MaxHit = maxhit
	RecapTotals_MaxHit:SetText(maxhit)
	recap_temp.List[recap_temp.ListSize].DmgIn = dmgin
	RecapTotals_DmgIn:SetText(dmgin)
	recap_temp.List[recap_temp.ListSize].DmgOut = dmgout
	RecapTotals_DmgOut:SetText(dmgout)
	recap_temp.List[recap_temp.ListSize].Kills = kills
	RecapTotals_Kills:SetText(kills)
	-- two hidden variables
	recap_temp.List[recap_temp.ListSize].RawHeal = rawheal
	recap_temp.List[recap_temp.ListSize].OverHeal = overheal
	-- calculated rather than assigned
	recap_temp.List[recap_temp.ListSize].Heal = rawheal - overheal
	RecapTotals_Heal:SetText(rawheal - overheal)
	if (recap_temp.List[recap_temp.ListSize].RawHeal > 0) then
		recap_temp.List[recap_temp.ListSize].Over = math.floor(((100*recap_temp.List[recap_temp.ListSize].OverHeal)/recap_temp.List[recap_temp.ListSize].RawHeal)+.5)
	else
		recap_temp.List[recap_temp.ListSize].Over = 0
	end
	RecapTotals_Over:SetText(string.format("%d%%",((recap_temp.List[recap_temp.ListSize].Over)+0.5)))

	-- three calculations of total friendly DPS, DPS In, and HPS
	if duration > recap_temp.MinTime then
		recap_temp.List[recap_temp.ListSize].TotDPSOut = dmgout/duration
	else
		recap_temp.List[recap_temp.ListSize].TotDPSOut = 0
	end
	if durationIn > recap_temp.MinTime then
		recap_temp.List[recap_temp.ListSize].TotDPSIn = dmgin/durationIn
	else
		recap_temp.List[recap_temp.ListSize].TotDPSIn = 0
	end
	if durationHeal > recap_temp.MinTime then
		recap_temp.List[recap_temp.ListSize].TotHPS = (rawheal - overheal)/durationHeal
	else
		recap_temp.List[recap_temp.ListSize].TotHPS = 0
	end

	if recap_temp.List[recap_temp.ListSize].TotHPS<1000 then
		RecapTotals_HPS:SetText(string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotHPS))
	else
		RecapTotals_HPS:SetText(string.format("%d",recap_temp.List[recap_temp.ListSize].TotHPS))
	end
	if recap_temp.List[recap_temp.ListSize].TotDPSIn<1000 then
		RecapTotals_DPSIn:SetText(string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotDPSIn))
	else
		RecapTotals_DPSIn:SetText(string.format("%d",recap_temp.List[recap_temp.ListSize].TotDPSIn))
	end
	if recap_temp.List[recap_temp.ListSize].TotDPSOut<1000 then
		RecapTotals_DPS:SetText(string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotDPSOut))
	else
		RecapTotals_DPS:SetText(string.format("%d",recap_temp.List[recap_temp.ListSize].TotDPSOut))
	end
	if recap_temp.List[recap_temp.ListSize].TotDPSOut<1000 then
		RecapTotals_DPSvsAll:SetText(string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotDPSOut))
	else
		RecapTotals_DPSvsAll:SetText(string.format("%d",recap_temp.List[recap_temp.ListSize].TotDPSOut))
	end

	-- Hawksy: Class column fix set to black, see also RecapScrollBar_Update and Recap_SetColumns.
	RecapTotals_Class:SetTexCoord(.9,1,.9,1)

	-- strings for when minimized
	RecapMinDPSIn_Text:SetText(string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotDPSIn))
	RecapMinDPSOut_Text:SetText(string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotDPSOut))
	if recap.Combatant[recap_temp.Player] and recap.Opt.View.value=="Last" then
		i = recap.Combatant[recap_temp.Player].LastDPS
		if (UnitName("pet")) and (UnitName("pet"))~=UNKNOWNOBJECT and recap.Combatant[(UnitName("pet"))] then
			i = i + recap.Combatant[(UnitName("pet"))].LastDPS
		end
		RecapMinYourDPS_Text:SetText(string.format("%.1f",i))
	elseif recap.Combatant[recap_temp.Player] and recap.Opt.View.value=="All" then
		i = recap.Combatant[recap_temp.Player].TotalDPS
		if (UnitName("pet")) and (UnitName("pet"))~=UNKNOWNOBJECT and recap.Combatant[(UnitName("pet"))] then
			i = i + recap.Combatant[(UnitName("pet"))].TotalDPS
		end
		RecapMinYourDPS_Text:SetText(string.format("%.1f",i))
	else
		RecapMinYourDPS_Text:SetText("--")
	end

	-- calculate percentages
	for i=1,recap_temp.ListSize-1 do
		recap_temp.List[i].HealP = 0
		recap_temp.List[i].DmgInP = 0
		recap_temp.List[i].DmgOutP = 0
		-- Hawksy: guard code
		thisCombatant = recap_temp.List[i].Name
		if thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
			if recap_temp.List[recap_temp.ListSize].Heal>0 then
				recap_temp.List[i].HealP = math.floor(.5+100*recap_temp.List[i].Heal/recap_temp.List[recap_temp.ListSize].Heal)
			end
			if recap_temp.List[recap_temp.ListSize].DmgIn>0 then
				recap_temp.List[i].DmgInP = math.floor(.5+100*recap_temp.List[i].DmgIn/recap_temp.List[recap_temp.ListSize].DmgIn)
			end
			if recap_temp.List[recap_temp.ListSize].DmgOut>0 then
				recap_temp.List[i].DmgOutP = math.floor(.5+100*recap_temp.List[i].DmgOut/recap_temp.List[recap_temp.ListSize].DmgOut)
			end
		end
	end

	-- restore a valid value for Selected if possible
	if tempname then -- follow name to pre-sort selected
		recap_temp.Selected = Recap_GetSelectedByName(tempname)
		if recap_temp.Selected~=0 then
			RecapPanel_Show(tempname)
		else
			RecapPanel_Hide()
		end
	end

	Recap_SortList()

	RecapScrollBar_Update()
	Recap_UpdatePlugins()

end

function Recap_DefineGauges()

	local i, thisCombatant

	recap_temp.GaugeMax = 0

	if recap_temp.ListSize>1 then
		recap_temp.GaugeBy = string.gsub(recap.Opt.SortBy.value,"P$","")
		recap_temp.GaugeBy = recap_temp.GaugeBy=="DPSvsAll" and "DmgOut" or recap_temp.GaugeBy
		if not gaugecolor[recap_temp.GaugeBy] then
			recap_temp.GaugeBy = false
		else
			for i=1,recap_temp.ListSize-1 do
				-- Hawksy: guard code
				thisCombatant = recap_temp.List[i].Name
				if thisCombatant and recap.Combatant[thisCombatant] and recap_temp.GaugeBy then
					recap_temp.GaugeMax = math.max(recap.Combatant[thisCombatant].Friend and recap_temp.List[i][recap_temp.GaugeBy] or 0,recap_temp.GaugeMax)
				end
			end
			for i=1,15 do
				getglobal("RecapList"..i.."_Gauge"):SetVertexColor(gaugecolor[recap_temp.GaugeBy].r,gaugecolor[recap_temp.GaugeBy].g,gaugecolor[recap_temp.GaugeBy].b)
			end
		end
	end
end


function Recap_GetSelectedByName(arg1)

	local sel, i = 0
	local thisCombatant

	if arg1 and recap_temp.ListSize>1 then
		for i=1,recap_temp.ListSize-1 do
			-- Hawksy: guard code
			thisCombatant = recap_temp.List[i].Name
			if thisCombatant and (thisCombatant==arg1) then
				sel = i
			end
		end
	end

	return sel
end


--[[ Sorting functions ]]

-- initial sort by field done with table.sort for speed, then
-- friends shifted to top with an insertion sort for "stable" list
function Recap_SortList()

	local temp, tempname

	if recap_temp.Selected~=0 then
		tempname = recap_temp.List[recap_temp.Selected].Name
	end

	-- remember the last ('Total') element
	temp = recap_temp.List[recap_temp.ListSize]

	-- Hawksy: remove the last element from the 'List' table in case it could
	--   confuse 'table.sort'.
	table.remove(recap_temp.List) -- without second parameter this defaults to last element

	if recap.Opt.SortBy.value then
		if recap.Opt.SortDir.value then
			table.sort(recap_temp.List,Recap_SortDown)
		else
			table.sort(recap_temp.List,Recap_SortUp)
		end
		Recap_SortFriends()
	end

	-- restore the last ('Total') element
	recap_temp.List[recap_temp.ListSize] = temp

	Recap_DefineGauges()
	recap_temp.Selected = Recap_GetSelectedByName(tempname)

end

function Recap_SortDown(e1,e2)

	-- Hawksy: guard code
	if e1[recap.Opt.SortBy.value] and e2[recap.Opt.SortBy.value] then
		-- bypass a problem with mixed strings and numbers (originally, people running too advanced a ParserLib)
		if type(e1[recap.Opt.SortBy.value]) == type(e2[recap.Opt.SortBy.value]) then
			-- types match, normal compare
			if ( e1[recap.Opt.SortBy.value] < e2[recap.Opt.SortBy.value] ) then
				return true
			else
				return false
			end
		else
			-- types don't match, force string compare
			-- this may be due to a damaged saved variables file, but bypass it anyway
			if ( tostring(e1[recap.Opt.SortBy.value]) < tostring(e2[recap.Opt.SortBy.value]) ) then
				return true
			else
				return false
			end
		end
	else
		return false
	end

end

function Recap_SortUp(e1,e2)

	-- Hawksy: guard code
	if e1[recap.Opt.SortBy.value] and e2[recap.Opt.SortBy.value] then
		-- bypass a problem with mixed strings and numbers (originally, people running too advanced a ParserLib)
		if type(e1[recap.Opt.SortBy.value]) == type(e2[recap.Opt.SortBy.value]) then
			-- types match, normal compare
			if ( e1[recap.Opt.SortBy.value] > e2[recap.Opt.SortBy.value] ) then
				return true
			else
				return false
			end
		else
			-- types don't match, force string compare
			-- this may be due to a damaged saved variables file, but bypass it anyway
			if ( tostring(e1[recap.Opt.SortBy.value]) > tostring(e2[recap.Opt.SortBy.value]) ) then
				return true
			else
				return false
			end
		end
	else
		return false
	end

end

-- perform stable insertion sort on the list
function Recap_SortFriends()

	local i, j, temp, other
	local changed = true

	if recap_temp.ListSize>2 then
		for i=2,(recap_temp.ListSize-1) do
			temp = recap_temp.List[i]
			-- Hawksy: guard code
			if (temp) and (temp.Name) and (recap.Combatant[temp.Name]) and (recap.Combatant[temp.Name].Friend) then
				j=i
				while (j>1) do
					other = recap_temp.List[j-1]
					-- Hawksy: guard code
					if (other) and (other.Name) and (recap.Combatant[other.Name]) and (not recap.Combatant[other.Name].Friend) then
						recap_temp.List[j] = other
						j = j - 1
					else
						break
					end
				end
				recap_temp.List[j] = temp
			end
		end
	end

end

-- Hawksy: three new functions allowing the Personal Details panel to be sorted
--   by column (damage effects always appear first, healing effects second) (function
--   Recap_SelfDamageSort, which was a fixed sort, has been removed)
function Recap_SortSelfList()

	if recap.Opt.ESortBy.value then
		if recap.Opt.ESortDir.value then
			table.sort(recap_temp.SelfList,Recap_SortSelfDown)
		else
			table.sort(recap_temp.SelfList,Recap_SortSelfUp)
		end
	end

end

function Recap_SortSelfDown(e1,e2)

	local effect1, effect2, value1, value2

	if e1.EName and e2.EName then
		effect1 = string.sub(e1.EName,1,1)
		effect2 = string.sub(e2.EName,1,1)
		if e1[recap.Opt.ESortBy.value] and e2[recap.Opt.ESortBy.value] then
			value1 = e1[recap.Opt.ESortBy.value]
			if value1 == "--" then
				value1 = -1
			end
			-- turn a percentage string into a number
			if (type(value1) == "string") and (string.sub(value1, -1) == "%") then
				value1 = tonumber(string.sub(value1, 1, string.len(value1)-1))
			end
			value2 = e2[recap.Opt.ESortBy.value]
			if value2 == "--" then
				value2 = -1
			end
			-- turn a percentage string into a number
			if (type(value2) == "string") and (string.sub(value2, -1) == "%") then
				value2 = tonumber(string.sub(value2, 1, string.len(value2)-1))
			end
			if (((value1 < value2) and (effect1==effect2)) or (effect1<effect2)) then
				return true
			else
				return false
			end
		else
			return false
		end
	else
		return false
	end

end

function Recap_SortSelfUp(e1,e2)

	local effect1, effect2, value1, value2

	if e1.EName and e2.EName then
		effect1 = string.sub(e1.EName,1,1)
		effect2 = string.sub(e2.EName,1,1)
		if e1[recap.Opt.ESortBy.value] and e2[recap.Opt.ESortBy.value] then
			value1 = e1[recap.Opt.ESortBy.value]
			if value1 == "--" then
				value1 = -1
			end
			-- turn a percentage string into a number
			if (type(value1) == "string") and (string.sub(value1, -1) == "%") then
				value1 = tonumber(string.sub(value1, 1, string.len(value1)-1))
			end
			value2 = e2[recap.Opt.ESortBy.value]
			if value2 == "--" then
				value2 = -1
			end
			-- turn a percentage string into a number
			if (type(value2) == "string") and (string.sub(value2, -1) == "%") then
				value2 = tonumber(string.sub(value2, 1, string.len(value2)-1))
			end
			if (((value1 > value2) and (effect1==effect2)) or (effect1<effect2)) then
				return true
			else
				return false
			end
		else
			return false
		end
	else
		return false
	end

end

--[[ Window movement functions ]]

function Recap_OnMouseDown(arg1)

	if recap_temp.Loaded and arg1=="LeftButton" and not recap.Opt.Pinned.value then
		RecapFrame:StartMoving()
	end
end

function Recap_OnMouseUp(arg1)

	if recap_temp.Loaded and arg1=="LeftButton" and not recap.Opt.Pinned.value then
		RecapFrame:StopMovingOrSizing()
		Recap_CheckWindowBounds()

	elseif arg1=="RightButton" and recap.Opt.Minimized.value then
		RecapCreateMenu(recap_temp.Localize.MinMenu, nil)
	end

end

-- repositions RecapFrame if any part of it goes off the screen
function Recap_CheckWindowBounds()

	local x1, y1, x2, y2, cx, cy
	local needs_moved = false

	if recap_temp.Loaded and RecapFrame then

		x1 = RecapFrame:GetLeft()
		y1 = RecapFrame:GetTop()
		cx = RecapFrame:GetWidth()
		cy = RecapFrame:GetHeight()

		if x1 and y1 and cx and cy then

			x2 = RecapFrame:GetRight()
			y2 = RecapFrame:GetBottom()
			uix2 = UIParent:GetRight()
			uix1 = UIParent:GetLeft()
			uiy1 = UIParent:GetTop()
			uiy2 = UIParent:GetBottom()

			if x2 and uix2 and x2 > uix2 then
				x1 = x1 + uix2 - x2
				needs_moved = true
			elseif uix1 and uix1 > x1 then
				x1 = x1 + uix1 - x1
				needs_moved = true
			end

			if y1 and y1 > uiy1 then
				y1 = y1 + uiy1 - y1
				needs_moved = true
			elseif uiy2 and uiy2 > y2 then
				y1 = y1 + uiy2 - y2
				needs_moved = true
			end

			if needs_moved then
				RecapFrame:ClearAllPoints()
				RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",x1,y1)
			end

			recap[recap_temp.p].WindowTop = RecapFrame:GetTop()
			recap[recap_temp.p].WindowLeft = RecapFrame:GetLeft()
			recap[recap_temp.p].WindowHeight = RecapFrame:GetHeight()
			recap[recap_temp.p].WindowWidth = RecapFrame:GetWidth()

		else
			-- default behaviour if any undefined
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("CENTER","UIParent","CENTER")
		end
	end
end

--[[ Dialog control functions ]]

function Recap_OnClick(arg1) -- clicks on buttons

	local i

	if not recap_temp.Loaded then
		return
	end

	local fight_count = 0
	local total_fight_count = 0
	local filen = RecapSetEditBox:GetText()
	local set_friend, set_dmgin, set_dmgout, set_maxhit, set_kills, set_heal, set_time

	PlaySound("GAMEGENERICBUTTONPRESS")

	if arg1=="Close" then
		RecapFrame_Hide()

	elseif arg1=="Minimize" then

		if recap.Opt.Minimized.value then
			Recap_Maximize()
		else
			Recap_Minimize()
		end
		RecapFrame:SetAlpha(1)
		recap_temp.FadeTimer = -1
		Recap_SetButtons()
		Recap_OnTooltip("Minimize")

	elseif arg1=="Pin" then
		recap.Opt.Pinned.value = not recap.Opt.Pinned.value
		Recap_SetButtons()
		Recap_OnTooltip("Pin")

	elseif arg1=="Pause" then
		if recap.Opt.Paused.value then
			recap.Opt.Paused.value = false
			Recap_SetState("Idle")
		else
			recap.Opt.Paused.value = true
			recap_temp.HoldFight = "Normal"
			Recap_EndFight(true)
			Recap_SetState("Stopped")
		end
		Recap_SetButtons()
		Recap_OnTooltip("Pause")

	elseif arg1=="ShowAll" then
		if recap.Opt.SelfView.value then
			recap.Opt.SelfView.value = false
		end
		if recap.Opt.View.value=="Last" then
			Recap_SetView("All")
		else
			Recap_SetView("Last")
		end
		Recap_SetColumns()
		Recap_OnTooltip("ShowAll")

	elseif arg1=="Options" then
		if RecapOptFrame:IsShown() then
			RecapOptFrame:Hide()
		else
			RecapOptFrame:Show()
			Recap_SetSyncDisplay()
		end

	elseif arg1=="Reset" then

		if recap.Opt.SelfView.value then
			Recap_ResetSelfPlayerAndPets()
			Recap_SetView()

		elseif recap.Opt.View.value=="All" then
			if ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
				-- ignore if in synchronization
				PlaySound("igQuestFailed")
				Recap_WarnNotAllowedDuringSync()
			else
				-- wipe everything --
				RecapPanel_Hide(1)
				Recap_ResetAllCombatants(false)
				recap_temp.HoldFight = "Normal"
				Recap_MakeFriends()
				Recap_SetView()
			end

		elseif recap.Opt.View.value=="Last" then
			if ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
				-- ignore if in synchronization
				PlaySound("igQuestFailed")
				Recap_WarnNotAllowedDuringSync()
			else
				-- Hawksy: add a confirm dialog
				StaticPopupDialogs["recap_temp.CONFIRMLASTRESET"] = {
					text = TEXT(recap_temp.Localize.ConfirmLastReset),
					button1 = TEXT(ACCEPT),
					button2 = TEXT(CANCEL),
					OnAccept = function()
						-- Hawksy: silent repair
						if not recap.Combatant then
							recap.Combatant = {}
						end
						-- SUBTRACT the last fight --
						for i in pairs(recap.Combatant) do
							if recap.Combatant[i].WasInLast then
								Recap_ResetLastFight(i)
							end
						end
						recap_temp.HoldFight = "Normal"
						recap[recap_temp.p].TotalDuration = recap[recap_temp.p].TotalDuration - recap[recap_temp.p].LastDuration
						recap[recap_temp.p].LastDuration = 0
						recap[recap_temp.p].TotalDurationIn = recap[recap_temp.p].TotalDurationIn - recap[recap_temp.p].LastDurationIn
						recap[recap_temp.p].LastDurationIn = 0
						recap[recap_temp.p].TotalDurationHeal = recap[recap_temp.p].TotalDurationHeal - recap[recap_temp.p].LastDurationHeal
						recap[recap_temp.p].LastDurationHeal = 0
						Recap_SetView()
					end,
					OnHide = function()
					end,
					OnCancel = function()
					end,
					timeout = 0,
					showAlert = 1,
					whileDead = 1
				}
				StaticPopup_Show("recap_temp.CONFIRMLASTRESET")
			end
		end

	elseif arg1=="Hold" then
		if recap.Opt.View.value=="Last" then
			if recap_temp.HoldFight=="Normal" then
				recap_temp.HoldFight = "HoldNext"
			elseif recap_temp.HoldFight=="Holding" then
				recap_temp.HoldFight = "Normal"
				-- Update the panel
				Recap_EndFight(true)
				Recap_CheckJoinSync()
				Recap_CheckSendSummary()
				Recap_CheckProcessSyncData()
			end
			Recap_SetView()
		end

	elseif arg1=="SaveAllSet" then
		RecapSetEditBox:ClearFocus()
		if filen and filen~="" then
			filen = string.gsub(filen,"\"","'") -- convert "s to 's
			filen = string.gsub(filen,"%[","(") -- convert [s to (s
			filen = string.gsub(filen,"%]",")") -- convert ]s to )s
		end
		RecapSetEditBox:SetText("")
		Recap_SaveCombatants(filen, false)
		Recap_BuildFightSets()
		recap_temp.FightSetSelected = 0
		RecapFightSetsScrollBar_Update()

	-- Hawksy: added function to save only the Last Fight data set
	elseif arg1=="SaveLastSet" then
		RecapSetEditBox:ClearFocus()
		if filen and filen~="" then
			filen = string.gsub(filen,"\"","'") -- convert "s to 's
			filen = string.gsub(filen,"%[","(") -- convert [s to (s
			filen = string.gsub(filen,"%]",")") -- convert ]s to )s
		end
		RecapSetEditBox:SetText("")
		Recap_SaveCombatants(filen, true)
		Recap_BuildFightSets()
		recap_temp.FightSetSelected = 0
		RecapFightSetsScrollBar_Update()

	elseif arg1=="LoadSet" then
		if ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
			-- ignore if in synchronization
			PlaySound("igQuestFailed")
			Recap_WarnNotAllowedDuringSync()
		else
			RecapSetEditBox:SetText("")
			RecapSetEditBox:ClearFocus()
			Recap_LoadCombatants(filen)
			recap_temp.HoldFight = "Normal"
			Recap_MakeFriends()
			Recap_SetView("All")
		end

	elseif arg1=="DeleteSet" then
		RecapSetEditBox:SetText("")
		RecapSetEditBox:ClearFocus()
		recap_set[filen] = nil -- keep as nil
		Recap_BuildFightSets()

	elseif arg1=="SelfView" then
		recap.Opt.SelfView.value = not recap.Opt.SelfView.value
		if recap.Opt.SelfView.value then
			RecapHoldButton:Hide()
		else
			if recap.Opt.View.value=="Last" then
				Recap_SetView("Last")
			else
				Recap_SetView("All")
			end
		end
		RecapPanel_Hide(1)
		Recap_SetColumns()
		Recap_OnTooltip("SelfView")
	end
end

function RecapHeader_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local msg=string.gsub(this:GetName(),"RecapHeader_","Header")
	Recap_OnTooltip(msg)

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapHeader_OnMouseUp(arg1)

	if arg1=="RightButton" then
		RecapCreateMenu(recap_temp.Localize.ColumnMenu, nil)
	end
end

-- sort column, toggle dir for new headers, name=ascending, rest=descending
function RecapHeader_OnClick()

	local text = ""
	local arg1 = string.gsub(this:GetName(),"RecapHeader_","")

	if not IsShiftKeyDown() then
		if recap.Opt.SortBy.value==string.gsub(arg1,"P$","") then
			recap.Opt.SortDir.value = not recap.Opt.SortDir.value
		else
			recap.Opt.SortBy.value = string.gsub(arg1,"P$","")
			if arg1=="Name" then
				recap.Opt.SortDir.value = true
			else
				recap.Opt.SortDir.value = false
			end
		end
		Recap_SortList()
		RecapScrollBar_Update()

	elseif IsShiftKeyDown() and recap_temp.ListSize>1 and arg1~="Name" then

		recap.Opt.SortBy.value = arg1
		recap.Opt.SortDir.value = false
		Recap_SortList()
		RecapScrollBar_Update()

		if recap.Opt.SpamRows.value then
			Recap_PostSpamRows(arg1, nil)
		else
			Recap_InsertChat(Recap_PostSpamLine(arg1))
		end

	elseif IsShiftKeyDown() and not ChatFrameEditBox:IsVisible() then
		-- Hawksy: will now happen seldom
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.RankUsage)
	end

end

function Recap_List_OnClick(arg1)

	local id, index, thisCombatant

	id = this:GetID()
	index = id + FauxScrollFrame_GetOffset(RecapScrollBar)

	if index<recap_temp.ListSize then
		recap_temp.OutgoingDetailSelected = 0
		recap_temp.IncomingDetailSelected = 0
		if recap_temp.Selected==index and arg1=="LeftButton" then
			recap_temp.Selected = 0
			getglobal("RecapList"..id):UnlockHighlight()
			RecapPanel_Hide()
		else
			-- Hawksy: guard code
			thisCombatant = recap_temp.List[index].Name
			if thisCombatant then
				recap_temp.Selected = index
				RecapScrollBar_Update()
				RecapPanel_Show(thisCombatant)
			else
				recap_temp.Selected = 0
				getglobal("RecapList"..id):UnlockHighlight()
				RecapPanel_Hide()
			end
		end

		if arg1=="RightButton" and IsShiftKeyDown() and not IsControlKeyDown() then
			-- Hawksy: modified to post Recent Incoming events when Recent Data Mode is active
			if recap.Opt.RecentData.value then
				Recap_PostRecentRows(false, recap_temp.List[index].Name, nil, nil)
			else
				Recap_InsertChat("Recap: "..recap_temp.List[index].Name.." "..getglobal("RecapList"..id.."_DPS"):GetText().." "..RECAP_DPS.." ")
			end

		elseif arg1=="RightButton" and IsControlKeyDown() and not IsShiftKeyDown() then
			-- Hawksy: modified to post Recent Outgoing events when Recent Data Mode is active
			if recap.Opt.RecentData.value then
				Recap_PostRecentRows(true, recap_temp.List[index].Name, nil, nil)
			else
				Recap_InsertChat("Recap: "..recap_temp.List[index].Name.." "..getglobal("RecapList"..id.."_DPS"):GetText().." "..RECAP_DPS.." ")
			end

		elseif arg1=="LeftButton" and IsShiftKeyDown() then
			Recap_InsertChat(string.format(recap_temp.Localize.VerboseLinkStart,
								recap_temp.List[index].Name,
								getglobal("RecapList"..id.."_DmgIn"):GetText(),
								getglobal("RecapList"..id.."_TimeIn"):GetText(),
								getglobal("RecapList"..id.."_DPSIn"):GetText(),
								getglobal("RecapList"..id.."_Kills"):GetText(),
								getglobal("RecapList"..id.."_DmgOut"):GetText(),
								getglobal("RecapList"..id.."_Time"):GetText(),
								getglobal("RecapList"..id.."_DPS"):GetText(),
								getglobal("RecapList"..id.."_MaxHit"):GetText(),
								getglobal("RecapList"..id.."_Heal"):GetText(),
								getglobal("RecapList"..id.."_TimeHeal"):GetText(),
								getglobal("RecapList"..id.."_HPS"):GetText() ).." "..recap_temp.Localize.For.." "..
								recap_temp.LastAll[recap.Opt.View.value])

		elseif arg1=="RightButton" and not IsShiftKeyDown() and not IsControlKeyDown() then
			-- Hawksy: guard code
			thisCombatant = recap_temp.List[index].Name
			if thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
				if recap.Opt.View.value == "Last" then
					RecapCreateMenu(recap_temp.Localize.DropMenu, 1)
				elseif recap.Combatant[thisCombatant].Locked then
					RecapCreateMenu(recap_temp.Localize.DropUnlockMenu, 1)
				else
					RecapCreateMenu(recap_temp.Localize.DropLockMenu, 1)
				end
			else
				if recap.Opt.View.value == "Last" then
					RecapCreateMenu(recap_temp.Localize.AddMenu, 1)
				elseif recap.Combatant[thisCombatant].Locked then
					RecapCreateMenu(recap_temp.Localize.AddUnlockMenu, 1)
				else
					RecapCreateMenu(recap_temp.Localize.AddLockMenu, 1)
				end
			end
		end
	end
end

-- Hawksy: added to allow click reporting off the totals line
function Recap_List_Totals_OnClick(arg1)

	local i, thisCombatant

	recap_temp.OutgoingDetailSelected = 0
	recap_temp.IncomingDetailSelected = 0
	recap_temp.Selected = 0
	for i=1,15 do
		getglobal("RecapList"..i):UnlockHighlight()
	end
	RecapPanel_Hide()

	-- Hawksy: control+shift+right-click on total line posts all recent combat event information
	-- to avoid impossible spam, this function only posts to "Self" (the player console)
	-- with a lot of events, it may be very slow
	-- this feature is deliberately undocumented
	if arg1=="RightButton" and IsShiftKeyDown() and IsControlKeyDown() then
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(true, nil, nil, true)
		end

	elseif arg1=="LeftButton" and IsShiftKeyDown() then
		Recap_InsertChat(string.format(recap_temp.Localize.VerboseFriendlyTotalStart,
							getglobal("RecapTotals_DmgIn"):GetText(),
							getglobal("RecapTotals_TimeIn"):GetText(),
							getglobal("RecapTotals_DPSIn"):GetText(),
							getglobal("RecapTotals_Kills"):GetText(),
							getglobal("RecapTotals_DmgOut"):GetText(),
							getglobal("RecapTotals_Time"):GetText(),
							getglobal("RecapTotals_DPS"):GetText(),
							getglobal("RecapTotals_MaxHit"):GetText(),
							getglobal("RecapTotals_Heal"):GetText(),
							getglobal("RecapTotals_TimeHeal"):GetText(),
							getglobal("RecapTotals_HPS"):GetText() ).." "..recap_temp.Localize.For.." "..
							recap_temp.LastAll[recap.Opt.View.value])
		-- create totals for the other (non-friendly) combatants
		local dmgin, kills, dmgout, heal, maxhit, duration, durationIn, DurationHeal
		dmgin = 0
		kills = 0
		dmgout = 0
		heal = 0
		maxhit = 0
		if recap.Opt.View.value=="Last" then
			duration = recap[recap_temp.p].LastDuration
			durationIn = recap[recap_temp.p].LastDurationIn
			durationHeal = recap[recap_temp.p].LastDurationHeal
		else
			duration = recap[recap_temp.p].TotalDuration
			durationIn = recap[recap_temp.p].TotalDurationIn
			durationHeal = recap[recap_temp.p].TotalDurationHeal
		end
		if duration>0 or durationIn>0 or durationHeal>0 then
			for j=1,recap_temp.ListSize-1 do
				thisCombatant = recap_temp.List[j].Name
				if thisCombatant and recap.Combatant[thisCombatant] and not recap.Combatant[thisCombatant].Friend then
					if recap_temp.List[j].MaxHit > maxhit then
						maxhit = recap_temp.List[j].MaxHit
					end
					dmgin = dmgin + recap_temp.List[j].DmgIn
					kills = kills + recap_temp.List[j].Kills
					dmgout = dmgout + recap_temp.List[j].DmgOut
					heal = heal + recap_temp.List[j].Heal
				end
			end
			Recap_InsertChat(string.format(recap_temp.Localize.VerboseNonFriendlyTotalStart,
								dmgin,
								getglobal("RecapTotals_TimeIn"):GetText(),
								(((durationIn>0) and (dmgin/durationIn)) or "--"),
								kills,
								dmgout,
								getglobal("RecapTotals_Time"):GetText(),
								(((duration>0) and (dmgout/duration)) or "--"),
								maxhit,
								heal,
								getglobal("RecapTotals_TimeHeal"):GetText(),
								(((durationHeal>0) and (heal/durationHeal)) or "--")).." "..recap_temp.Localize.For.." "..
								recap_temp.LastAll[recap.Opt.View.value])
		end
	end
end


--[[ Tooltip functions ]]

function Recap_Tooltip(arg1,arg2)

	if recap_temp.Loaded and recap.Opt.ShowTooltips.value then
		if not recap.Opt.TooltipFollow.value then
			GameTooltip_SetDefaultAnchor(GameTooltip,this)
		else
			GameTooltip:SetOwner(this,Recap_TooltipAnchor())
		end
		GameTooltip:SetText(arg1)
		GameTooltip:AddLine(arg2, .75, .75, .75, 1)
		GameTooltip:Show()
	end
end

-- returns line1,line2 of a tooltip for arg1 in OptList (for generic static tooltips)
function Recap_GetTooltip(arg1)

	local i

	if arg1 then
		for i in pairs(recap_temp.OptList) do
			if recap_temp.OptList[i][1]==arg1 then
				return recap_temp.OptList[i][2],recap_temp.OptList[i][3]
			end
		end
	end
end

-- arg1=tooltip index, arg2=optional addition to header
function Recap_OnTooltip(arg1,arg2)

	local line1, line2

	if arg1=="Status" then
		Recap_Tooltip(recap_temp.Recap.." "..recap_temp.Localize.Status..": "..recap_temp.StoppedActiveIdle[recap.Opt.State.value],recap_temp.Localize.StatusTooltip)
	elseif arg1=="Close" then
		if recap.Opt.State.value=="Stopped" then
			Recap_OnTooltip("ExitRecap")
		else
			Recap_OnTooltip("HideWindow")
		end
	elseif arg1=="Minimize" then
		if recap.Opt.Minimized.value then
			Recap_OnTooltip("ExpandWindow")
		else
			Recap_OnTooltip("MinimizeWindow")
		end
	elseif arg1=="Pin" then
		if recap.Opt.Pinned.value then
			Recap_OnTooltip("UnPinWindow")
		else
			Recap_OnTooltip("PinWindow")
		end
	elseif arg1=="Pause" then
		if recap.Opt.State.value=="Stopped" then
			Recap_OnTooltip("Resume")
		else
			Recap_OnTooltip("PauseMonitoring")
		end
	elseif arg1=="ShowAll" then
		if recap.Opt.View.value=="Last" then
			Recap_OnTooltip("ShowAllFights")
		else
			Recap_OnTooltip("ShowLastFight")
		end
	elseif arg1=="HeaderName" then
		if recap.Opt.View.value=="Last" then
			Recap_OnTooltip("CombatLast")
		else
			Recap_OnTooltip("CombatAll")
		end
	elseif arg1=="Reset" then
		if recap.Opt.SelfView.value then
			Recap_OnTooltip("ResetSelfView")
		elseif recap.Opt.View.value=="Last" then
			Recap_OnTooltip("ResetLastFight")
		else
			Recap_OnTooltip("ResetAllTotals")
		end

	elseif arg1=="Hold" then
		if recap.Opt.View.value=="Last" then
			if recap_temp.HoldFight=="Normal" then
				Recap_OnTooltip("HoldFight")
			elseif recap_temp.HoldFight=="Holding" then
				Recap_OnTooltip("ResumeUpdate")
			end
		end

	elseif arg1=="SelfView" then
		if recap.Opt.SelfView.value then
			Recap_OnTooltip("HideSelfView")
		else
			Recap_OnTooltip("ShowSelfView")
		end
	else
		line1,line2 = Recap_GetTooltip(arg1)
		if line1 and line2 then
			if arg2 then
				Recap_Tooltip(line1..": "..arg2,line2)
			else
				Recap_Tooltip(line1, line2)
			end
		end
	end
end

function Recap_List_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local index, id, class, faction, secondLine

	id = this:GetID()
	index = id + FauxScrollFrame_GetOffset(RecapScrollBar)

	thisCombatant = recap_temp.List[index].Name
	if thisCombatant then
		class = recap_temp.Localize.ClassName[recap.Combatant[thisCombatant].Class]
		faction = recap_temp.Localize.FactionName[recap.Combatant[thisCombatant].Faction]
		secondLine = ""
		if class then
			secondLine = class
		end
		if faction then
			secondLine = secondLine.." ("..faction..")"
		end
		if recap.Combatant[thisCombatant].OwnedBy then
			secondLine = secondLine.." ("..recap_temp.Localize.OwnedBy.." "..recap.Combatant[thisCombatant].OwnedBy..")"
		end
		Recap_Tooltip(thisCombatant, secondLine)
	end
	if (index<recap_temp.ListSize) and recap_temp.Selected==0 then
		if thisCombatant then
			RecapPanel_Show(thisCombatant)
		end
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function Recap_Totals_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local i, thisCombatant
	local r = recap_temp.ColorWhite.r
	local g = recap_temp.ColorWhite.g
	local b = recap_temp.ColorWhite.b
	local other_kills = 0

	if recap.Opt.ShowTooltips.value then

		if recap.Opt.TooltipFollow.value then
			GameTooltip:SetOwner(this,Recap_TooltipAnchor())
		else
			GameTooltip_SetDefaultAnchor(GameTooltip,this)
		end

		GameTooltip:AddLine(recap_temp.Localize.Totals.." "..recap_temp.Localize.For.." "..recap_temp.LastAll[recap.Opt.View.value])

		if recap_temp.ListSize>1 then

			for i=1,(recap_temp.ListSize-1) do
				-- Hawksy: guard code
				thisCombatant = recap_temp.List[i].Name
				if thisCombatant and recap.Combatant[thisCombatant] and (not recap.Combatant[thisCombatant].Friend) then
					other_kills = other_kills + recap_temp.List[i].Kills
				end
			end
			GameTooltip:AddDoubleLine(RECAP_COMBATANTS..":",recap_temp.ListSize-1,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine(RECAP_DAMAGE_OUT..":",recap_temp.List[recap_temp.ListSize].DmgOut,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			GameTooltip:AddDoubleLine(RECAP_TIME_FIGHTING..":",Recap_FormatTime(recap_temp.List[recap_temp.ListSize].Time),recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			GameTooltip:AddDoubleLine(RECAP_DPS_OUT..":",string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotDPSOut),recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			GameTooltip:AddDoubleLine(RECAP_MAX_HIT..":",recap_temp.List[recap_temp.ListSize].MaxHit,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine(RECAP_KILLS..":",other_kills,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine(RECAP_DAMAGE_IN..":",recap_temp.List[recap_temp.ListSize].DmgIn,recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b,recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			GameTooltip:AddDoubleLine(RECAP_TIME_DAMAGED..":",Recap_FormatTime(recap_temp.List[recap_temp.ListSize].TimeIn),recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b,recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			GameTooltip:AddDoubleLine(RECAP_DPS_IN..":",string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotDPSIn),recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b,recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			GameTooltip:AddDoubleLine(RECAP_DEATHS..":",recap_temp.List[recap_temp.ListSize].Kills,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine(RECAP_HEALS..":",recap_temp.List[recap_temp.ListSize].Heal,recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b,recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			GameTooltip:AddDoubleLine(RECAP_TIME_HEALING..":",Recap_FormatTime(recap_temp.List[recap_temp.ListSize].TimeHeal),recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b,recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			GameTooltip:AddDoubleLine(RECAP_HPS_OUT..":",string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotHPS),recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b,recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
		else
			GameTooltip:AddLine(RECAP_COMBATANTS..": "..recap_temp.Localize.None,r,g,b)
		end

		GameTooltip:Show()
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function Recap_TooltipAnchor()

	local anchor = "ANCHOR_LEFT"

	if GetCursorPosition("UIParent")<(UIParent:GetWidth()/2) then
		anchor = "ANCHOR_RIGHT"
	end
	return anchor
end

--[[ Plug-in functions ]]

function Recap_GetIB_Tooltip()
	return recap_temp.LastAll[recap.Opt.View.value]..":\n"..RECAP_YOUR_DPS..": "..RecapMinYourDPS_Text:GetText().."\n"..RECAP_MAX_HIT..": "..RecapTotals_MaxHit:GetText().."\n"..RECAP_TOTAL_DPS_OUT..": "..RecapTotals_DPS:GetText().."\n"..RECAP_TOTAL_DPS_IN..": "..RecapTotals_TotDPSIn:GetText()
end

function Recap_UpdatePlugins()

	local yourdps = RecapMinYourDPS_Text:GetText()
	local dpsin = RecapMinDPSIn_Text:GetText()
	local dpsout = RecapMinDPSOut_Text:GetText()

	if IB_Recap_Update then
		IB_Recap_Update("DPS: "..yourdps)
	end

	if TitanPanelRecap_Update then
		TitanPanelRecap_Update(recap.Opt.State.value,yourdps,dpsin,dpsout)
	end
end

--[[ Context menu functions ]]

function RecapMenu_OnUpdate(arg1)

	local i

	if not recap_temp.MenuTimer or MouseIsOver(RecapMenu) then
		recap_temp.MenuTimer = 0
	end

	recap_temp.MenuTimer = recap_temp.MenuTimer + arg1
	-- Hawksy: doubled the time that a menu hangs around
	if recap_temp.MenuTimer > 0.5 then
		recap_temp.MenuTimer = 0
		RecapMenu:Hide()
		if recap_temp.SelectedEffect then
			for i=1,15 do
				getglobal("RecapSelf"..i):UnlockHighlight()
			end
		end
	end
end

function RecapMenu_SetCheck(id,arg1)

	if arg1 then
		getglobal("RecapMenu"..id.."_Check"):Show()
	else
		getglobal("RecapMenu"..id.."_Check"):Hide()
	end
end

function RecapMenu_OnClick()

	local i
	local id, index = this:GetID()

	if recap_temp.Menu==recap_temp.Localize.ColumnMenu or recap_temp.Menu==recap_temp.Localize.MinMenu or recap_temp.Menu==recap_temp.Localize.EffectOptMenu then
		index = recap_temp.Menu[id].Info -- .Info of menu item selected
		if recap.Opt[index] then
			recap.Opt[index].value = not recap.Opt[index].value
			getglobal("RecapOptCheck_"..recap_temp.Menu[id].Info):SetChecked(0)
			if recap.Opt[index].value then
				getglobal("RecapOptCheck_"..recap_temp.Menu[id].Info):SetChecked(1)
			end
			RecapMenu_SetCheck(id,recap.Opt[index].value)
			if recap.Opt.Minimized.value then
				Recap_Minimize()
			else
				Recap_Maximize()
			end
		elseif index=="Pin" then
			recap.Opt.Pinned.value = not recap.Opt.Pinned.value
			Recap_SetButtons()
			RecapMenu:Hide()
		end
	end

	-- popup menu
	if recap_temp.Menu==recap_temp.Localize.AddMenu or recap_temp.Menu==recap_temp.Localize.AddLockMenu or recap_temp.Menu==recap_temp.Localize.AddUnlockMenu or
	   recap_temp.Menu==recap_temp.Localize.DropMenu or recap_temp.Menu==recap_temp.Localize.DropLockMenu or recap_temp.Menu==recap_temp.Localize.DropUnlockMenu then
		RecapMenu:Hide()
		index = recap_temp.List[recap_temp.Selected].Name -- name selected

		if id==1 and ((recap_temp.Menu==recap_temp.Localize.DropMenu) or (recap_temp.Menu==recap_temp.Localize.DropLockMenu) or (recap_temp.Menu==recap_temp.Localize.DropUnlockMenu)) and index~=recap_temp.Player then
			-- Make them no longer a 'friendly combatant'
			recap.Combatant[index].Friend = false

		elseif id==1 and index~=recap_temp.Player then
			-- Toggle their 'friendly combatant' status
			recap.Combatant[index].Friend = not recap.Combatant[index].Friend

		elseif id==2 and recap.Opt.View.value=="All" then -- reset
			if ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
				-- ignore if in synchronization
				PlaySound("igQuestFailed")
				Recap_WarnNotAllowedDuringSync()
			else
				-- 'Reset' a combatant, meaning force all their numbers to zero
				recap.Combatant[index] = nil -- keep as nil
				Recap_CreateBlankCombatant(index)
			end

		elseif id==2 and recap.Opt.View.value=="Last" then
			if ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
				-- ignore if in synchronization
				PlaySound("igQuestFailed")
				Recap_WarnNotAllowedDuringSync()
			else
				-- Partially 'Reset' a combatant, meaning SUBTRACT their last fight information
				if recap.Combatant[index].WasInLast then
					-- Hawksy: add a confirm dialog
					StaticPopupDialogs["recap_temp.CONFIRMLASTRESET"] = {
						text = TEXT(recap_temp.Localize.ConfirmLastReset),
						button1 = TEXT(ACCEPT),
						button2 = TEXT(CANCEL),
						OnAccept = function()
							-- SUBTRACT the last fight --
							Recap_ResetLastFight(index)
							-- force redisplay (not sure if there might not be a cheaper call)
							Recap_Maximize()
						end,
						OnHide = function()
						end,
						OnCancel = function()
						end,
						timeout = 0,
						showAlert = 1,
						whileDead = 1
					}
					StaticPopup_Show("recap_temp.CONFIRMLASTRESET")
				end
			end

		elseif id==3 and index~=recap_temp.Player then
			if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
				-- don't ignore a combatant when in synchronization
				PlaySound("igQuestFailed")
				Recap_WarnNotAllowedDuringSync()
			else
				-- Put this combatant on 'ignore' (after forcing all their numbers to zero)
				recap.Combatant[index] = nil -- keep as nil
				Recap_CreateBlankCombatant(index)
				recap.Combatant[index].Ignore = true
			end

		elseif id==4 then
			-- Toggle their Locked status (makes them stick around through an All Fights Reset)
			recap.Combatant[index].Locked = not recap.Combatant[index].Locked
			-- Set the Locked status of any pets to match
			for i in pairs(recap.Combatant) do
				if recap.Combatant[i].OwnedBy == index then
					recap.Combatant[i].Locked = recap.Combatant[index].Locked
				end
			end
		end

		Recap_SetView()
	else
		-- remove this effect
		if recap_temp.Menu==recap_temp.Localize.EffectMenu then
			RecapMenu:Hide()
			for i=1,15 do
				getglobal("RecapSelf"..i):UnlockHighlight()
			end
			if recap_temp.SelectedEffect then
				-- could be for player, or could be for one of the player's pets
				if (string.sub(recap_temp.SelectedEffect,1,1)=="1") or (string.sub(recap_temp.SelectedEffect,1,1)=="3") then
					-- player
					if recap.Self[recap_temp.s][recap_temp.SelectedEffect] then
						recap.Self[recap_temp.s][recap_temp.SelectedEffect] = nil -- keep as nil
					end
				else
					-- player's pet
					local selfType, selfName, selfEffect, selfIndex
					_,_,selfType,selfName,selfEffect = string.find(recap_temp.SelectedEffect, "^(.)(.+): (.+)")
					selfIndex = recap_temp.s..":"..selfName
					if selfType=="2" then
						selfEffect = "1"..selfEffect
					else
						selfEffect = "3"..selfEffect
					end
					if recap.Self[selfIndex][selfEffect] then
						recap.Self[selfIndex][selfEffect] = nil -- keep as nil
					end
				end
				recap_temp.SelectedEffect = false
				Recap_SetView()
			end
		end

		-- other actions
		if recap_temp.Menu==recap_temp.Localize.DetailMenu then
			if recap_temp.OutgoingDetailsList and recap_temp.OutgoingDetailsList[1][recap_temp.Menu[id].Info] then
				recap.Opt.OutgoingPanelDetail.value = recap_temp.Menu[id].Info
				RecapPanel_Populate(recap_temp.List[recap_temp.Selected].Name)
			end
			RecapMenu:Hide()
		end
		if recap_temp.Menu==recap_temp.Localize.DetailMenu then
			if recap_temp.IncomingDetailsList and recap_temp.IncomingDetailsList[1][recap_temp.Menu[id].Info] then
				recap.Opt.IncomingPanelDetail.value = recap_temp.Menu[id].Info
				RecapPanel_Populate(recap_temp.List[recap_temp.Selected].Name)
			end
			RecapMenu:Hide()
		end
	end
end

-- offset=1 to move window so it's under pointer
function RecapCreateMenu(menu, offset)

	local i, cy, x, y, lines

	recap_temp.Menu = menu

	if recap_temp.Menu==recap_temp.Localize.ColumnMenu or recap_temp.Menu==recap_temp.Localize.MinMenu or recap_temp.Menu==recap_temp.Localize.EffectOptMenu then
		for i in pairs(recap_temp.Menu) do
			if recap.Opt[recap_temp.Menu[i].Info] then
				recap_temp.Menu[i].Check = recap.Opt[recap_temp.Menu[i].Info].value
			end
		end
	end

	if recap_temp.Menu==recap_temp.Localize.MinMenu then
		recap_temp.Menu[1].Check = recap.Opt.Pinned.value
	end

	if recap_temp.Menu==recap_temp.Localize.DetailMenu then
		for i in pairs(recap_temp.Menu) do
			recap_temp.Menu[i].Check = recap.Opt.OutgoingPanelDetail.value==recap_temp.Menu[i].Info
		end
	end
	if recap_temp.Menu==recap_temp.Localize.DetailMenu then
		for i in pairs(recap_temp.Menu) do
			recap_temp.Menu[i].Check = recap.Opt.IncomingPanelDetail.value==recap_temp.Menu[i].Info
		end
	end

	lines = min(#recap_temp.Menu,recap_temp.MaxMenuLines)

	cy = 44 -- 12+32
	for i=1,lines do
		getglobal("RecapMenu"..i.."_Text"):SetText(recap_temp.Menu[i].Text)
		RecapMenu_SetCheck(i,recap_temp.Menu[i].Check)
		getglobal("RecapMenu"..i):Show()
		cy = cy + 14
	end
	for i=(lines+1),recap_temp.MaxMenuLines do
		getglobal("RecapMenu"..i):Hide()
	end

	RecapMenu:SetHeight(cy)
	RecapDropSubFrame:SetHeight(cy-32)
	RecapMenu:ClearAllPoints()

	x,y = GetCursorPosition()
	if recap.Opt.GrowUpwards.value and not offset then
		y = this:GetBottom()*UIParent:GetScale()
		RecapMenu:SetPoint("BOTTOMRIGHT","UIParent","BOTTOMLEFT",(x/UIParent:GetScale())+20,(y/UIParent:GetScale())-4)
	else
		y = this:GetTop()*UIParent:GetScale()
		RecapMenu:SetPoint("TOPRIGHT","UIParent","BOTTOMLEFT",(x/UIParent:GetScale())+20,(y/UIParent:GetScale())+(offset and 16 or 4))
	end

	RecapMenu:Show()

end

function RecapMenu_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	id = this:GetID()
	if recap_temp.Menu[id] and recap_temp.Menu[id].Info then
		if recap_temp.Menu==recap_temp.Localize.AddLockMenu or recap_temp.Menu==recap_temp.Localize.AddUnlockMenu or recap_temp.Menu==recap_temp.Localize.DropLockMenu or recap_temp.Menu==recap_temp.Localize.DropUnlockMenu then
			Recap_OnTooltip(recap_temp.Menu[id].Info,recap_temp.List[recap_temp.Selected].Name)
		elseif recap_temp.Menu==recap_temp.Localize.EffectMenu and recap_temp.SelectedEffect then
			Recap_OnTooltip(recap_temp.Menu[id].Info,string.sub(recap_temp.SelectedEffect,2))
		else
			Recap_OnTooltip(recap_temp.Menu[id].Info)
		end
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapEffectsHeader_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local s

	_,_,s = string.find(this:GetName(),"RecapHeader_(.+)")
	if s then
		Recap_OnTooltip("Header"..s)
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapSelf_OnMouseUp(arg1)

	recap_temp.SelectedEffect = false
	Recap_OnMouseUp(arg1)
end

function RecapEffectsHeader_OnMouseUp(arg1)

	if arg1=="RightButton" then
		RecapCreateMenu(recap_temp.Localize.EffectOptMenu, nil)
	end
end

function RecapEffectsHeader_OnClick()

	local i, lineno
	local alltext, text = ""
	local hasdata
	local arg1 = string.gsub(this:GetName(),"RecapHeader_","")
	local print_chat = SendChatMessage
	local chatchan, chatnum

	-- Hawksy: new code to support sorting effects on a per column basis
	if not IsShiftKeyDown() then
		if recap.Opt.ESortBy.value==string.gsub(arg1,"$","") then
			recap.Opt.ESortDir.value = not recap.Opt.ESortDir.value
		else
			recap.Opt.ESortBy.value = string.gsub(arg1,"$","")
			if (arg1=="EName") or (arg1=="EElement") then
				recap.Opt.ESortDir.value = true
			else
				recap.Opt.ESortDir.value = false
			end
		end
		Recap_SortSelfList()
		RecapScrollBar_Update()

	elseif IsShiftKeyDown() and recap_temp.SelfListSize>1 then

		recap.Opt.ESortBy.value = arg1

		-- only report for columns other than EName
		if recap.Opt.ESortBy.value~="EName" then

			if (arg1=="EName") or (arg1=="EElement") then
				recap.Opt.ESortDir.value = true
			else
				recap.Opt.ESortDir.value = false
			end
			Recap_SortSelfList()
			RecapScrollBar_Update()

			chatchan, chatnum = Recap_GetChannelInfo(nil, nil)
			if chatchan=="Self" then
				print_chat = Recap_Print
			end

			-- If the Clip box on the Options/Reports panel is open, post there instead of to chat or console
			if RecapClipEditBox:IsVisible() then
				print_chat = Recap_LineToClipboard
			end

			alltext = (recap.Opt.SpamRows.value and "__ Recap: " or "Recap: ")..recap_temp.Player..": "..Recap_GetTooltip("Header"..arg1)..(recap.Opt.SpamRows.value and " __" or ":")
			lineno = 0
			for i=1,recap_temp.SelfListSize-1 do
				if recap_temp.SelfList[i][arg1]~="0.0%" and recap_temp.SelfList[i][arg1]~="--" and tostring(recap_temp.SelfList[i][arg1])~="0" then
					if not hasdata and recap.Opt.SpamRows.value then
						print_chat(alltext, chatchan, nil, chatnum)
					end
					hasdata = true
					text = string.sub(recap_temp.SelfList[i].EName,2)..(((string.sub(recap_temp.SelfList[i].EName,1,1)=="3") or (string.sub(recap_temp.SelfList[i].EName,1,1)=="4")) and "*" or "")..(recap.Opt.SpamRows.value and ": " or " ")..recap_temp.SelfList[i][arg1]..(arg1=="ETotal" and (" ("..recap_temp.SelfList[i].ETotalP..")") or "")
					if not recap.Opt.SpamRows.value then
						alltext = alltext.." ["..text.."]"
					else
						lineno = lineno + 1
						text = tostring(lineno)..". "..text
						print_chat(text, chatchan, nil, chatnum)
					end
				end
			end
			if not recap.Opt.SpamRows.value and hasdata then
				Recap_InsertChat(alltext)
			end
		end

	else
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.RankUsage)
	end
end


function RecapSelf_OnClick(arg1)

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapScrollBar)
	local linkformat = recap_temp.Localize.OutgoingDamageDetailLink
	local text = ""

	if arg1=="LeftButton" and IsShiftKeyDown() and index<recap_temp.SelfListSize then
		if (string.sub(recap_temp.SelfList[index].EName,1,1)=="3") or (string.sub(recap_temp.SelfList[index].EName,1,1)=="4") then
			linkformat = recap_temp.Localize.OutgoingHealDetailLink
		end
		text = string.format(linkformat,recap_temp.Player,string.sub(recap_temp.SelfList[index].EName,2),(recap_temp.SelfList[index].EElement or "?"),recap_temp.SelfList[index].ETotal,recap_temp.SelfList[index].ETotalP,recap_temp.Player )..", "..RECAP_MAX_HIT..": "..recap_temp.SelfList[index].EMaxAll..((recap_temp.SelfList[index].ECritsP~="--") and (", "..RECAP_CRIT_RATE..": "..recap_temp.SelfList[index].ECritsP) or "")
		Recap_InsertChat(text," ")

	elseif arg1=="RightButton" and index<recap_temp.SelfListSize and IsControlKeyDown() and not IsShiftKeyDown() then
		-- Hawksy: post Recent Outgoing events for this effect for this combatant when Recent Data Mode is active
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(true, recap_temp.Player, string.sub(recap_temp.SelfList[index].EName,2), nil)
		end

	elseif arg1=="RightButton" and index<recap_temp.SelfListSize and not IsShiftKeyDown() and not IsControlKeyDown() then
		this:LockHighlight()
		recap_temp.SelectedEffect = recap_temp.SelfList[index].EName
		RecapCreateMenu(recap_temp.Localize.EffectMenu, 1)
	end
end


Recap_lua_367 = true
