--[[ Fight functions ]]

function Recap_CreateCombatantAndUpdateTimers(thisCombatant, event_time, timeType)

	-- guard code
	if not thisCombatant then
		return
	end

	-- if thisCombatant not a .Combatant, add an entry for them
	if not recap.Combatant[thisCombatant] then
		Recap_CreateBlankCombatant(thisCombatant)
	end

	-- if they haven't fought this session, add a .Last entry for them
	if not recap_temp.Last[thisCombatant] then
		recap_temp.Last[thisCombatant] = { StartOut = 0, EndOut = 0, StartIn = 0, EndIn = 0, StartHeal = 0, EndHeal = 0, DmgIn = 0, DmgOut = 0, MaxHit = 0, Kills = 0, RawHeal = 0, OverHeal = 0 }
	end

	-- Hawksy: If we get here for any reason we might as well start the overall fight timers
	--   (Recap 3.32 would only do so for a source doing damage, or for a spellcast start)
	--   (Is healing after a fight, thus starting a new fight, any different from someone ganking a toad?)
	-- Hawksy: now three independent overall fight timers
	if timeType == "Out" then
		if recap_temp.FightStart==0 then
			recap_temp.FightStart = event_time
			recap_temp.FightEnd = recap_temp.FightStart
		else
			recap_temp.FightEnd = event_time
		end
	end
	if timeType == "In" then
		if recap_temp.FightStartIn==0 then
			recap_temp.FightStartIn = event_time
			recap_temp.FightEndIn = recap_temp.FightStartIn
		else
			recap_temp.FightEndIn = event_time
		end
	end
	if timeType == "Heal" then
		if recap_temp.FightStartHeal==0 then
			recap_temp.FightStartHeal = event_time
			recap_temp.FightEndHeal = recap_temp.FightStartHeal
		else
			recap_temp.FightEndHeal = event_time
		end
	end

	-- Hawksy: three independent timers per combatant for damage dealt, damage received, and healing done
	--   (at the moment no timer for healing received)
	if timeType == "Out" then
		if recap_temp.Last[thisCombatant].StartOut==0 then
			recap_temp.Last[thisCombatant].StartOut = event_time
			recap_temp.Last[thisCombatant].EndOut = recap_temp.Last[thisCombatant].StartOut
		else
			recap_temp.Last[thisCombatant].EndOut = event_time
		end
	elseif timeType == "In" then
		if recap_temp.Last[thisCombatant].StartIn==0 then
			recap_temp.Last[thisCombatant].StartIn = event_time
			recap_temp.Last[thisCombatant].EndIn = recap_temp.Last[thisCombatant].StartIn
		else
			recap_temp.Last[thisCombatant].EndIn = event_time
		end
	elseif timeType == "Heal" then
		if recap_temp.Last[thisCombatant].StartHeal==0 then
			recap_temp.Last[thisCombatant].StartHeal = event_time
			recap_temp.Last[thisCombatant].EndHeal = recap_temp.Last[thisCombatant].StartHeal
		else
			recap_temp.Last[thisCombatant].EndHeal = event_time
		end
	end

	-- anyone who does damage, takes damage, or heals will be flagged as having
	--   been in the fight, but simply getting healed still doesn't count
	recap.Combatant[thisCombatant].WasInCurrent = true
end

function Recap_StartFight()

	local i

	if recap.Opt.AutoHide.value and not recap.Opt.Minimized.value then
		RecapFrame_Hide()
	end

	-- Hawksy: only clear WasInCurrent once per fight
	if	not recap_temp.ResetWasInCurrent then
		-- Hawksy: silent repair
		if not recap.Combatant then
			recap.Combatant = {}
		end
		for i in pairs(recap.Combatant) do
			recap.Combatant[i].WasInCurrent = false
		end
		recap_temp.ResetWasInCurrent = true
	end

	-- start idle timer
	if not recap.Opt.LimitFights.value and recap.Opt.IdleFight.value then
		recap_temp.IdleTimer = 0
	end
	RecapIdleFrame:Show() -- start idle OnUpdates

	-- make sure end of fight delay timer is off
	recap_temp.EndFightDelayTimer = -1
	RecapEndFightDelayFrame:Hide()

	-- Hawksy: check friends at start of fight to get HP for more accurate
	--   calculation of overhealing.  See also Recap_GetUnitInfo.
	Recap_MakeFriends()

	-- clear any existing synchronization update data
	Recap_ClearSyncData()

	Recap_SetState("Active")

end

function Recap_AddPetToOwner(pet, owner)

	-- Hawksy: guard code
	if not recap.Combatant[pet] then
		return
	end
	if not recap.Combatant[owner] then
		return
	end
	if not recap_temp.Last[pet] then
		return
	end
	if not recap_temp.Last[owner] then
		return
	end

	-- Hawksy: now three independent times, for damage, incoming, and healing
	if (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartOut > 0)) and
	   (recap.Combatant[owner].WasInCurrent and (recap_temp.Last[owner].StartOut > 0)) then
		-- both pet and owner were in the fight and did damage
		recap_temp.Last[owner].StartOut = math.min(recap_temp.Last[pet].StartOut,recap_temp.Last[owner].StartOut)
		recap_temp.Last[owner].EndOut = math.max(recap_temp.Last[pet].EndOut,recap_temp.Last[owner].EndOut)
	elseif (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartOut > 0)) and
		   (not recap.Combatant[owner].WasInCurrent or (recap_temp.Last[owner].StartOut == 0)) then
		-- the pet was in the fight and did damage, the owner did no damage
		recap_temp.Last[owner].StartOut = recap_temp.Last[pet].StartOut
		recap_temp.Last[owner].EndOut = recap_temp.Last[pet].EndOut
	end
	if (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartIn > 0)) and
	   (recap.Combatant[owner].WasInCurrent and (recap_temp.Last[owner].StartIn > 0)) then
		-- both pet and owner were in the fight and took damage
		recap_temp.Last[owner].StartIn = math.min(recap_temp.Last[pet].StartIn,recap_temp.Last[owner].StartIn)
		recap_temp.Last[owner].EndIn = math.max(recap_temp.Last[pet].EndIn,recap_temp.Last[owner].EndIn)
	elseif (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartIn > 0)) and
		   (not recap.Combatant[owner].WasInCurrent or (recap_temp.Last[owner].StartIn == 0)) then
		-- the pet was in the fight and took damage, the owner took no damage
		recap_temp.Last[owner].StartIn = recap_temp.Last[pet].StartIn
		recap_temp.Last[owner].EndIn = recap_temp.Last[pet].EndIn
	end
	if (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartHeal > 0)) and
	   (recap.Combatant[owner].WasInCurrent and (recap_temp.Last[owner].StartHeal > 0)) then
		-- both pet and owner were in the fight and did healing
		recap_temp.Last[owner].StartHeal = math.min(recap_temp.Last[pet].StartHeal,recap_temp.Last[owner].StartHeal)
		recap_temp.Last[owner].EndHeal = math.max(recap_temp.Last[pet].EndHeal,recap_temp.Last[owner].EndHeal)
	elseif (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartHeal > 0)) and
		   (not recap.Combatant[owner].WasInCurrent or (recap_temp.Last[owner].StartHeal == 0)) then
		-- the pet was in the fight and did healing, the owner did no healing
		recap_temp.Last[owner].StartHeal = recap_temp.Last[pet].StartHeal
		recap_temp.Last[owner].EndHeal = recap_temp.Last[pet].EndHeal
	end

	recap_temp.Last[owner].DmgIn = recap_temp.Last[owner].DmgIn + recap_temp.Last[pet].DmgIn
	recap_temp.Last[owner].DmgOut = recap_temp.Last[owner].DmgOut + recap_temp.Last[pet].DmgOut
	recap_temp.Last[owner].MaxHit = math.max(recap_temp.Last[owner].MaxHit,recap_temp.Last[pet].MaxHit)
	-- Hawksy: add in heals and kills
	recap_temp.Last[owner].Kills = recap_temp.Last[owner].Kills + recap_temp.Last[pet].Kills
	recap_temp.Last[owner].RawHeal = recap_temp.Last[owner].RawHeal + recap_temp.Last[pet].RawHeal
	recap_temp.Last[owner].OverHeal = math.min((recap_temp.Last[owner].OverHeal + recap_temp.Last[pet].OverHeal),recap_temp.Last[owner].RawHeal)

	recap_temp.Last[pet].StartOut = 0
	recap_temp.Last[pet].EndOut = 0
	recap_temp.Last[pet].StartIn = 0
	recap_temp.Last[pet].EndIn = 0
	recap_temp.Last[pet].StartHeal = 0
	recap_temp.Last[pet].EndHeal = 0
	recap_temp.Last[pet].DmgIn = 0
	recap_temp.Last[pet].DmgOut = 0
	recap_temp.Last[pet].MaxHit = 0
	recap_temp.Last[pet].Kills = 0
	recap_temp.Last[pet].RawHeal = 0
	recap_temp.Last[pet].OverHeal = 0

	Recap_AccumulateCombatant(owner)

	recap.Combatant[pet].WasInCurrent = false
	recap.Combatant[owner].WasInCurrent = false
	recap.Combatant[pet].WasInLast = false
	recap.Combatant[owner].WasInLast = true

	-- Hawksy: effects are now treated separately

end

function Recap_AccumulateCombatant(i)

	recap.Combatant[i].LastDmgIn = recap_temp.Last[i].DmgIn
	recap.Combatant[i].LastDmgOut = recap_temp.Last[i].DmgOut
	recap.Combatant[i].LastTime = recap_temp.Last[i].EndOut - recap_temp.Last[i].StartOut
	recap.Combatant[i].LastTimeIn = recap_temp.Last[i].EndIn - recap_temp.Last[i].StartIn
	recap.Combatant[i].LastTimeHeal = recap_temp.Last[i].EndHeal - recap_temp.Last[i].StartHeal
	recap.Combatant[i].LastMaxHit = recap_temp.Last[i].MaxHit
	recap.Combatant[i].LastKills = recap_temp.Last[i].Kills
	recap.Combatant[i].LastRawHeal = recap_temp.Last[i].RawHeal
	recap.Combatant[i].LastOverHeal = math.min(recap_temp.Last[i].OverHeal,recap.Combatant[i].LastRawHeal)

	-- Hawksy: If there is non-zero damage or healing, force the matching time to be
	--   (somewhat arbitrarily) at least as large as the global cooldown ("DeemedTime").
	--   This gives some duration to single-shot fights (which are accumulated), keeping
	--   the anomalous DPS down a bit, without displaying them or their DPS.
	if (recap.Combatant[i].LastDmgOut > 0) and (recap.Combatant[i].LastTime < recap_temp.DeemedTime) then
		recap.Combatant[i].LastTime = recap_temp.DeemedTime
	end
	if (recap.Combatant[i].LastDmgIn > 0) and (recap.Combatant[i].LastTimeIn < recap_temp.DeemedTime) then
		recap.Combatant[i].LastTimeIn = recap_temp.DeemedTime
	end
	if (recap.Combatant[i].LastRawHeal > 0) and (recap.Combatant[i].LastTimeHeal < recap_temp.DeemedTime) then
		recap.Combatant[i].LastTimeHeal = recap_temp.DeemedTime
	end

	recap_temp.Last[i].StartOut = 0
	recap_temp.Last[i].EndOut = 0
	recap_temp.Last[i].StartIn = 0
	recap_temp.Last[i].EndIn = 0
	recap_temp.Last[i].StartHeal = 0
	recap_temp.Last[i].EndHeal = 0
	recap_temp.Last[i].DmgIn = 0
	recap_temp.Last[i].DmgOut = 0
	recap_temp.Last[i].MaxHit = 0
	recap_temp.Last[i].Kills = 0
	recap_temp.Last[i].RawHeal = 0
	recap_temp.Last[i].OverHeal = 0

	recap.Combatant[i].TotalDmgIn = recap.Combatant[i].TotalDmgIn + recap.Combatant[i].LastDmgIn
	recap.Combatant[i].TotalDmgOut = recap.Combatant[i].TotalDmgOut + recap.Combatant[i].LastDmgOut
	recap.Combatant[i].TotalTime = recap.Combatant[i].TotalTime + recap.Combatant[i].LastTime
	recap.Combatant[i].TotalTimeIn = recap.Combatant[i].TotalTimeIn + recap.Combatant[i].LastTimeIn
	recap.Combatant[i].TotalTimeHeal = recap.Combatant[i].TotalTimeHeal + recap.Combatant[i].LastTimeHeal
	recap.Combatant[i].TotalKills = recap.Combatant[i].TotalKills + recap.Combatant[i].LastKills
	recap.Combatant[i].TotalRawHeal = recap.Combatant[i].TotalRawHeal + (recap.Combatant[i].LastRawHeal or 0)
	recap.Combatant[i].TotalOverHeal = math.min((recap.Combatant[i].TotalOverHeal + (recap.Combatant[i].LastOverHeal or 0)),recap.Combatant[i].TotalRawHeal)

	-- Hawksy: three flavours of "per second" calculation
	if recap.Combatant[i].LastTime > recap_temp.MinTime then
		recap.Combatant[i].LastDPS = recap.Combatant[i].LastDmgOut/recap.Combatant[i].LastTime
	else
		recap.Combatant[i].LastDPS = 0
	end
	if recap.Combatant[i].TotalTime > recap_temp.MinTime then
		recap.Combatant[i].TotalDPS = recap.Combatant[i].TotalDmgOut/recap.Combatant[i].TotalTime
	else
		recap.Combatant[i].TotalDPS = 0
	end
	if recap.Combatant[i].LastTimeIn > recap_temp.MinTime then
		recap.Combatant[i].LastDPSIn = recap.Combatant[i].LastDmgIn/recap.Combatant[i].LastTimeIn
	else
		recap.Combatant[i].LastDPSIn = 0
	end
	if recap.Combatant[i].TotalTimeIn > recap_temp.MinTime then
		recap.Combatant[i].TotalDPSIn = recap.Combatant[i].TotalDmgIn/recap.Combatant[i].TotalTimeIn
	else
		recap.Combatant[i].TotalDPSIn = 0
	end
	if recap.Combatant[i].LastTimeHeal > recap_temp.MinTime then
		recap.Combatant[i].LastHPS = (recap.Combatant[i].LastRawHeal - recap.Combatant[i].LastOverHeal)/recap.Combatant[i].LastTimeHeal
	else
		recap.Combatant[i].LastHPS = 0
	end
	if recap.Combatant[i].TotalTimeHeal > recap_temp.MinTime then
		recap.Combatant[i].TotalHPS = (recap.Combatant[i].TotalRawHeal - recap.Combatant[i].TotalOverHeal)/recap.Combatant[i].TotalTimeHeal
	else
		recap.Combatant[i].TotalHPS = 0
	end

	recap.Combatant[i].TotalMaxHit = math.max(recap.Combatant[i].LastMaxHit,recap.Combatant[i].TotalMaxHit)

	recap.Combatant[i].WasInCurrent = false
	recap.Combatant[i].WasInLast = true
end

-- used to remove pet effects when Merge Pets has been set to off, after having been on
function Recap_RemovePetEffects()

	local thisCombatant, i, effectNum

	-- Hawksy: silent repair
	if not recap.Combatant then
		recap.Combatant = {}
	end
	for thisCombatant in pairs(recap.Combatant) do
		-- Hawksy: silent repair
		if not recap.Combatant[thisCombatant].OutgoingDetail then
			recap.Combatant[thisCombatant].OutgoingDetail = {}
		end
		if not recap.Combatant[thisCombatant].IncomingDetail then
			recap.Combatant[thisCombatant].IncomingDetail = {}
		end
		for i in pairs(recap.Combatant[thisCombatant].OutgoingDetail) do
			effectNum = string.sub(i,1,1)
			if (effectNum == "2") or (effectNum == "4") then
				-- remove pet effect
				recap.Combatant[thisCombatant].OutgoingDetail[i] = nil -- keep as nil
			end
		end
		for i in pairs(recap.Combatant[thisCombatant].IncomingDetail) do
			effectNum = string.sub(i,1,1)
			if (effectNum == "2") or (effectNum == "4") then
				-- remove pet effect
				recap.Combatant[thisCombatant].IncomingDetail[i] = nil -- keep as nil
			end
		end
	end
	-- and personal details
	-- Hawksy: with 3.67 this next bit is now only for cleanup: no new effects of this type will be created
	for i in pairs(recap.Self[recap_temp.s]) do
		effectNum = string.sub(i,1,1)
		if (effectNum == "2") or (effectNum == "4") then
			-- remove pet effect
			recap.Self[recap_temp.s][i] = nil -- keep as nil
		end
	end
end

function Recap_CopyPetEffectsToOwner(pet, owner)

	local i
	local peteffect

	if not recap.Opt.LightData.value then
		for i in pairs(recap.Combatant[pet].OutgoingDetail) do
			if string.sub(i,1,1)=="1" then
				peteffect = "2"..pet..": "..string.sub(i,2)
			else
				peteffect = "4"..pet..": "..string.sub(i,2)
			end
			recap.Combatant[owner].OutgoingDetail[peteffect] = {
				Hits = recap.Combatant[pet].OutgoingDetail[i].Hits, HitsDmg = recap.Combatant[pet].OutgoingDetail[i].HitsDmg,
				HitsMax = recap.Combatant[pet].OutgoingDetail[i].HitsMax, Crits = recap.Combatant[pet].OutgoingDetail[i].Crits,
				CritsEvents = recap.Combatant[pet].OutgoingDetail[i].CritsEvents, CritsDmg = recap.Combatant[pet].OutgoingDetail[i].CritsDmg,
				CritsMax = recap.Combatant[pet].OutgoingDetail[i].CritsMax, Missed = recap.Combatant[pet].OutgoingDetail[i].Missed,
				Ticks = recap.Combatant[pet].OutgoingDetail[i].Ticks, TicksDmg = recap.Combatant[pet].OutgoingDetail[i].TicksDmg,
				TicksMax = recap.Combatant[pet].OutgoingDetail[i].TicksMax, Element = recap.Combatant[pet].OutgoingDetail[i].Element,
				Dodged = recap.Combatant[pet].OutgoingDetail[i].Dodged, Parried = recap.Combatant[pet].OutgoingDetail[i].Parried,
				Blocked = recap.Combatant[pet].OutgoingDetail[i].Blocked, Absorbed = recap.Combatant[pet].OutgoingDetail[i].Absorbed,
				Deflected = recap.Combatant[pet].OutgoingDetail[i].Deflected, Evaded = recap.Combatant[pet].OutgoingDetail[i].Evaded,
				Resisted = recap.Combatant[pet].OutgoingDetail[i].Resisted, Reflected = recap.Combatant[pet].OutgoingDetail[i].Reflected,
				Immune = recap.Combatant[pet].OutgoingDetail[i].Immune }
		end
		for i in pairs(recap.Combatant[pet].IncomingDetail) do
			if string.sub(i,1,1)=="1" then
				peteffect = "2"..string.sub(i,2).." "..recap_temp.Localize.On.." "..pet
			else
				peteffect = "4"..string.sub(i,2).." "..recap_temp.Localize.On.." "..pet
			end
			recap.Combatant[owner].IncomingDetail[peteffect] = {
				Hits = recap.Combatant[pet].IncomingDetail[i].Hits, HitsDmg = recap.Combatant[pet].IncomingDetail[i].HitsDmg,
				HitsMax = recap.Combatant[pet].IncomingDetail[i].HitsMax, Crits = recap.Combatant[pet].IncomingDetail[i].Crits,
				CritsEvents = recap.Combatant[pet].IncomingDetail[i].CritsEvents, CritsDmg = recap.Combatant[pet].IncomingDetail[i].CritsDmg,
				CritsMax = recap.Combatant[pet].IncomingDetail[i].CritsMax, Missed = recap.Combatant[pet].IncomingDetail[i].Missed,
				Ticks = recap.Combatant[pet].IncomingDetail[i].Ticks, TicksDmg = recap.Combatant[pet].IncomingDetail[i].TicksDmg,
				TicksMax = recap.Combatant[pet].IncomingDetail[i].TicksMax, Element = recap.Combatant[pet].IncomingDetail[i].Element,
				Dodged = recap.Combatant[pet].IncomingDetail[i].Dodged, Parried = recap.Combatant[pet].IncomingDetail[i].Parried,
				Blocked = recap.Combatant[pet].IncomingDetail[i].Blocked, Absorbed = recap.Combatant[pet].IncomingDetail[i].Absorbed,
				Deflected = recap.Combatant[pet].IncomingDetail[i].Deflected, Evaded = recap.Combatant[pet].IncomingDetail[i].Evaded,
				Resisted = recap.Combatant[pet].IncomingDetail[i].Resisted, Reflected = recap.Combatant[pet].IncomingDetail[i].Reflected,
				Immune = recap.Combatant[pet].IncomingDetail[i].Immune }
		end
	end
end

function Recap_EndFight(forceEnd)

	local i

	-- turn off idle and end of fight delay timers
	recap_temp.IdleTimer = -1
	RecapIdleFrame:Hide()
	recap_temp.EndFightDelayTimer = -1
	RecapEndFightDelayFrame:Hide()

	if forceEnd or recap.Opt.State.value=="Active" then

		local debug_time = GetTime()

		Recap_MakeFriends()

		-- Hawksy: three states for HoldFight
		--   Normal: do nothing different from the usual Recap behaviour
		--   HoldNext: process this end of fight as usual; then hold further fights by switching to Holding
		--   Holding: currently holding and awaiting release
		if forceEnd or (recap_temp.HoldFight == "Normal") or (recap_temp.HoldFight == "HoldNext") then

			recap.Opt.State.value = "Stopped" -- suspend logging to calculate

			-- Hawksy: silent repair
			if not recap.Combatant then
				recap.Combatant = {}
			end
			-- Hawksy: set to false as part of a fix to ensure that pet owners always get a chance
			--   to show up in the Last panel when Merge is on
			for i in pairs(recap.Combatant) do
				recap.Combatant[i].WasInLast = false
			end

			if recap_temp.FightEnd>recap_temp.FightStart then
				recap[recap_temp.p].LastDuration = recap_temp.FightEnd - recap_temp.FightStart
			else
				recap[recap_temp.p].LastDuration = 0
			end
			if recap_temp.FightEndIn>recap_temp.FightStartIn then
				recap[recap_temp.p].LastDurationIn = recap_temp.FightEndIn - recap_temp.FightStartIn
			else
				recap[recap_temp.p].LastDurationIn = 0
			end
			if recap_temp.FightEndHeal>recap_temp.FightStartHeal then
				recap[recap_temp.p].LastDurationHeal = recap_temp.FightEndHeal - recap_temp.FightStartHeal
			else
				recap[recap_temp.p].LastDurationHeal = 0
			end
			recap[recap_temp.p].TotalDuration = recap[recap_temp.p].TotalDuration + recap[recap_temp.p].LastDuration
			recap[recap_temp.p].TotalDurationIn = recap[recap_temp.p].TotalDurationIn + recap[recap_temp.p].LastDurationIn
			recap[recap_temp.p].TotalDurationHeal = recap[recap_temp.p].TotalDurationHeal + recap[recap_temp.p].LastDurationHeal
			recap_temp.FightStart = 0
			recap_temp.FightEnd = 0
			recap_temp.FightStartin = 0
			recap_temp.FightEndIn = 0
			recap_temp.FightStartHeal = 0
			recap_temp.FightEndHeal = 0

			-- Hawksy: silent repair
			if not recap_temp.Last then
				recap_temp.Last = {}
			end
			for i in pairs(recap_temp.Last) do

				if recap.Combatant[i].WasInCurrent then

					-- Accumulate combatant and possibly merge pets

					-- Hawksy: one of these calls will set WasInLast to true if appropriate
					-- Hawksy: synchronization overrides the regular MergePets setting
					if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
						-- synchronization active
						if recap.Combatant[i].OwnedBy and (recap.Opt.SyncMergePets.value == "on") then
							Recap_AddPetToOwner(i, recap.Combatant[i].OwnedBy)
							Recap_CopyPetEffectsToOwner(i, recap.Combatant[i].OwnedBy)
						elseif recap.Combatant[i].OwnsPet and (recap.Opt.SyncMergePets.value == "on") then
							Recap_AddPetToOwner(recap.Combatant[i].OwnsPet, i)
							Recap_CopyPetEffectsToOwner(recap.Combatant[i].OwnsPet, i)
						else
							Recap_AccumulateCombatant(i)
						end
					else
						-- not synchronization
						if recap.Combatant[i].OwnedBy and recap.Opt.MergePets.value then
							Recap_AddPetToOwner(i, recap.Combatant[i].OwnedBy)
							Recap_CopyPetEffectsToOwner(i, recap.Combatant[i].OwnedBy)
						elseif recap.Combatant[i].OwnsPet and recap.Opt.MergePets.value then
							Recap_AddPetToOwner(recap.Combatant[i].OwnsPet, i)
							Recap_CopyPetEffectsToOwner(recap.Combatant[i].OwnsPet, i)
						else
							Recap_AccumulateCombatant(i)
						end
					end

				else
					-- Hawksy: don't set WasInLast here, it could exclude an owner from the Last panel
					--   if the owner is processed later than the pet when Merge is true
				end

				-- ensure that every combatant has their 'Last' record cleared, even if not shown as having been in current fight
				recap_temp.Last[i].StartOut = 0
				recap_temp.Last[i].EndOut = 0
				recap_temp.Last[i].StartIn = 0
				recap_temp.Last[i].EndIn = 0
				recap_temp.Last[i].StartHeal = 0
				recap_temp.Last[i].EndHeal = 0
				recap_temp.Last[i].DmgIn = 0
				recap_temp.Last[i].DmgOut = 0
				recap_temp.Last[i].MaxHit = 0
				recap_temp.Last[i].Kills = 0
				recap_temp.Last[i].RawHeal = 0
				recap_temp.Last[i].OverHeal = 0
			end

			-- Hawksy: if the Last Fight panel is being held, this supports multiple fights happening in the background
			recap_temp.ResetWasInCurrent = false

			-- Hawksy: if Recent Data Mode is active, add an 'end of fight' event (signalled by false values for source and dest)
			--   (if holding the Last Fight panel this could be later than otherwise)
			if recap.Opt.RecentData.value then
				Recap_AddRecentEvent(debug_time, false, false, false, false, false, false, false, false)
			end

			-- Hawksy: special case correction for some dual damage/healing tickers
			Recap_CorrectDualDoTHoT()

			-- If holding the Last Fight panel then postings will be later than otherwise
			if recap.Opt.AutoPost.value then
				recap.Opt.SortBy.value = Recap_AutoPostGetStatID(recap.Opt.AutoPost.Stat)
				recap.Opt.SortDir.value = false
			end

			Recap_ConstructList()
			Recap_SetState("Idle")

			-- Hawksy: refresh the Blizzard party frame at the end of a fight (if the group changes during a fight the frame is not always updated)
			if recap.Opt.RefreshPartyFrame.value then
				ShowPartyFrame()
			end

			if recap.Opt.AutoPost.value then
				Recap_PostFight()
			end

			if recap.Opt.AutoLeader.value then
				Recap_PostLeader()
			end

			if recap.debug then
				DEFAULT_CHAT_FRAME:AddMessage("(recap.debug) Fight processed in "..string.format("%.4f",GetTime()-debug_time).." seconds for "..(recap_temp.ListSize-1).." combatants.")
			end

		end

		-- If we just set the Last Fight, and need to hold it, set the state variable
		if (recap_temp.HoldFight == "HoldNext") then
			recap_temp.HoldFight = "Holding"
		end

		-- If we are holding the Last Fight, display the Last Fight panel at every end of fight (until they click the Resume button)
		if (recap_temp.HoldFight == "Holding") then
			recap.Opt.View.value = "Last"
			recap.Opt.SelfView.value = false
			Recap_Maximize()
		end

		if recap.Opt.AutoHide.value and not recap.Opt.Minimized.value then
			RecapFrame_Show()
			if recap.Opt.AutoFade.value then
				recap_temp.FadeTimer = 0
			end
		end

		-- Hawksy: force a garbage collection, as this is probably the best time, just after the end of a fight
		if recap.Opt.CollectGarbage.value then
			local gc_time = GetTime()
			collectgarbage("collect")
			if recap.debug then
				DEFAULT_CHAT_FRAME:AddMessage("(recap.debug) Garbage collected in "..string.format("%.4f",GetTime()-gc_time).." seconds.")
				DEFAULT_CHAT_FRAME:AddMessage("Memory in MB "..string.format("%.3f",collectgarbage("count")/1000))
			end
		end
	end
end

function Recap_ClearAsIfEndOfFight()
	-- ensure that fight timers are cleared (had previously only been cleared on end of fight)
	recap_temp.FightStart = 0
	recap_temp.FightEnd = 0
	recap_temp.FightStartin = 0
	recap_temp.FightEndIn = 0
	recap_temp.FightStartHeal = 0
	recap_temp.FightEndHeal = 0
	if not recap_temp.Last then
		recap_temp.Last = {}
	end
	for i in pairs(recap_temp.Last) do
		-- ensure that every 'Last' record is cleared
		recap_temp.Last[i].StartOut = 0
		recap_temp.Last[i].EndOut = 0
		recap_temp.Last[i].StartIn = 0
		recap_temp.Last[i].EndIn = 0
		recap_temp.Last[i].StartHeal = 0
		recap_temp.Last[i].EndHeal = 0
		recap_temp.Last[i].DmgIn = 0
		recap_temp.Last[i].DmgOut = 0
		recap_temp.Last[i].MaxHit = 0
		recap_temp.Last[i].Kills = 0
		recap_temp.Last[i].RawHeal = 0
		recap_temp.Last[i].OverHeal = 0
	end
	-- turn off idle and end of fight delay timers
	recap_temp.IdleTimer = -1
	RecapIdleFrame:Hide()
	recap_temp.EndFightDelayTimer = -1
	RecapEndFightDelayFrame:Hide()
end

-- adds CHAT_MSG_SPELL/COMBAT parsings (from ParserLib) to .Last running totals
ParserLib_SELF = 103
ParserLib_MELEE = 112
ParserLib_DAMAGESHIELD = 113
ParserLib_NONE = 200 -- (source is absent)
function Recap_DamageEvent(event, info)

	local source, dest, damage, effect, element
	local event_time = GetTime()

	if recap.Opt.State.value~="Stopped" then

		if recap.dump then
			local printable
			printable = Recap_CreateMessage(event)
			Recap_DumpMessage("DAMAGE;"..printable)
		end

--if info.isGlancing or info.amountAbsorb or info.amountBlock or info.amountResist or info.amountVulnerable or info.isItem then
--	local printable
--	printable = Recap_CreateMessage(event)
--	Recap_DumpMessage("DAMAGE;"..printable)
--end

		source = info.source
		if tonumber(source) == ParserLib_SELF then
			source = recap_temp.Player
		end
		if tonumber(source) == ParserLib_NONE then
			-- damage with no source
			source = false
		end
		dest = info.victim
		if tonumber(dest) == ParserLib_SELF then
			dest = recap_temp.Player
		end
		-- Hawksy: there are still occasional appearances of the personal pronoun despite ParserLib
		if source then
			if recap_temp.SelfPronouns[string.lower(source)] then
DEFAULT_CHAT_FRAME:AddMessage("Recap_DamageEvent personal pronoun arg1 ("..tostring(arg1)..") event "..tostring(event).." source "..tostring(source))
if recap_temp.snap then
Screenshot()
end
				source = recap_temp.Player
			end
		end
		if recap_temp.SelfPronouns[string.lower(dest)] then
DEFAULT_CHAT_FRAME:AddMessage("Recap_DamageEvent personal pronoun arg1 ("..tostring(arg1)..") event "..tostring(event).." dest "..tostring(dest))
if recap_temp.snap then
Screenshot()
end
			dest = recap_temp.Player
		end

		damage = 0
		damage = tonumber(info.amount)
		effect = info.skill
		if tonumber(effect) == ParserLib_MELEE then
			effect = RECAP_MELEE
		end
		if tonumber(effect) == ParserLib_DAMAGESHIELD then
			effect = recap_temp.Localize.DamageShield
		end

		-- default element is Melee or Other when Blizzard doesn't specify
		if effect == RECAP_MELEE then
			element = RECAP_MELEE
		else
			element = recap_temp.Localize.ElementOther
		end
		if info.element then
			element = info.element
		end

		-- Hawksy: check for known duplicates
		if source then
			if Recap_CheckForDuplicateSource(source) then
				return
			end
		end
		if Recap_CheckForDuplicateEffect(effect) then
			return
		end

		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
			-- in synchronization, don't ignore anyone (if 'ignore' they will still be hidden on the Fights panel)
		else
			if (source and recap.Combatant[source] and recap.Combatant[source].Ignore) then
				-- source is an ignored combatant, pretend the hit didn't happen at all
				return
			end
			if (recap.Combatant[dest] and recap.Combatant[dest].Ignore) then
				-- dest is an ignored combatant, pretend the hit didn't happen at all
				return
			end
		end

		-- possibly start the fight
		if not recap.Opt.LimitFights.value and recap.Opt.State.value=="Idle" then
			Recap_StartFight()
		end

		-- if the fight is ongoing, start or restart the idle timer
		if recap.Opt.State.value=="Active" then
			if recap.Opt.IdleFight.value then
				recap_temp.IdleTimer = 0
			end
			RecapIdleFrame:Show() -- start idle OnUpdates
		end

		-- if the fight is ongoing and if the end of fight delay timer is running, restart the end of fight delay timer
		--   (the player left combat, perhaps temporarily, but the fight continues)
		if recap.Opt.State.value=="Active" and recap_temp.EndFightDelayTimer~=-1 then
			recap_temp.EndFightDelayTimer = 0
			RecapEndFightDelayFrame:Show() -- start end fight delay OnUpdates
		end

		-- Hawksy: events are already almost entirely filtered down to damage or miss, so let's start the timers
		if source then
			Recap_CreateCombatantAndUpdateTimers(source, event_time, "Out")
		end
		Recap_CreateCombatantAndUpdateTimers(dest, event_time, "In")

		-- remove friend flag from hostiles no longer charmed
		if source then
			if recap.Combatant[source].Friend and recap.Combatant[dest].Friend then
				recap.Combatant[source].Friend = false
			end
		end

		if damage and (damage > 0) then
			-- process totals

			if (source == dest) then
				-- Hawksy: ignore the Outgoing component of damage to self (such as Shadow Word: Death damage to self) (otherwise would inflate DPS)
			else
				if source then
					if (recap.Combatant[source].OwnsPet == dest) or (recap.Combatant[source].OwnedBy == dest) then
						-- Hawksy: ignore the Outgoing component of damage to or from a combatant's pet (Soul Link does this) (otherwise would inflate DPS)
					else
						recap_temp.Last[source].DmgOut = recap_temp.Last[source].DmgOut + damage
						if damage > recap_temp.Last[source].MaxHit then
							recap_temp.Last[source].MaxHit = damage
						end
					end
				end
			end

			recap_temp.Last[dest].DmgIn = recap_temp.Last[dest].DmgIn + damage
			if recap_temp.Last[dest].HP then
				recap_temp.Last[dest].HP = recap_temp.Last[dest].HP - damage
			end

			if not recap.Opt.LightData.value then
				-- determine whether it could have critted
				if string.find(event,"PERIODIC") then
					-- periodic event, assume not crit-capable, pass 'nil' in the crit slot (and in the crush slot) -- keep crit and crush as 'nil', not false
					Recap_CreateEffect(source, dest, effect, 1, damage, nil, info.isDOT, element, false, nil)
				else
					-- assume all others are crit/crush capable
					local critty, crushy
					critty = info.isCrit or false
					crushy = info.isCrushing or false
					Recap_CreateEffect(source, dest, effect, 1, damage, critty, info.isDOT, element, false, crushy)
				end
			end

			-- Hawksy: support for Recent Data Mode
			if recap.Opt.RecentData.value then
				Recap_AddRecentEvent(event_time, source, dest, 1, effect, element, damage, info.isCrit, info.isCrushing)
			end
		else
			-- no actual damage, probably a miss
			if not recap.Opt.LightData.value then
				if (source == dest) then
					-- Hawksy: ignore misses related to self-damage (if any)
				else
					if (info.type == "miss") then
						-- Hawksy: ignore misses that haven't yet caused damage for this source
						-- this could lose some opening misses -- tough
						-- there are a lot of non-damage misses that get sent to us as damage events, and we need to lose those
						local typeEffect, total
						typeEffect = "1"..effect
						total = 0
						if source and recap.Combatant[source].OutgoingDetail and recap.Combatant[source].OutgoingDetail[typeEffect] then
							total = (recap.Combatant[source].OutgoingDetail[typeEffect].HitsDmg or 0) +
									(recap.Combatant[source].OutgoingDetail[typeEffect].CritsDmg or 0) +
									(recap.Combatant[source].OutgoingDetail[typeEffect].CrushDmg or 0) +
									(recap.Combatant[source].OutgoingDetail[typeEffect].TicksDmg or 0)
						end
						if total > 0 then
							-- this effect has done damage, so count the miss
							-- register the miss as a non-crit-capable event -- keep crit and crush as 'nil', not false
							Recap_CreateEffect(source, dest, effect, 1, 0, nil, false, element, info.missType, nil)

							-- Hawksy: support for Recent Data Mode
							if recap.Opt.RecentData.value then
								local missText = (info.missType or "miss")
								Recap_AddRecentEvent(event_time, source, dest, 1, effect, element, missText, false, false)
							end
						else
							-- Hawksy: FUTURE: perhaps store it as a buff/debuff/proc/etc.
						end
					else
						-- Hawksy: no damage and not a miss
						Recap_CheckUnexpectedEvent()
					end
				end
			end
		end
	end
end


function Recap_CreateEffect(source, dest, effect, type, total, did_crit, is_dot, element, missType, did_crush)

	-- Hawksy: parameter is_dot is not currently used

	-- Hawksy: guard code
	if not effect then
		return
	end

	local typeEffect
	typeEffect = type..effect

	-- is the source the player or one of the player's pets?
	local selfIndex = nil -- keep as nil
	if source then
		if source == recap_temp.Player then
			selfIndex = recap_temp.s
		elseif recap.Combatant[source].OwnedBy == recap_temp.Player then
			selfIndex = recap_temp.s..":"..source
		end
	end

	if (source == dest) and (type < 3) then
		-- Hawksy: ignore the Outgoing component of self-damage (such as Shadow Word: Death damage to self) (otherwise would inflate DPS)
		if (did_crit==true) then
			Recap_MakeIncomingDetail(dest, typeEffect)
			recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents = (recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents or 0) + 1
			recap.Combatant[dest].IncomingDetail[typeEffect].Crits = (recap.Combatant[dest].IncomingDetail[typeEffect].Crits or 0) + 1
			recap.Combatant[dest].IncomingDetail[typeEffect].CritsDmg = (recap.Combatant[dest].IncomingDetail[typeEffect].CritsDmg or 0) + total
			recap.Combatant[dest].IncomingDetail[typeEffect].CritsMax = math.max(total,recap.Combatant[dest].IncomingDetail[typeEffect].CritsMax or 0)
			Recap_AddIncomingElement(dest, typeEffect, element)
		elseif (did_crush==true) then
			Recap_MakeIncomingDetail(dest, typeEffect)
			recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents = (recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents or 0) + 1
			recap.Combatant[dest].IncomingDetail[typeEffect].Crushes = (recap.Combatant[dest].IncomingDetail[typeEffect].Crushes or 0) + 1
			recap.Combatant[dest].IncomingDetail[typeEffect].CrushDmg = (recap.Combatant[dest].IncomingDetail[typeEffect].CrushDmg or 0) + total
			recap.Combatant[dest].IncomingDetail[typeEffect].CrushMax = math.max(total,recap.Combatant[dest].IncomingDetail[typeEffect].CrushMax or 0)
			Recap_AddIncomingElement(dest, typeEffect, element)
		elseif (did_crit==false) and (did_crush==false) then
			Recap_MakeIncomingDetail(dest, typeEffect)
			recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents = (recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents or 0) + 1
			recap.Combatant[dest].IncomingDetail[typeEffect].Hits = (recap.Combatant[dest].IncomingDetail[typeEffect].Hits or 0) + 1
			recap.Combatant[dest].IncomingDetail[typeEffect].HitsDmg = (recap.Combatant[dest].IncomingDetail[typeEffect].HitsDmg or 0) + total
			recap.Combatant[dest].IncomingDetail[typeEffect].HitsMax = math.max(total,recap.Combatant[dest].IncomingDetail[typeEffect].HitsMax or 0)
			Recap_AddIncomingElement(dest, typeEffect, element)
		elseif (total==0) then
			-- ignore any miss for self damage entirely
		else
			Recap_MakeIncomingDetail(dest, typeEffect)
			recap.Combatant[dest].IncomingDetail[typeEffect].Ticks = (recap.Combatant[dest].IncomingDetail[typeEffect].Ticks or 0) + 1
			recap.Combatant[dest].IncomingDetail[typeEffect].TicksDmg = (recap.Combatant[dest].IncomingDetail[typeEffect].TicksDmg or 0) + total
			recap.Combatant[dest].IncomingDetail[typeEffect].TicksMax = math.max(total,recap.Combatant[dest].IncomingDetail[typeEffect].TicksMax or 0)
			Recap_AddIncomingElement(dest, typeEffect, element)
		end
	else
		-- source and destination are different (standard case)
		if (did_crit==true) then
			-- crit
			if source then
				Recap_MakeOutgoingDetail(source, typeEffect)
				recap.Combatant[source].OutgoingDetail[typeEffect].CritsEvents = (recap.Combatant[source].OutgoingDetail[typeEffect].CritsEvents or 0) + 1
				recap.Combatant[source].OutgoingDetail[typeEffect].Crits = (recap.Combatant[source].OutgoingDetail[typeEffect].Crits or 0) + 1
				recap.Combatant[source].OutgoingDetail[typeEffect].CritsDmg = (recap.Combatant[source].OutgoingDetail[typeEffect].CritsDmg or 0) + total
				recap.Combatant[source].OutgoingDetail[typeEffect].CritsMax = math.max(total,recap.Combatant[source].OutgoingDetail[typeEffect].CritsMax or 0)
				Recap_AddOutgoingElement(source, typeEffect, element)
				if selfIndex then
					Recap_MakeSelfDetail(selfIndex, typeEffect)
					recap.Self[selfIndex][typeEffect].CritsEvents = (recap.Self[selfIndex][typeEffect].CritsEvents or 0) + 1
					recap.Self[selfIndex][typeEffect].Crits = (recap.Self[selfIndex][typeEffect].Crits or 0) + 1
					recap.Self[selfIndex][typeEffect].CritsDmg = (recap.Self[selfIndex][typeEffect].CritsDmg or 0) + total
					recap.Self[selfIndex][typeEffect].CritsMax = math.max(total,recap.Self[selfIndex][typeEffect].CritsMax or 0)
					Recap_AddSelfElement(selfIndex, typeEffect, element)
				end
			end
			if recap.Combatant[dest] then
				Recap_MakeIncomingDetail(dest, typeEffect)
				recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents = (recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents or 0) + 1
				recap.Combatant[dest].IncomingDetail[typeEffect].Crits = (recap.Combatant[dest].IncomingDetail[typeEffect].Crits or 0) + 1
				recap.Combatant[dest].IncomingDetail[typeEffect].CritsDmg = (recap.Combatant[dest].IncomingDetail[typeEffect].CritsDmg or 0) + total
				recap.Combatant[dest].IncomingDetail[typeEffect].CritsMax = math.max(total,recap.Combatant[dest].IncomingDetail[typeEffect].CritsMax or 0)
				Recap_AddIncomingElement(dest, typeEffect, element)
			end
		elseif (did_crush==true) then
			-- crush
			if source then
				Recap_MakeOutgoingDetail(source, typeEffect)
				recap.Combatant[source].OutgoingDetail[typeEffect].CritsEvents = (recap.Combatant[source].OutgoingDetail[typeEffect].CritsEvents or 0) + 1
				recap.Combatant[source].OutgoingDetail[typeEffect].Crushes = (recap.Combatant[source].OutgoingDetail[typeEffect].Crushes or 0) + 1
				recap.Combatant[source].OutgoingDetail[typeEffect].CrushDmg = (recap.Combatant[source].OutgoingDetail[typeEffect].CrushDmg or 0) + total
				recap.Combatant[source].OutgoingDetail[typeEffect].CrushMax = math.max(total,recap.Combatant[source].OutgoingDetail[typeEffect].CrushMax or 0)
				Recap_AddOutgoingElement(source, typeEffect, element)
				-- self and pets can never crush
			end
			if recap.Combatant[dest] then
				Recap_MakeIncomingDetail(dest, typeEffect)
				recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents = (recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents or 0) + 1
				recap.Combatant[dest].IncomingDetail[typeEffect].Crushes = (recap.Combatant[dest].IncomingDetail[typeEffect].Crushes or 0) + 1
				recap.Combatant[dest].IncomingDetail[typeEffect].CrushDmg = (recap.Combatant[dest].IncomingDetail[typeEffect].CrushDmg or 0) + total
				recap.Combatant[dest].IncomingDetail[typeEffect].CrushMax = math.max(total,recap.Combatant[dest].IncomingDetail[typeEffect].CrushMax or 0)
				Recap_AddIncomingElement(dest, typeEffect, element)
			end
		elseif (did_crit==false) and (did_crush==false) then
			-- hit that could have critted (does not include misses, so that crit percentages are accurate)
			if source then
				Recap_MakeOutgoingDetail(source, typeEffect)
				recap.Combatant[source].OutgoingDetail[typeEffect].CritsEvents = (recap.Combatant[source].OutgoingDetail[typeEffect].CritsEvents or 0) + 1
				recap.Combatant[source].OutgoingDetail[typeEffect].Hits = (recap.Combatant[source].OutgoingDetail[typeEffect].Hits or 0) + 1
				recap.Combatant[source].OutgoingDetail[typeEffect].HitsDmg = (recap.Combatant[source].OutgoingDetail[typeEffect].HitsDmg or 0) + total
				recap.Combatant[source].OutgoingDetail[typeEffect].HitsMax = math.max(total,recap.Combatant[source].OutgoingDetail[typeEffect].HitsMax or 0)
				Recap_AddOutgoingElement(source, typeEffect, element)
				if selfIndex then
					Recap_MakeSelfDetail(selfIndex, typeEffect)
					recap.Self[selfIndex][typeEffect].CritsEvents = (recap.Self[selfIndex][typeEffect].CritsEvents or 0) + 1
					recap.Self[selfIndex][typeEffect].Hits = (recap.Self[selfIndex][typeEffect].Hits or 0) + 1
					recap.Self[selfIndex][typeEffect].HitsDmg = (recap.Self[selfIndex][typeEffect].HitsDmg or 0) + total
					recap.Self[selfIndex][typeEffect].HitsMax = math.max(total,recap.Self[selfIndex][typeEffect].HitsMax or 0)
					Recap_AddSelfElement(selfIndex, typeEffect, element)
				end
			end
			if recap.Combatant[dest] then
				Recap_MakeIncomingDetail(dest, typeEffect)
				recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents = (recap.Combatant[dest].IncomingDetail[typeEffect].CritsEvents or 0) + 1
				recap.Combatant[dest].IncomingDetail[typeEffect].Hits = (recap.Combatant[dest].IncomingDetail[typeEffect].Hits or 0) + 1
				recap.Combatant[dest].IncomingDetail[typeEffect].HitsDmg = (recap.Combatant[dest].IncomingDetail[typeEffect].HitsDmg or 0) + total
				recap.Combatant[dest].IncomingDetail[typeEffect].HitsMax = math.max(total,recap.Combatant[dest].IncomingDetail[typeEffect].HitsMax or 0)
				Recap_AddIncomingElement(dest, typeEffect, element)
			end
		elseif (total==0) then
			-- miss
			if source then
				Recap_MakeOutgoingDetail(source, typeEffect)
				if (missType == "miss") then
					recap.Combatant[source].OutgoingDetail[typeEffect].Missed = (recap.Combatant[source].OutgoingDetail[typeEffect].Missed or 0) + 1
				elseif (missType == "absorb") then
					recap.Combatant[source].OutgoingDetail[typeEffect].Absorbed = (recap.Combatant[source].OutgoingDetail[typeEffect].Absorbed or 0) + 1
				elseif (missType == "dodge") then
					recap.Combatant[source].OutgoingDetail[typeEffect].Dodged = (recap.Combatant[source].OutgoingDetail[typeEffect].Dodged or 0) + 1
				elseif (missType == "parry") or (missType == "parr") then
					recap.Combatant[source].OutgoingDetail[typeEffect].Parried = (recap.Combatant[source].OutgoingDetail[typeEffect].Parried or 0) + 1
				elseif (missType == "block") then
					recap.Combatant[source].OutgoingDetail[typeEffect].Blocked = (recap.Combatant[source].OutgoingDetail[typeEffect].Blocked or 0) + 1
				elseif (missType == "deflect") then
					recap.Combatant[source].OutgoingDetail[typeEffect].Deflected = (recap.Combatant[source].OutgoingDetail[typeEffect].Deflected or 0) + 1
				elseif (missType == "evade") then
					recap.Combatant[source].OutgoingDetail[typeEffect].Evaded = (recap.Combatant[source].OutgoingDetail[typeEffect].Evaded or 0) + 1
				elseif (missType == "resist") then
					recap.Combatant[source].OutgoingDetail[typeEffect].Resisted = (recap.Combatant[source].OutgoingDetail[typeEffect].Resisted or 0) + 1
				elseif (missType == "reflect") then
					recap.Combatant[source].OutgoingDetail[typeEffect].Reflected = (recap.Combatant[source].OutgoingDetail[typeEffect].Reflected or 0) + 1
				elseif (missType == "immune") then
					recap.Combatant[source].OutgoingDetail[typeEffect].Immune = (recap.Combatant[source].OutgoingDetail[typeEffect].Immune or 0) + 1
				end
				Recap_AddOutgoingElement(source, typeEffect, element)
				if selfIndex then
					Recap_MakeSelfDetail(selfIndex, typeEffect)
					if (missType == "miss") then
						recap.Self[selfIndex][typeEffect].Missed = (recap.Self[selfIndex][typeEffect].Missed or 0) + 1
					elseif (missType == "absorb") then
						recap.Self[selfIndex][typeEffect].Absorbed = (recap.Self[selfIndex][typeEffect].Absorbed or 0) + 1
					elseif (missType == "dodge") then
						recap.Self[selfIndex][typeEffect].Dodged = (recap.Self[selfIndex][typeEffect].Dodged or 0) + 1
					elseif (missType == "parry") or (missType == "parr") then
						recap.Self[selfIndex][typeEffect].Parried = (recap.Self[selfIndex][typeEffect].Parried or 0) + 1
					elseif (missType == "block") then
						recap.Self[selfIndex][typeEffect].Blocked = (recap.Self[selfIndex][typeEffect].Blocked or 0) + 1
					elseif (missType == "deflect") then
						recap.Self[selfIndex][typeEffect].Deflected = (recap.Self[selfIndex][typeEffect].Deflected or 0) + 1
					elseif (missType == "evade") then
						recap.Self[selfIndex][typeEffect].Evaded = (recap.Self[selfIndex][typeEffect].Evaded or 0) + 1
					elseif (missType == "resist") then
						recap.Self[selfIndex][typeEffect].Resisted = (recap.Self[selfIndex][typeEffect].Resisted or 0) + 1
					elseif (missType == "reflect") then
						recap.Self[selfIndex][typeEffect].Reflected = (recap.Self[selfIndex][typeEffect].Reflected or 0) + 1
					elseif (missType == "immune") then
						recap.Self[selfIndex][typeEffect].Immune = (recap.Self[selfIndex][typeEffect].Immune or 0) + 1
					end
					Recap_AddSelfElement(selfIndex, typeEffect, element)
				end
			end
			if recap.Combatant[dest] then
				Recap_MakeIncomingDetail(dest, typeEffect)
				if (missType == "miss") then
					recap.Combatant[dest].IncomingDetail[typeEffect].Missed = (recap.Combatant[dest].IncomingDetail[typeEffect].Missed or 0) + 1
				elseif (missType == "absorb") then
					recap.Combatant[dest].IncomingDetail[typeEffect].Absorbed = (recap.Combatant[dest].IncomingDetail[typeEffect].Absorbed or 0) + 1
				elseif (missType == "dodge") then
					recap.Combatant[dest].IncomingDetail[typeEffect].Dodged = (recap.Combatant[dest].IncomingDetail[typeEffect].Dodged or 0) + 1
				elseif (missType == "parry") or (missType == "parr") then
					recap.Combatant[dest].IncomingDetail[typeEffect].Parried = (recap.Combatant[dest].IncomingDetail[typeEffect].Parried or 0) + 1
				elseif (missType == "block") then
					recap.Combatant[dest].IncomingDetail[typeEffect].Blocked = (recap.Combatant[dest].IncomingDetail[typeEffect].Blocked or 0) + 1
				elseif (missType == "deflect") then
					recap.Combatant[dest].IncomingDetail[typeEffect].Deflected = (recap.Combatant[dest].IncomingDetail[typeEffect].Deflected or 0) + 1
				elseif (missType == "evade") then
					recap.Combatant[dest].IncomingDetail[typeEffect].Evaded = (recap.Combatant[dest].IncomingDetail[typeEffect].Evaded or 0) + 1
				elseif (missType == "resist") then
					recap.Combatant[dest].IncomingDetail[typeEffect].Resisted = (recap.Combatant[dest].IncomingDetail[typeEffect].Resisted or 0) + 1
				elseif (missType == "reflect") then
					recap.Combatant[dest].IncomingDetail[typeEffect].Reflected = (recap.Combatant[dest].IncomingDetail[typeEffect].Reflected or 0) + 1
				elseif (missType == "immune") then
					recap.Combatant[dest].IncomingDetail[typeEffect].Immune = (recap.Combatant[dest].IncomingDetail[typeEffect].Immune or 0) + 1
				end
				Recap_AddIncomingElement(dest, typeEffect, element)
			end
		else
			-- hit that couldn't have critted (must be a DoT or HoT)
			if source then
				Recap_MakeOutgoingDetail(source, typeEffect)
				recap.Combatant[source].OutgoingDetail[typeEffect].Ticks = (recap.Combatant[source].OutgoingDetail[typeEffect].Ticks or 0) + 1
				recap.Combatant[source].OutgoingDetail[typeEffect].TicksDmg = (recap.Combatant[source].OutgoingDetail[typeEffect].TicksDmg or 0) + total
				recap.Combatant[source].OutgoingDetail[typeEffect].TicksMax = math.max(total,recap.Combatant[source].OutgoingDetail[typeEffect].TicksMax or 0)
				Recap_AddOutgoingElement(source, typeEffect, element)
				if selfIndex then
					Recap_MakeSelfDetail(selfIndex, typeEffect)
					recap.Self[selfIndex][typeEffect].Ticks = (recap.Self[selfIndex][typeEffect].Ticks or 0) + 1
					recap.Self[selfIndex][typeEffect].TicksDmg = (recap.Self[selfIndex][typeEffect].TicksDmg or 0) + total
					recap.Self[selfIndex][typeEffect].TicksMax = math.max(total,recap.Self[selfIndex][typeEffect].TicksMax or 0)
					Recap_AddSelfElement(selfIndex, typeEffect, element)
				end
			end
			if recap.Combatant[dest] then
				Recap_MakeIncomingDetail(dest, typeEffect)
				recap.Combatant[dest].IncomingDetail[typeEffect].Ticks = (recap.Combatant[dest].IncomingDetail[typeEffect].Ticks or 0) + 1
				recap.Combatant[dest].IncomingDetail[typeEffect].TicksDmg = (recap.Combatant[dest].IncomingDetail[typeEffect].TicksDmg or 0) + total
				recap.Combatant[dest].IncomingDetail[typeEffect].TicksMax = math.max(total,recap.Combatant[dest].IncomingDetail[typeEffect].TicksMax or 0)
				Recap_AddIncomingElement(dest, typeEffect, element)
			end
		end
	end
end

-- six auxiliary functions for Recap_CreateEffect to create spots in the tables as necessary
function Recap_MakeIncomingDetail(dest, typeEffect)
	if not recap.Combatant[dest].IncomingDetail then
		recap.Combatant[dest].IncomingDetail = {}
	end
	if not recap.Combatant[dest].IncomingDetail[typeEffect] then
		recap.Combatant[dest].IncomingDetail[typeEffect] = {}
	end
end
function Recap_MakeOutgoingDetail(source, typeEffect)
	if not recap.Combatant[source].OutgoingDetail then
		recap.Combatant[source].OutgoingDetail = {}
	end
	if not recap.Combatant[source].OutgoingDetail[typeEffect] then
		recap.Combatant[source].OutgoingDetail[typeEffect] = {}
	end
end
function Recap_MakeSelfDetail(selfIndex, typeEffect)
	if not recap.Self[selfIndex] then
		recap.Self[selfIndex] = {}
	end
	if not recap.Self[selfIndex][typeEffect] then
		recap.Self[selfIndex][typeEffect] = {}
	end
end
function Recap_AddIncomingElement(dest, typeEffect, element)
	if not recap.Combatant[dest].IncomingDetail[typeEffect].Element then
		-- no element yet, set it
		recap.Combatant[dest].IncomingDetail[typeEffect].Element = element
	else
		-- if there is a new element that is not 'Other', check whether the new and old elements are the same
		if element and (element ~= recap_temp.Localize.ElementOther) then
			if not string.find(recap.Combatant[dest].IncomingDetail[typeEffect].Element, element) then
				-- new element not found in old element, append it
				-- for example, perhaps the combatant switched wands
				if (recap.Combatant[dest].IncomingDetail[typeEffect].Element == "?") or (recap.Combatant[dest].IncomingDetail[typeEffect].Element == recap_temp.Localize.ElementOther) then
					-- replace a "?" or "Other"
					recap.Combatant[dest].IncomingDetail[typeEffect].Element = element
				else
					-- append to anything else
					recap.Combatant[dest].IncomingDetail[typeEffect].Element = recap.Combatant[dest].IncomingDetail[typeEffect].Element.."+"..element
				end
			end
		end
	end
end
function Recap_AddOutgoingElement(source, typeEffect, element)
	if not recap.Combatant[source].OutgoingDetail[typeEffect].Element then
		-- no element yet, set it
		recap.Combatant[source].OutgoingDetail[typeEffect].Element = element
	else
		-- if there is a new element that is not 'Other', check whether the new and old elements are the same
		if element and (element ~= recap_temp.Localize.ElementOther) then
			if not string.find(recap.Combatant[source].OutgoingDetail[typeEffect].Element, element) then
				-- new element not found in old element, append it
				-- for example, perhaps the combatant switched wands
				if (recap.Combatant[source].OutgoingDetail[typeEffect].Element == "?") or (recap.Combatant[source].OutgoingDetail[typeEffect].Element == recap_temp.Localize.ElementOther) then
					-- replace a "?" or "Other"
					recap.Combatant[source].OutgoingDetail[typeEffect].Element = element
				else
					-- append to anything else
					recap.Combatant[source].OutgoingDetail[typeEffect].Element = recap.Combatant[source].OutgoingDetail[typeEffect].Element.."+"..element
				end
			end
		end
	end
end
function Recap_AddSelfElement(selfIndex, typeEffect, element)
	if not recap.Self[selfIndex][typeEffect].Element then
		-- no element yet, set it
		recap.Self[selfIndex][typeEffect].Element = element
	else
		-- if there is a new element that is not 'Other', check whether the new and old elements are the same
		if element and (element ~= recap_temp.Localize.ElementOther) then
			if not string.find(recap.Self[selfIndex][typeEffect].Element, element) then
				-- new element not found in old element, append it
				-- for example, perhaps the combatant switched wands
				if (recap.Self[selfIndex][typeEffect].Element == "?") or (recap.Self[selfIndex][typeEffect].Element == recap_temp.Localize.ElementOther) then
					-- replace a "?" or "Other"
					recap.Self[selfIndex][typeEffect].Element = element
				else
					-- append to anything else
					recap.Self[selfIndex][typeEffect].Element = recap.Self[selfIndex][typeEffect].Element.."+"..element
				end
			end
		end
	end
end


function Recap_DeathEvent(event, info)

	local j, victim, spot

	if recap.Opt.State.value~="Stopped" then

		if recap.dump then
			local printable
			printable = Recap_CreateMessage(event)
			Recap_DumpMessage("DEATH;"..printable)
		end

		victim = info.victim
		if tonumber(victim) == ParserLib_SELF then
			victim = recap_temp.Player
		end

		-- Hawksy: we often get two death events for a single death, one event with source and one without.
		-- Some testing indicates that the event without the source was more reliable, so we're using that one,
		--   but we will miss some deaths.
		if not (info.source) then
			if info.isItem then
				-- ignore items (such as totems) that are killed
			else
				-- other deaths
				-- check for FD
				if recap.Combatant[victim] and recap.Combatant[victim].Class=="Hunter" then
					if (UnitName("player"))==victim then
						spot = "player"
					end
					for j=1,4 do
						if (UnitName("party"..j))==victim then
							spot = "party"..j
							break
						end
					end
					for j=1,40 do
						if (UnitName("raid"..j))==victim then
							spot = "raid"..j
							break
						end
					end
					if spot then
						-- Hawksy: modified to handle an indefinite number of buffs, not just 16
						j = 1
						while (UnitBuff(spot, j)) do
							if (UnitBuff(spot, j) == "Interface\\Icons\\Ability_Rogue_FeignDeath") then
								victim = false
								break
							end
							j = j + 1
						end
					end
				end

				if victim and recap_temp.Last[victim] then
					recap_temp.Last[victim].Kills = recap_temp.Last[victim].Kills + 1
				end
			end
		end
	end
end


function Recap_HealEvent(event, info)

	local source, dest, rawheal, effect, element, actualheal, overheal
	local event_time = GetTime()

	if recap.Opt.State.value~="Stopped" then

		if recap.dump then
			local printable
			printable = Recap_CreateMessage(event)
			Recap_DumpMessage("HEALING;"..printable)
		end

		-- possibly start the fight
		if not recap.Opt.LimitFights.value and recap.Opt.State.value=="Idle" then
			Recap_StartFight()
		end

		-- if the fight is ongoing, start or restart the idle timer
		if recap.Opt.State.value=="Active" then
			if recap.Opt.IdleFight.value then
				recap_temp.IdleTimer = 0
			end
			RecapIdleFrame:Show() -- start idle OnUpdates
		end

		-- if the fight is ongoing and if the end of fight delay timer is running, restart the end of fight delay timer
		--   (the player left combat, perhaps temporarily, but the fight continues)
		if recap.Opt.State.value=="Active" and recap_temp.EndFightDelayTimer~=-1 then
			recap_temp.EndFightDelayTimer = 0
			RecapEndFightDelayFrame:Show() -- start end fight delay OnUpdates
		end

		source = info.source
		if tonumber(source) == ParserLib_SELF then
			source = recap_temp.Player
		end
		if tonumber(source) == ParserLib_NONE then
			-- healing with no source
			source = false
		end
		dest = info.victim
		if tonumber(dest) == ParserLib_SELF then
			dest = recap_temp.Player
		end
		-- Hawksy: there are still occasional appearances of the personal pronoun despite ParserLib
		--   (apparently only in the destination slot, but we check both)
		if source then
			if recap_temp.SelfPronouns[string.lower(source)] then
DEFAULT_CHAT_FRAME:AddMessage("Recap_HealEvent personal pronoun arg1 ("..tostring(arg1)..") event "..tostring(event).." source "..tostring(source))
if recap_temp.snap then
Screenshot()
end
				source = recap_temp.Player
			end
		end
		if recap_temp.SelfPronouns[string.lower(dest)] then
DEFAULT_CHAT_FRAME:AddMessage("Recap_HealEvent personal pronoun arg1 ("..tostring(arg1)..") event "..tostring(event).." dest "..tostring(dest))
if recap_temp.snap then
Screenshot()
end
			dest = recap_temp.Player
		end

		rawheal = 0
		rawheal = tonumber(info.amount)
		effect = info.skill
		if tonumber(effect) == ParserLib_MELEE then -- shouldn't happen for a heal, just being paranoid
			effect = RECAP_MELEE
		end
		if tonumber(effect) == ParserLib_DAMAGESHIELD then -- shouldn't happen for a heal, just being paranoid
			effect = recap_temp.Localize.DamageShield
		end

		-- default element is Healing
		element = recap_temp.Localize.ElementHealing
		if info.element then
			element = info.element
		end

		-- Hawksy: check for known duplicates
		if source then
			if Recap_CheckForDuplicateSource(source) then
				return
			end
		end
		if Recap_CheckForDuplicateEffect(effect) then
			return
		end

		-- Hawksy: now keeping an independent timer for healing, and setting the 'WasInCurrent' flag for the healer
		--   (receiving healing still gets no registration as a combatant)
		--   (note that this means that recipients of heals who do not yet have status will not get IncomingDetail entries)
		if source then
			Recap_CreateCombatantAndUpdateTimers(source, event_time, "Heal")
		end

		if rawheal and (rawheal > 0) then
			if recap_temp.Last[dest] and recap_temp.Last[dest].HP then
				-- reduce 'actualheal' if we know the hit points of the target
				actualheal = math.max(0,math.min(recap_temp.Last[dest].MaxHP-recap_temp.Last[dest].HP,rawheal))
				recap_temp.Last[dest].HP = recap_temp.Last[dest].HP + actualheal
				overheal = rawheal - actualheal
			else
				actualheal = rawheal
				overheal = 0
			end

			-- two hidden columns, gross healing (RawHeal) and overhealing (OverHeal)
			-- a calculated column, Heal, includes actual healing
			if source then
				recap_temp.Last[source].RawHeal = recap_temp.Last[source].RawHeal + rawheal
				recap_temp.Last[source].OverHeal = math.min((recap_temp.Last[source].OverHeal + overheal),recap_temp.Last[source].RawHeal)
			end

			if not recap.Opt.LightData.value then
				-- Hawksy: the effect is credited with the full value of the heal
				-- determine whether it could have critted
				if string.find(event,"PERIODIC") then
					-- periodic event, assume not crit-capable, pass 'nil' in the crit slot -- keep crit and crush as 'nil', not false
					Recap_CreateEffect(source, dest, effect, 3, rawheal, nil, info.isDOT, element, false, nil)
				else
					-- assume all others are crit capable
					local critty
					critty = info.isCrit or false
					Recap_CreateEffect(source, dest, effect, 3, rawheal, critty, info.isDOT, element, false, false)
				end

				if source then
					if overheal>0 and recap.Combatant[source].OutgoingDetail["3"..effect] then
						-- reuse .Missed detail to store overhealing since heals never miss
						recap.Combatant[source].OutgoingDetail["3"..effect].Missed = (recap.Combatant[source].OutgoingDetail["3"..effect].Missed or 0) + overheal
					elseif overheal>0 and recap.Combatant[source].OutgoingDetail["4"..effect] then
						recap.Combatant[source].OutgoingDetail["4"..effect].Missed = (recap.Combatant[source].OutgoingDetail["4"..effect].Missed or 0) + overheal
					end
				end
				if overheal>0 and recap.Combatant[dest].IncomingDetail["3"..effect] then
					-- reuse .Missed detail to store overhealing since heals never miss
					recap.Combatant[dest].IncomingDetail["3"..effect].Missed = (recap.Combatant[dest].IncomingDetail["3"..effect].Missed or 0) + overheal
				elseif overheal>0 and recap.Combatant[dest].IncomingDetail["4"..effect] then
					recap.Combatant[dest].IncomingDetail["4"..effect].Missed = (recap.Combatant[dest].IncomingDetail["4"..effect].Missed or 0) + overheal
				end
			end

			-- Hawksy: support for Recent Data Mode
			if recap.Opt.RecentData.value then
				Recap_AddRecentEvent(event_time, source, dest, 3, effect, element, rawheal, info.isCrit, false)
			end
		else
			-- Hawksy: no healing, can't be a miss, this is unexpected
			Recap_CheckUnexpectedEvent()
		end
	end
end


-- removes combatant i from last fight
function Recap_ResetLastFight(i)
	recap.Combatant[i].TotalTime = recap.Combatant[i].TotalTime - recap.Combatant[i].LastTime
	recap.Combatant[i].TotalTimeIn = recap.Combatant[i].TotalTimeIn - recap.Combatant[i].LastTimeIn
	recap.Combatant[i].TotalTimeHeal = recap.Combatant[i].TotalTimeHeal - recap.Combatant[i].LastTimeHeal
	recap.Combatant[i].TotalDmgIn = recap.Combatant[i].TotalDmgIn - recap.Combatant[i].LastDmgIn
	recap.Combatant[i].TotalDmgOut = recap.Combatant[i].TotalDmgOut - recap.Combatant[i].LastDmgOut
	recap.Combatant[i].TotalRawHeal = recap.Combatant[i].TotalRawHeal - recap.Combatant[i].LastRawHeal
	recap.Combatant[i].TotalOverHeal = math.min((recap.Combatant[i].TotalOverHeal - recap.Combatant[i].LastOverHeal),recap.Combatant[i].TotalRawHeal)
	recap.Combatant[i].TotalKills = recap.Combatant[i].TotalKills - recap.Combatant[i].LastKills
	if recap.Combatant[i].TotalTime > recap_temp.MinTime then
		recap.Combatant[i].TotalDPS = recap.Combatant[i].TotalDmgOut/recap.Combatant[i].TotalTime
	else
		recap.Combatant[i].TotalDPS = 0
	end
	if recap.Combatant[i].TotalTimeIn > recap_temp.MinTime then
		recap.Combatant[i].TotalDPSIn = recap.Combatant[i].TotalDmgIn/recap.Combatant[i].TotalTimeIn
	else
		recap.Combatant[i].TotalDPSIn = 0
	end
	if recap.Combatant[i].TotalTimeHeal > recap_temp.MinTime then
		recap.Combatant[i].TotalHPS = (recap.Combatant[i].TotalRawHeal - recap.Combatant[i].TotalOverHeal)/recap.Combatant[i].TotalTimeHeal
	else
		recap.Combatant[i].TotalHPS = 0
	end
	recap.Combatant[i].WasInLast = false
	recap.Combatant[i].LastTime = 0
	recap.Combatant[i].LastTimeIn = 0
	recap.Combatant[i].LastTimeHeal = 0
	recap.Combatant[i].LastDmgIn = 0
	recap.Combatant[i].LastDmgOut = 0
	recap.Combatant[i].LastDPS = 0
	recap.Combatant[i].LastDPSIn = 0
	recap.Combatant[i].LastHPS = 0
	recap.Combatant[i].LastMaxHit = 0
	recap.Combatant[i].LastKills = 0
	recap.Combatant[i].LastRawHeal = 0
	recap.Combatant[i].LastOverHeal = 0
end


-- Hawksy: two functions to support Recent Data Mode
function Recap_CreateRecent()

	local i

	recap_temp.Recent = {}
	i = 1
	while i <= recap.Opt.RecentEventCount.value do
		recap_temp.Recent[i] = {}
		i = i + 1
	end
	recap_temp.Recent.First = 1
	recap_temp.Recent.Last = 1

end

function Recap_AddRecentEvent(event_time, source, dest, effectType, effect, element, amount, crit, crush)

	local i

	-- play with the pointers
	if recap_temp.Recent[recap_temp.Recent.Last] and recap_temp.Recent[recap_temp.Recent.Last].T then
		-- events already stored, move Last pointer
		if recap_temp.Recent.Last >= recap.Opt.RecentEventCount.value then
			recap_temp.Recent.Last = 1
		else
			recap_temp.Recent.Last = recap_temp.Recent.Last + 1
		end
		if recap_temp.Recent.First == recap_temp.Recent.Last then
			-- pointers have met, move the First pointer out of the way
			if recap_temp.Recent.First >= recap.Opt.RecentEventCount.value then
				recap_temp.Recent.First = 1
			else
				recap_temp.Recent.First = recap_temp.Recent.First + 1
			end
		end
	else
		-- no event stored yet, store at Last
	end
	i = recap_temp.Recent.Last
	-- single character keys to keep memory cost down (T = time, S = source D = destination, E = effect, L = element, A = amount, C = crit, R = crush)
	recap_temp.Recent[i].T = event_time
	if source or dest then
		recap_temp.Recent[i].S = source
		recap_temp.Recent[i].D = dest
		recap_temp.Recent[i].E = tostring(effectType)..effect
		recap_temp.Recent[i].L = element
		recap_temp.Recent[i].A = amount
		recap_temp.Recent[i].C = crit
		recap_temp.Recent[i].R = crush
	else
		-- if source and dest are false, this represents an 'end of fight' event
		recap_temp.Recent[i].S = "**EOF**"
		recap_temp.Recent[i].D = nil -- keep as nil
		recap_temp.Recent[i].E = nil -- keep as nil
		recap_temp.Recent[i].L = nil -- keep as nil
		recap_temp.Recent[i].A = nil -- keep as nil
		recap_temp.Recent[i].C = nil -- keep as nil
		recap_temp.Recent[i].R = nil -- keep as nil
	end
end


-- Hawsky: new function to handle ParserLib events
function Recap_ParserLib_OnEvent(event, info)

	if (info.type == "death") then
		-- used to trigger only on CHAT_MSG_COMBAT_FRIENDLY_DEATH and CHAT_MSG_COMBAT_HOSTILE_DEATH
		-- now includes deaths such as "Boom Bot is killed by Suicide.", "Gan'arg Sapper is killed by Power Burn.",
		--    "Gurubashi Bat Rider is killed by Unstable Concoction.", and "Cattneth is killed by Demonic Sacrifice."
		Recap_DeathEvent(event, info)
	elseif (info.type == "hit") or (info.type == "miss") then
		Recap_DamageEvent(event, info)
	elseif (info.type == "heal") then
		Recap_HealEvent(event, info)
	else
		-- info.type currently ignored: extraattack, leech, fade, drain, debuff, gain, environment, interrupt, fail, buff, cast, dispel

		-- info.type="extraattack" -- a proc, with the actual extra attack(s) occurring separately -- e.g. shaman's windfury totem gave Galassa one extra attack

		-- info currently handled
		-- source, victim, skill, element, amount, isCrit, isDOT, missType, (isItem for death), isCrushing

		-- info currently ignored
		-- isGlancing, amountAbsorb, amountBlock, amountResist, amountVulnerable (hit)
		-- isSplit (?)
		-- amountRank (for buffs and debuffs)
		-- damageType (environmental)
		-- isBegin, isPerform (for cast)
		-- attribute (for gain, drain, leech)
		-- sourceGained, amountGained, attributeGained (for leech)
		-- sourceSkill, isFailed (for dispel)
		-- reason (for fail)

		-- Hawksy: events that could be relevant that I'm not yet requesting are
		-- CHAT_MSG_SPELL_AURA_GONE_OTHER, CHAT_MSG_SPELL_AURA_GONE_PARTY, CHAT_MSG_SPELL_AURA_GONE_SELF, CHAT_MSG_SPELL_BREAK_AURA
	end
end

-- Hawksy: moved some events to Recap_ParserLib_OnEvent
function Recap_Combat_OnEvent()

	local source, damageEffect, healEffect, event_time

	if event=="UPDATE_MOUSEOVER_UNIT" then
		Recap_GetUnitInfo("mouseover")
	elseif event=="PLAYER_TARGET_CHANGED" then
		Recap_GetUnitInfo("target")

	-- Hawksy: extensively rewritten
	elseif (event=="UNIT_SPELLCAST_SUCCEEDED") and recap.Opt.State.value~="Stopped" then
		-- this event is said to occur for anyone in the party or raid, and perhaps also the combatant's target.
		--   does not report the target for the spell (only UNIT_SPELLCAST_SENT does that, and it occurs also for failed casts)
		--   includes a lot of extra noise, such as pet attacks, dismounting, adding an Argent Dawn commission, etc.
		--   (2007-01-05: not sure it is catching everything, need to check into it, currently not checking for whether pets get this event)
		source = (UnitName(arg1))
		if source and recap.Combatant[source] then
			-- we have data for this combatant
			damageEffect = "1"..arg2
			healEffect = "3"..arg2
			if recap.Combatant[source].OutgoingDetail[damageEffect] or recap.Combatant[source].OutgoingDetail[healEffect] then
				-- this is a cast with which this combatant has done damage or healing before
				if recap.dump then
					Recap_DumpMessage("CAST;"..event..";"..source..";"..string.gsub(arg2,"\ ","\_"))
				end

				event_time = GetTime()

				-- possibly start the fight
				if not recap.Opt.LimitFights.value and recap.Opt.State.value=="Idle" then
					Recap_StartFight()
				end

				-- if the fight is ongoing, start or restart the idle timer
				if recap.Opt.State.value=="Active" then
					if recap.Opt.IdleFight.value then
						recap_temp.IdleTimer = 0
					end
					RecapIdleFrame:Show() -- start idle OnUpdates
				end

				-- if the fight is ongoing and if the end of fight delay timer is running, restart the end of fight delay timer
				--   (the player left combat, perhaps temporarily, but the fight continues)
				if recap.Opt.State.value=="Active" and recap_temp.EndFightDelayTimer~=-1 then
					recap_temp.EndFightDelayTimer = 0
					RecapEndFightDelayFrame:Show() -- start end fight delay OnUpdates
				end

				-- perhaps add the combatant into the fight, and update timers
				-- this takes the time from the moment of successful cast, rather than when it strikes the target
				if recap.Combatant[source].OutgoingDetail[damageEffect] then
					Recap_CreateCombatantAndUpdateTimers(source, event_time, "Out")
					if recap.Combatant[source].OutgoingDetail[damageEffect].HitsDmg then
						-- this has a direct damage component, so it is already being counted; do nothing
					else
						if recap.Combatant[source].OutgoingDetail[damageEffect].TicksDmg then
							-- this is the casting of a DoT (with no direct damage component), so count it as a hit
							recap.Combatant[source].OutgoingDetail[damageEffect].Hits = (recap.Combatant[source].OutgoingDetail[damageEffect].Hits or 0) + 1
							recap.Combatant[source].OutgoingDetail[damageEffect].CritsEvents = (recap.Combatant[source].OutgoingDetail[damageEffect].CritsEvents or 0) + 1
							-- is the source the player or one of the player's pets?
							local selfIndex = nil -- keep as nil
							if source then
								if source == recap_temp.Player then
									selfIndex = recap_temp.s
								elseif recap.Combatant[source].OwnedBy == recap_temp.Player then
									selfIndex = recap_temp.s..":"..source
								end
							end
							if selfIndex then
								Recap_MakeSelfDetail(selfIndex, damageEffect)
								recap.Self[selfIndex][damageEffect].Hits = (recap.Self[selfIndex][damageEffect].Hits or 0) + 1
								recap.Self[selfIndex][damageEffect].CritsEvents = (recap.Self[selfIndex][damageEffect].CritsEvents or 0) + 1
							end

							-- Hawksy: support for Recent Data Mode
							if recap.Opt.RecentData.value then
								Recap_AddRecentEvent(event_time, source, "?", 1, arg2, false, "cast", false, false)
							end
						end
					end
				end

				-- for heals, just update the timers
				-- a standard heal will get counted as it lands, and a HoT only counts ticks
				if recap.Combatant[source].OutgoingDetail[healEffect] then
					Recap_CreateCombatantAndUpdateTimers(source, event_time, "Heal")
				end
			end
		end
	end
end

function Recap_Register_CombatEvents()

	-- Hawksy: get pointer to ParserLib
	recap_temp.parserlib = ParserLib:GetInstance("1.1")

	if not recap_temp.CombatEventsRegistered then
	-- register damage events
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_PARTY_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_PARTY_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_FRIENDLYPLAYER_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_FRIENDLYPLAYER_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PARTY_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PARTY_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PET_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PET_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_SELF_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_SELF_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_PARTY_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_DAMAGESHIELDS_ON_OTHERS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_DAMAGESHIELDS_ON_SELF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PARTY_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PET_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_SELF_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_PARTY_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE", "Recap_ParserLib_OnEvent")

		-- Hawksy: added event
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_FAILED_LOCALPLAYER", "Recap_ParserLib_OnEvent")

		-- register death events (these aren't the only two, actually -- see ParserLib for the others)
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_FRIENDLY_DEATH", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_HOSTILE_DEATH", "Recap_ParserLib_OnEvent")

		-- register healing events
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_CREATURE_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_PARTY_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_SELF_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PARTY_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PET_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_SELF_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS", "Recap_ParserLib_OnEvent")

		RecapCombatEvents:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
		RecapCombatEvents:RegisterEvent("PLAYER_TARGET_CHANGED")
		-- Hawksy: change of event
		RecapCombatEvents:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")

		-- register for receiving messages on sync channel
		Recap:RegisterComm(Recap.commPrefix, "GROUP", "OnCommReceive")

		recap_temp.CombatEventsRegistered = 1
	end
end

function Recap_Unregister_CombatEvents()
	if recap_temp.CombatEventsRegistered then
		RecapCombatEvents:UnregisterAllEvents()
		recap_temp.parserlib:UnregisterAllEvents("Recap")
		Recap:UnregisterComm(Recap.commPrefix, "GROUP")
		recap_temp.CombatEventsRegistered = false
	end
end


RecapCombat_lua_367 = true
