--[[ RecapPanel.lua ]]

--[[ local tables ]]

local dockoffset = { ["TOPRIGHTTOPLEFT"] = { x=-4,y=0 },
					 ["BOTTOMRIGHTBOTTOMLEFT"] = { x=-4,y=0 },
					 ["TOPLEFTTOPRIGHT"] = { x=4,y=0 },
					 ["BOTTOMLEFTBOTTOMRIGHT"] = { x=4,y=0 },
					 ["TOPRIGHTBOTTOMRIGHT"] = { x=0,y=-4 },
					 ["BOTTOMRIGHTTOPRIGHT"] = { x=0,y=4 },
					 ["TOPLEFTBOTTOMLEFT"] = { x=0,y=-4 },
					 ["BOTTOMLEFTTOPLEFT"] = { x=0,y=4 } }


--[[ Functions for the popup panel ]]

function RecapPanel_Populate(arg1)
	-- Hawksy: recap.Opt.IncomingPanelDetail.value and recap.Opt.OutgoingPanelDetail.value contain the do-not-translate titles
	--   We need to convert them when creating header lines

	local i,totalmiss,text,found

	if recap.Combatant[arg1] and not recap.Opt.LightData.value then

		--[[ Populate Panel 1: Incoming details ]]

		recap_temp.IncomingDetailSelected = 0
		RecapPanel_ConstructIncomingDetails(arg1)
		if recap_temp.IncomingDetailsListSize>1 then
			RecapPanel_PopulateIncomingDetails(arg1,1)
		else
			RecapPanel_PopulateIncomingDetails(arg1,0)
		end
		RecapPanelIncomingTotalLabel:SetText(recap_temp.Localize.DetailTitle[recap.Opt.IncomingPanelDetail.value]) --  or RECAP_MISS_OVER
		RecapPanelIncomingDetailsScrollBar_Update()

		--[[ Populate Panel 2: Outgoing details ]]

		recap_temp.OutgoingDetailSelected = 0
		RecapPanel_ConstructOutgoingDetails(arg1)
		if recap_temp.OutgoingDetailsListSize>1 then
			RecapPanel_PopulateOutgoingDetails(arg1,1)
		else
			RecapPanel_PopulateOutgoingDetails(arg1,0)
		end
		RecapPanelOutgoingTotalLabel:SetText(recap_temp.Localize.DetailTitle[recap.Opt.OutgoingPanelDetail.value]) --  or RECAP_MISS_OVER
		RecapPanelOutgoingDetailsScrollBar_Update()

	end

	-- populate elements that work in and out of light data mode

	if recap.Combatant[arg1] then

		--[[ Populate common elements ]]

		if recap.Combatant[arg1].Faction and recap_temp.FactionIcons[recap.Combatant[arg1].Faction] then
			RecapPanelFaction:SetTexture(recap_temp.FactionIcons[recap.Combatant[arg1].Faction])
		else
			RecapPanelFaction:SetTexture("")
		end

		if recap.Combatant[arg1].Level==-1 then
			RecapPanelLevel:SetText("??")
		elseif tonumber(recap.Combatant[arg1].Level) and tonumber(recap.Combatant[arg1].Level)>0 then
			RecapPanelLevel:SetText(recap.Combatant[arg1].Level)
		else
			RecapPanelLevel:SetText(" ")
		end

		if recap.Combatant[arg1].Class and recap_temp.ClassIcons[recap.Combatant[arg1].Class] then
			RecapPanelClass:SetTexCoord(recap_temp.ClassIcons[recap.Combatant[arg1].Class].left,recap_temp.ClassIcons[recap.Combatant[arg1].Class].right,recap_temp.ClassIcons[recap.Combatant[arg1].Class].top,recap_temp.ClassIcons[recap.Combatant[arg1].Class].bottom)
		else
			RecapPanelClass:SetTexCoord(.9,1,.9,1)
		end

		if recap.Combatant[arg1].Friend then
			RecapPanelName:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
		else
			RecapPanelName:SetTextColor(1,1,1)
		end
		RecapPanelName:SetText(arg1)

		--[[ Populate Panel 3: Summary ]]

		RecapPanelLastTimeText:SetText(Recap_FormatTime(recap.Combatant[arg1].LastTime))
		RecapPanelLastTimeInText:SetText(Recap_FormatTime(recap.Combatant[arg1].LastTimeIn))
		RecapPanelLastTimeHealText:SetText(Recap_FormatTime(recap.Combatant[arg1].LastTimeHeal))
		RecapPanelLastMaxText:SetText(recap.Combatant[arg1].LastMaxHit)
		RecapPanelLastDeathsText:SetText(recap.Combatant[arg1].LastKills)
		RecapPanelLastHealsText:SetText((recap.Combatant[arg1].LastRawHeal or 0)-(recap.Combatant[arg1].LastOverHeal or 0))
		RecapPanelLastHPSText:SetText(string.format("%.1f",recap.Combatant[arg1].LastHPS))
		RecapPanelLastDmgInText:SetText(recap.Combatant[arg1].LastDmgIn)
		RecapPanelLastDPSInText:SetText(string.format("%.1f",recap.Combatant[arg1].LastDPSIn))
		RecapPanelLastDmgOutText:SetText(recap.Combatant[arg1].LastDmgOut)
		RecapPanelLastDPSText:SetText(string.format("%.1f",recap.Combatant[arg1].LastDPS))

		RecapPanelAllTimeText:SetText(Recap_FormatTime(recap.Combatant[arg1].TotalTime))
		RecapPanelAllTimeInText:SetText(Recap_FormatTime(recap.Combatant[arg1].TotalTimeIn))
		RecapPanelAllTimeHealText:SetText(Recap_FormatTime(recap.Combatant[arg1].TotalTimeHeal))
		RecapPanelAllMaxText:SetText(recap.Combatant[arg1].TotalMaxHit)
		RecapPanelAllDeathsText:SetText(recap.Combatant[arg1].TotalKills)
		RecapPanelAllHealsText:SetText(recap.Combatant[arg1].TotalRawHeal-recap.Combatant[arg1].TotalOverHeal)
		RecapPanelAllHPSText:SetText(string.format("%.1f",recap.Combatant[arg1].TotalHPS))
		RecapPanelAllDmgInText:SetText(recap.Combatant[arg1].TotalDmgIn)
		RecapPanelAllDPSInText:SetText(string.format("%.1f",recap.Combatant[arg1].TotalDPSIn))
		RecapPanelAllDmgOutText:SetText(recap.Combatant[arg1].TotalDmgOut)
		RecapPanelAllDPSText:SetText(string.format("%.1f",recap.Combatant[arg1].TotalDPS))

		if recap.Combatant[arg1].Friend then
			RecapPanelLastTimeText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			RecapPanelAllTimeText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			RecapPanelLastTimeInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelAllTimeInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelLastTimeHealText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelAllTimeHealText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelLastHealsText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelAllHealsText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelLastHPSText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelAllHPSText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelLastDmgInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelAllDmgInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelLastDPSInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelAllDPSInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelLastDmgOutText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			RecapPanelAllDmgOutText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			RecapPanelLastDPSText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			RecapPanelAllDPSText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
		else
			RecapPanelLastTimeText:SetTextColor(1,1,1)
			RecapPanelAllTimeText:SetTextColor(1,1,1)
			RecapPanelLastTimeInText:SetTextColor(1,1,1)
			RecapPanelAllTimeInText:SetTextColor(1,1,1)
			RecapPanelLastTimeHealText:SetTextColor(1,1,1)
			RecapPanelAllTimeHealText:SetTextColor(1,1,1)
			RecapPanelLastHealsText:SetTextColor(1,1,1)
			RecapPanelAllHealsText:SetTextColor(1,1,1)
			RecapPanelLastHPSText:SetTextColor(1,1,1)
			RecapPanelAllHPSText:SetTextColor(1,1,1)
			RecapPanelLastDmgInText:SetTextColor(1,1,1)
			RecapPanelAllDmgInText:SetTextColor(1,1,1)
			RecapPanelLastDPSInText:SetTextColor(1,1,1)
			RecapPanelAllDPSInText:SetTextColor(1,1,1)
			RecapPanelLastDmgOutText:SetTextColor(1,1,1)
			RecapPanelAllDmgOutText:SetTextColor(1,1,1)
			RecapPanelLastDPSText:SetTextColor(1,1,1)
			RecapPanelAllDPSText:SetTextColor(1,1,1)
		end

	end

end

-- tooltips for individual entries
function RecapPanel_Entry_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	id = this:GetID()

	if id and id>0 then
		if this:GetName()=="RecapPanelOutgoingMissP" and RecapPanelOutgoingMissPLabel:GetText()==RECAP_OVERHEALING then
			Recap_OnTooltip("PanelEntryOverheal",getglobal(this:GetName().."Text"):GetText())
		else
			Recap_OnTooltip("PanelEntry"..id,getglobal(this:GetName().."Text"):GetText())
		end
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

-- clicks on individual fields (the 'magic numbers' here are messy)
function RecapPanel_Entry_OnClick()

	local id, header, text
	local i, j, k

	id = this:GetID()
	if id and id>0 and getglobal(this:GetName().."Text"):GetText()~=" " and getglobal(this:GetName().."Text"):GetText()~="--" and getglobal(this:GetName().."Text"):GetText()~="?" then
		header = Recap_GetTooltip("PanelEntry"..id)
		if header and IsShiftKeyDown() and recap_temp.Selected~=0 then
			if (id>=70 and id<=79) then
				-- an Outgoing miss, post all miss entries
				local noData = true
				text = "Recap: "..recap_temp.List[recap_temp.Selected].Name..": "..RECAP_MISSES
				text = text.." "..recap_temp.Localize.For.." "..RecapPanelOutgoingEffectText:GetText()..":"
				for j in pairs(recap_temp.Localize.MissTypes) do
					k = getglobal("RecapPanelOutgoingMiss"..j.."Text"):GetText()
					if (k ~= "--") and (k ~= "0 (0.0%)") then
						text = text.." "..recap_temp.Localize.MissTypes[j].." "..k
						noData = false
					end
				end
				if noData then
					text = text.." 0"
				end
			elseif (id>=55 and id<=64) then
				-- an Incoming miss, post all miss entries not just one
				local noData = true
				text = "Recap: "..recap_temp.List[recap_temp.Selected].Name..": "..RECAP_INCOMING.." "..RECAP_MISSES
				text = text.." "..recap_temp.Localize.For.." "..RecapPanelIncomingEffectText:GetText()..":"
				for j in pairs(recap_temp.Localize.MissTypes) do
					k = getglobal("RecapPanelIncomingMiss"..j.."Text"):GetText()
					if (k ~= "--") and (k ~= "0 (0.0%)") then
						text = text.." "..recap_temp.Localize.MissTypes[j].." "..k
						noData = false
					end
				end
				if noData then
					text = text.." 0"
				end
			elseif ((id>=25 and id<=42) or (id>=80 and id<=83)) then
				text = "Recap: "..recap_temp.List[recap_temp.Selected].Name..": "..header
				text = text..": "..getglobal(this:GetName().."Text"):GetText()
			else
				if ((this:GetName()=="RecapPanelOutgoingMissP" and RecapPanelOutgoingMissPLabel:GetText()==RECAP_OVERHEALING) or
					(this:GetName()=="RecapPanelIncomingMissP" and RecapPanelIncomingMissPLabel:GetText()==RECAP_OVERHEALING)) then
					header = RECAP_OVERHEALING
				end
				text = ""
				-- the magic numbers are for PanelEntry..id
				if ((id>=45 and id<=54) or (id>=65 and id<=67) or (id>=90 and id<=93)) then
					-- I think the 'Outgoing' and 'Incoming' trigger off the xml names, so don't need localization
					if string.find(this:GetName(),"Outgoing") and recap_temp.OutgoingDetailsListSize>1 then
						text = "Recap: "..recap_temp.List[recap_temp.Selected].Name..": "..header
						text = text.." "..recap_temp.Localize.For.." "..RecapPanelOutgoingEffectText:GetText()
					elseif string.find(this:GetName(),"Incoming") and recap_temp.IncomingDetailsListSize>1 then
						text = "Recap: "..recap_temp.List[recap_temp.Selected].Name..": "..RECAP_INCOMING.." "..header
						text = text.." "..recap_temp.Localize.For.." "..RecapPanelIncomingEffectText:GetText()
					end
				end
				text = text..": "..getglobal(this:GetName().."Text"):GetText()
			end
			Recap_InsertChat(text)
		end
	end
end

function RecapPanel_Show(name)

	if recap.Opt.ShowPanel.value then

		if recap_temp.Selected~=0 then
			name = recap_temp.List[recap_temp.Selected].Name
		end

		RecapPanel_Populate(name)
		RecapPanel:Show()
	end
end

function RecapPanel_Hide(arg1)

	if arg1 then
		recap_temp.Selected = 0
		recap_temp.OutgoingDetailSelected = 0
		recap_temp.IncomingDetailSelected = 0
		RecapScrollBar_Update()
	end

	if recap_temp.Selected==0 then
		RecapPanel:Hide()
	end
end

function RecapPanelTab_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local id = this:GetID()

	if id and id>0 then
		if recap.Opt.LightData.value and id<3 then
			Recap_OnTooltip("PanelTabDisabled"..id)
		else
			Recap_OnTooltip("PanelTab"..id)
		end
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapPanelTab_OnClick()

	local id = this:GetID()
	if recap.Opt.LightData.value then
		id = 3
	end
	recap.Opt.PanelView.value = id
	RecapPanel_SwitchPanels(id)
end

function RecapPanel_SwitchPanels(panel)

	local i

	for i=1,3 do
		getglobal("RecapPanelTab"..i):UnlockHighlight()
		getglobal("RecapSubPanel"..i):Hide()
	end
	getglobal("RecapPanelTab"..recap.Opt.PanelView.value):LockHighlight()
	getglobal("RecapSubPanel"..recap.Opt.PanelView.value):Show()
end

function RecapPanel_ConstructOutgoingDetails(name)

	local dmgtotal, healtotal, subtotal, i = 0,0
	local miss

	-- clear the details
	recap_temp.OutgoingDetailsListSize = 1
	recap_temp.OutgoingDetailsList = {}

	for i in pairs(recap.Combatant[name].OutgoingDetail) do
		if not recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize] then
			recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize] = {}
		end
		recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Effect = i
		recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Total = (recap.Combatant[name].OutgoingDetail[i].HitsDmg or 0) + (recap.Combatant[name].OutgoingDetail[i].CritsDmg or 0) + (recap.Combatant[name].OutgoingDetail[i].CrushDmg or 0) + (recap.Combatant[name].OutgoingDetail[i].TicksDmg or 0)
		recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Max = math.max(recap.Combatant[name].OutgoingDetail[i].TicksMax or 0,math.max(recap.Combatant[name].OutgoingDetail[i].HitsMax or 0,math.max(recap.Combatant[name].OutgoingDetail[i].CritsMax or 0,recap.Combatant[name].OutgoingDetail[i].CrushMax or 0)))
		subtotal = (recap.Combatant[name].OutgoingDetail[i].Hits or 0) + (recap.Combatant[name].OutgoingDetail[i].Crits or 0) + (recap.Combatant[name].OutgoingDetail[i].Crushes or 0) + (recap.Combatant[name].OutgoingDetail[i].Ticks or 0)
		recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Avg = string.format("%d",(subtotal>0 and ((((recap.Combatant[name].OutgoingDetail[i].HitsDmg or 0)+(recap.Combatant[name].OutgoingDetail[i].CritsDmg or 0)+(recap.Combatant[name].OutgoingDetail[i].CrushDmg or 0)+(recap.Combatant[name].OutgoingDetail[i].TicksDmg or 0))/subtotal)+0.5) or 0))

		-- misses or overhealing (and crit % calcs)
		recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Miss = " "
		if tonumber(string.sub(i,1,1))<3 then
			dmgtotal = dmgtotal + recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Total
			-- Hawksy: corrected
			miss = (recap.Combatant[name].OutgoingDetail[i].Missed or 0) + (recap.Combatant[name].OutgoingDetail[i].Blocked or 0) + (recap.Combatant[name].OutgoingDetail[i].Dodged or 0) + (recap.Combatant[name].OutgoingDetail[i].Parried or 0) +
					(recap.Combatant[name].OutgoingDetail[i].Deflected or 0) + (recap.Combatant[name].OutgoingDetail[i].Resisted or 0) + (recap.Combatant[name].OutgoingDetail[i].Reflected or 0) + (recap.Combatant[name].OutgoingDetail[i].Absorbed or 0) +
					(recap.Combatant[name].OutgoingDetail[i].Immune or 0) + (recap.Combatant[name].OutgoingDetail[i].Evaded or 0)
			subtotal = miss + (recap.Combatant[name].OutgoingDetail[i].CritsEvents or 0)
			if (subtotal > 0) then
				recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Crits = string.format("%.1f%%",100*(recap.Combatant[name].OutgoingDetail[i].Crits or 0)/subtotal)
				recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Miss = string.format("%.1f%%",100*miss/subtotal)
			else
				recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Crits = "--"
			end
		else
			-- crits for heals
			subtotal = recap.Combatant[name].OutgoingDetail[i].CritsEvents or 0
			if subtotal>0 then
				recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Crits = string.format("%.1f%%",100*(recap.Combatant[name].OutgoingDetail[i].Crits or 0)/subtotal)
			else
				recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Crits = "--"
			end
			-- overhealing (stored in the miss slot) calcs
			healtotal = healtotal + recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Total
			-- Hawksy: corrected
			subtotal = (recap.Combatant[name].OutgoingDetail[i].HitsDmg or 0) + (recap.Combatant[name].OutgoingDetail[i].CritsDmg or 0) + (recap.Combatant[name].OutgoingDetail[i].CrushDmg or 0) + (recap.Combatant[name].OutgoingDetail[i].Missed or 0) + (recap.Combatant[name].OutgoingDetail[i].TicksDmg or 0)
			if (subtotal > 0) then
				recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Miss = string.format("%.1f%%",100*(recap.Combatant[name].OutgoingDetail[i].Missed or 0)/subtotal)
			end
		end

		-- add element to the list, though it's not displayed at the moment
		if recap.Combatant[name].OutgoingDetail[i].Element then
			recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Element = recap.Combatant[name].OutgoingDetail[i].Element
		else
			recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize].Element = "?"
		end

		recap_temp.OutgoingDetailsListSize = recap_temp.OutgoingDetailsListSize + 1
	end
	recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize] = nil -- keep as nil

	for i=1,recap_temp.OutgoingDetailsListSize-1 do
		if tonumber(string.sub(recap_temp.OutgoingDetailsList[i].Effect,1,1))<3 then
			subtotal = dmgtotal
		else
			subtotal = healtotal
		end
		if subtotal>0 then
			recap_temp.OutgoingDetailsList[i].TotalP = math.floor((100*recap_temp.OutgoingDetailsList[i].Total)/subtotal+.5)
		else
			recap_temp.OutgoingDetailsList[i].TotalP = 0
		end
	end

	table.sort(recap_temp.OutgoingDetailsList,RecapPanel_DetailsDamageSort)

end

function RecapPanel_ConstructIncomingDetails(name)

	local dmgtotal, healtotal, subtotal, i = 0,0
	local miss

	-- clear the details
	recap_temp.IncomingDetailsListSize = 1
	recap_temp.IncomingDetailsList = {}

	for i in pairs(recap.Combatant[name].IncomingDetail) do
		if not recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize] then
			recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize] = {}
		end
		recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Effect = i
		recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Total = (recap.Combatant[name].IncomingDetail[i].HitsDmg or 0) + (recap.Combatant[name].IncomingDetail[i].CritsDmg or 0) + (recap.Combatant[name].IncomingDetail[i].CrushDmg or 0) + (recap.Combatant[name].IncomingDetail[i].TicksDmg or 0)
		recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Max = math.max(recap.Combatant[name].IncomingDetail[i].TicksMax or 0,math.max(recap.Combatant[name].IncomingDetail[i].HitsMax or 0,math.max(recap.Combatant[name].IncomingDetail[i].CritsMax or 0,recap.Combatant[name].IncomingDetail[i].CrushMax or 0)))
		subtotal = (recap.Combatant[name].IncomingDetail[i].Hits or 0) + (recap.Combatant[name].IncomingDetail[i].Crits or 0) + (recap.Combatant[name].IncomingDetail[i].Crushes or 0) + (recap.Combatant[name].IncomingDetail[i].Ticks or 0)
		recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Avg = string.format("%d",(subtotal>0 and ((((recap.Combatant[name].IncomingDetail[i].HitsDmg or 0)+(recap.Combatant[name].IncomingDetail[i].CritsDmg or 0)+(recap.Combatant[name].IncomingDetail[i].CrushDmg or 0)+(recap.Combatant[name].IncomingDetail[i].TicksDmg or 0))/subtotal)+0.5) or 0))

		-- misses or overhealing (and crit % calcs)
		recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Miss = " "
		if tonumber(string.sub(i,1,1))<3 then
			dmgtotal = dmgtotal + recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Total
			-- Hawksy: corrected
			miss = (recap.Combatant[name].IncomingDetail[i].Missed or 0) + (recap.Combatant[name].IncomingDetail[i].Blocked or 0) + (recap.Combatant[name].IncomingDetail[i].Dodged or 0) + (recap.Combatant[name].IncomingDetail[i].Parried or 0) +
					(recap.Combatant[name].IncomingDetail[i].Deflected or 0) + (recap.Combatant[name].IncomingDetail[i].Resisted or 0) + (recap.Combatant[name].IncomingDetail[i].Reflected or 0) + (recap.Combatant[name].IncomingDetail[i].Absorbed or 0) +
					(recap.Combatant[name].IncomingDetail[i].Immune or 0) + (recap.Combatant[name].IncomingDetail[i].Evaded or 0)
			subtotal = miss + (recap.Combatant[name].IncomingDetail[i].CritsEvents or 0)
			if (subtotal > 0) then
				recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Crits = string.format("%.1f%%",100*(recap.Combatant[name].IncomingDetail[i].Crits or 0)/subtotal)
				recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Miss = string.format("%.1f%%",100*miss/subtotal)
			else
				recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Crits = "--"
			end
		else
			-- crits for heals
			subtotal = recap.Combatant[name].IncomingDetail[i].CritsEvents or 0
			if subtotal>0 then
				recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Crits = string.format("%.1f%%",100*(recap.Combatant[name].IncomingDetail[i].Crits or 0)/subtotal)
			else
				recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Crits = "--"
			end
			-- overhealing (stored in the miss slot) calcs
			healtotal = healtotal + recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Total
			-- Hawksy: corrected
			subtotal = (recap.Combatant[name].IncomingDetail[i].HitsDmg or 0) + (recap.Combatant[name].IncomingDetail[i].CritsDmg or 0) + (recap.Combatant[name].IncomingDetail[i].CrushDmg or 0) + (recap.Combatant[name].IncomingDetail[i].Missed or 0) + (recap.Combatant[name].IncomingDetail[i].TicksDmg or 0)
			if (subtotal > 0) then
				recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Miss = string.format("%.1f%%",100*(recap.Combatant[name].IncomingDetail[i].Missed or 0)/subtotal)
			end
		end

		-- add element to the list, though it's not displayed at the moment
		if recap.Combatant[name].IncomingDetail[i].Element then
			recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Element = recap.Combatant[name].IncomingDetail[i].Element
		else
			recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize].Element = "?"
		end

		recap_temp.IncomingDetailsListSize = recap_temp.IncomingDetailsListSize + 1
	end
	recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize] = nil -- keep as nil

	for i=1,recap_temp.IncomingDetailsListSize-1 do
		if tonumber(string.sub(recap_temp.IncomingDetailsList[i].Effect,1,1))<3 then
			subtotal = dmgtotal
		else
			subtotal = healtotal
		end
		if subtotal>0 then
			recap_temp.IncomingDetailsList[i].TotalP = math.floor((100*recap_temp.IncomingDetailsList[i].Total)/subtotal+.5)
		else
			recap_temp.IncomingDetailsList[i].TotalP = 0
		end
	end

	table.sort(recap_temp.IncomingDetailsList,RecapPanel_DetailsDamageSort)

end

function RecapPanel_DetailsDamageSort(e1,e2)

	local effect1,effect2

	-- Hawksy: guard code
	if e1.Effect then
		effect1 = string.sub(e1.Effect,1,1)
	else
		effect1 = 0
	end
	if e2.Effect then
		effect2 = string.sub(e2.Effect,1,1)
	else
		effect2 = 0
	end
	if not e1.Total then
		e1.Total = 0
	end
	if not e2.Total then
		e2.Total = 0
	end

	if ((e1.Total>e2.Total) and (effect1==effect2)) or (effect1<effect2) then
		return true
	else
		return false
	end
end

function RecapPanelOutgoingDetailsScrollBar_Update()

	local i, index, item, r, g, b

	if recap_temp.Loaded and recap_temp.OutgoingDetailsListSize then

		FauxScrollFrame_Update(RecapPanelOutgoingDetailsScrollBar,recap_temp.OutgoingDetailsListSize-1,6,6)

		for i=1,6 do
			index = i + FauxScrollFrame_GetOffset(RecapPanelOutgoingDetailsScrollBar)
			if index < recap_temp.OutgoingDetailsListSize then
				if (string.sub(recap_temp.OutgoingDetailsList[index].Effect,1,1)=="3") or (string.sub(recap_temp.OutgoingDetailsList[index].Effect,1,1)=="4") then
					r,g,b = recap_temp.ColorHeal.r, recap_temp.ColorHeal.g, recap_temp.ColorHeal.b
				else
					r,g,b = recap_temp.ColorDmgOut.r, recap_temp.ColorDmgOut.g, recap_temp.ColorDmgOut.b
				end
				getglobal("RecapPanelOutgoingDetail"..i.."_Name"):SetText(string.sub(recap_temp.OutgoingDetailsList[index].Effect,2))
				getglobal("RecapPanelOutgoingDetail"..i.."_Name"):SetTextColor(r,g,b)
				getglobal("RecapPanelOutgoingDetail"..i.."_Total"):SetText(recap_temp.OutgoingDetailsList[index][recap.Opt.OutgoingPanelDetail.value])
				if recap.Opt.OutgoingPanelDetail.value=="Total" then
					getglobal("RecapPanelOutgoingDetail"..i.."_Total"):SetTextColor(r,g,b)
				else
					getglobal("RecapPanelOutgoingDetail"..i.."_Total"):SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
				end
				getglobal("RecapPanelOutgoingDetail"..i.."_TotalP"):SetText(recap_temp.OutgoingDetailsList[index].TotalP.."%")
				getglobal("RecapPanelOutgoingDetail"..i.."_TotalP"):SetTextColor(r,g,b)
				item = getglobal("RecapPanelOutgoingDetail"..i)
				item:Show()
				if recap_temp.OutgoingDetailSelected == index then
					item:LockHighlight()
				else
					item:UnlockHighlight()
				end
			else
				item = getglobal("RecapPanelOutgoingDetail"..i)
				item:Hide()
				item:UnlockHighlight()
			end
		end
	end

end

function RecapPanelIncomingDetailsScrollBar_Update()

	local i, index, item, r, g, b

	if recap_temp.Loaded and recap_temp.IncomingDetailsListSize then

		FauxScrollFrame_Update(RecapPanelIncomingDetailsScrollBar,recap_temp.IncomingDetailsListSize-1,6,6)

		for i=1,6 do
			index = i + FauxScrollFrame_GetOffset(RecapPanelIncomingDetailsScrollBar)
			if index < recap_temp.IncomingDetailsListSize then
				if (string.sub(recap_temp.IncomingDetailsList[index].Effect,1,1)=="3") or (string.sub(recap_temp.IncomingDetailsList[index].Effect,1,1)=="4") then
					r,g,b = recap_temp.ColorHeal.r, recap_temp.ColorHeal.g, recap_temp.ColorHeal.b
				else
					r,g,b = recap_temp.ColorDmgIn.r, recap_temp.ColorDmgIn.g, recap_temp.ColorDmgIn.b
				end
				getglobal("RecapPanelIncomingDetail"..i.."_Name"):SetText(string.sub(recap_temp.IncomingDetailsList[index].Effect,2))
				getglobal("RecapPanelIncomingDetail"..i.."_Name"):SetTextColor(r,g,b)
				getglobal("RecapPanelIncomingDetail"..i.."_Total"):SetText(recap_temp.IncomingDetailsList[index][recap.Opt.IncomingPanelDetail.value])
				if recap.Opt.IncomingPanelDetail.value=="Total" then
					getglobal("RecapPanelIncomingDetail"..i.."_Total"):SetTextColor(r,g,b)
				else
					getglobal("RecapPanelIncomingDetail"..i.."_Total"):SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
				end
				getglobal("RecapPanelIncomingDetail"..i.."_TotalP"):SetText(recap_temp.IncomingDetailsList[index].TotalP.."%")
				getglobal("RecapPanelIncomingDetail"..i.."_TotalP"):SetTextColor(r,g,b)
				item = getglobal("RecapPanelIncomingDetail"..i)
				item:Show()
				if recap_temp.IncomingDetailSelected == index then
					item:LockHighlight()
				else
					item:UnlockHighlight()
				end
			else
				item = getglobal("RecapPanelIncomingDetail"..i)
				item:Hide()
				item:UnlockHighlight()
			end
		end
	end

end

function RecapPanel_PopulateOutgoingDetails(arg1,sel)

	local effect, i, totalmiss

	-- Hawksy: guard code
	if not arg1 then
		return
	end
	if (not sel) or (sel < 0) then
		return
	end
	if not recap_temp.OutgoingDetailsList then
		return
	end
	if not recap.Combatant then
		return
	end
	if not recap.Combatant[arg1] then
		return
	end
	if not recap.Combatant[arg1].OutgoingDetail then
		return
	end

	RecapPanelOutgoingEffectText:SetText("--")
	RecapPanelOutgoingElementText:SetText("--")
	RecapPanelOutgoingDamageText:SetText("--")
	RecapPanelOutgoingHitsText:SetText("--")
	RecapPanelOutgoingCritsText:SetText("--")
	RecapPanelOutgoingCrushesText:SetText("--")
	RecapPanelOutgoingTicksText:SetText("--")
	RecapPanelOutgoingHitsAvgText:SetText("--")
	RecapPanelOutgoingCritsAvgText:SetText("--")
	RecapPanelOutgoingCrushAvgText:SetText("--")
	RecapPanelOutgoingTicksAvgText:SetText("--")
	RecapPanelOutgoingHitsMaxText:SetText("--")
	RecapPanelOutgoingCritsMaxText:SetText("--")
	RecapPanelOutgoingCrushMaxText:SetText("--")
	RecapPanelOutgoingTicksMaxText:SetText("--")
	RecapPanelOutgoingMissMissedText:SetText("--")
	RecapPanelOutgoingMissDodgedText:SetText("--")
	RecapPanelOutgoingMissParriedText:SetText("--")
	RecapPanelOutgoingMissBlockedText:SetText("--")
	RecapPanelOutgoingMissAbsorbedText:SetText("--")
	RecapPanelOutgoingMissEvadedText:SetText("--")
	RecapPanelOutgoingMissDeflectedText:SetText("--")
	RecapPanelOutgoingMissResistedText:SetText("--")
	RecapPanelOutgoingMissReflectedText:SetText("--")
	RecapPanelOutgoingMissImmuneText:SetText("--")
	RecapPanelOutgoingMissPText:SetText("--")
	RecapPanelOutgoingCritPText:SetText("--")
	RecapPanelOutgoingCrushPText:SetText("--")

	if sel == 0 then
		return
	end

	if not recap_temp.OutgoingDetailsList[sel] then
		return
	end

	effect = recap_temp.OutgoingDetailsList[sel].Effect

	-- Hawksy: guard code
	if not effect then
		return
	end

	if (string.sub(effect,1,1)=="3") or (string.sub(effect,1,1)=="4") then
		RecapPanelOutgoingDamageLabel:SetText(RECAP_HEAL)
		RecapPanelOutgoingMissPLabel:SetText(RECAP_OVERHEALING)
	else
		RecapPanelOutgoingDamageLabel:SetText(RECAP_DAMAGE)
		RecapPanelOutgoingMissPLabel:SetText(RECAP_MISSES)
	end

	-- Hawksy: guard code
	if not recap.Combatant[arg1].OutgoingDetail[effect] then
		return
	end

	-- effect (repeated so we know which effect we are viewing)
	RecapPanelOutgoingEffectText:SetText(string.sub(effect,2))
	if string.sub(effect,1,1) < "3" then
		RecapPanelOutgoingEffectText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	else
		RecapPanelOutgoingEffectText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	end

	-- element
	RecapPanelOutgoingElementText:SetText(recap.Combatant[arg1].OutgoingDetail[effect].Element or "?")

	-- damage
	RecapPanelOutgoingDamageText:SetText((recap.Combatant[arg1].OutgoingDetail[effect].HitsDmg or 0)+(recap.Combatant[arg1].OutgoingDetail[effect].CritsDmg or 0)+(recap.Combatant[arg1].OutgoingDetail[effect].CrushDmg or 0)+(recap.Combatant[arg1].OutgoingDetail[effect].TicksDmg or 0))

	-- hits
	i = (recap.Combatant[arg1].OutgoingDetail[effect].Hits or 0)
	if i>0 then
		RecapPanelOutgoingHitsText:SetText(i)
		RecapPanelOutgoingHitsMaxText:SetText(recap.Combatant[arg1].OutgoingDetail[effect].HitsMax or 0)
		RecapPanelOutgoingHitsAvgText:SetText(string.format("%d",(((recap.Combatant[arg1].OutgoingDetail[effect].HitsDmg or 0)/i)+0.5)))
	elseif recap.Combatant[arg1].OutgoingDetail[effect].CritsEvents then
		RecapPanelOutgoingHitsText:SetText("0")
	end

	-- crits
	i = (recap.Combatant[arg1].OutgoingDetail[effect].Crits or 0)
	if i>0 then
		RecapPanelOutgoingCritsText:SetText(i)
		RecapPanelOutgoingCritsMaxText:SetText(recap.Combatant[arg1].OutgoingDetail[effect].CritsMax or 0)
		RecapPanelOutgoingCritsAvgText:SetText(string.format("%d",(((recap.Combatant[arg1].OutgoingDetail[effect].CritsDmg or 0)/i)+0.5)))
	elseif recap.Combatant[arg1].OutgoingDetail[effect].CritsEvents then
		RecapPanelOutgoingCritsText:SetText("0")
	end

	-- crushes
	i = (recap.Combatant[arg1].OutgoingDetail[effect].Crushes or 0)
	if i>0 then
		RecapPanelOutgoingCrushesText:SetText(i)
		RecapPanelOutgoingCrushMaxText:SetText(recap.Combatant[arg1].OutgoingDetail[effect].CrushMax or 0)
		RecapPanelOutgoingCrushAvgText:SetText(string.format("%d",(((recap.Combatant[arg1].OutgoingDetail[effect].CrushDmg or 0)/i)+0.5)))
	elseif recap.Combatant[arg1].OutgoingDetail[effect].CritsEvents then
		RecapPanelOutgoingCrushesText:SetText("0")
	end

	-- ticks
	i = (recap.Combatant[arg1].OutgoingDetail[effect].Ticks or 0)
	if i>0 then
		RecapPanelOutgoingTicksText:SetText(i)
		RecapPanelOutgoingTicksMaxText:SetText(recap.Combatant[arg1].OutgoingDetail[effect].TicksMax or 0)
		RecapPanelOutgoingTicksAvgText:SetText(string.format("%d",(((recap.Combatant[arg1].OutgoingDetail[effect].TicksDmg or 0)/i)+0.5)))
	elseif recap.Combatant[arg1].OutgoingDetail[effect].TicksMax then
		RecapPanelOutgoingTicksText:SetText("0")
	end

	-- miss/overheal % (and crit % and crush % calcs)
	if string.sub(effect,1,1) < "3" then
		-- miss
		miss = (recap.Combatant[arg1].OutgoingDetail[effect].Missed or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Dodged or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Parried or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Blocked or 0) +
				(recap.Combatant[arg1].OutgoingDetail[effect].Deflected or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Resisted or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Reflected or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Absorbed or 0) +
				(recap.Combatant[arg1].OutgoingDetail[effect].Immune or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Evaded or 0)
		i = miss + (recap.Combatant[arg1].OutgoingDetail[effect].CritsEvents or 0)
		if i>0 then
			RecapPanelOutgoingCritPText:SetText(((i==recap.Combatant[arg1].OutgoingDetail[effect].Crits) and "100%") or string.format("%.1f%%",100*(recap.Combatant[arg1].OutgoingDetail[effect].Crits or 0)/i))
			RecapPanelOutgoingCrushPText:SetText(((i==recap.Combatant[arg1].OutgoingDetail[effect].Crushes) and "100%") or string.format("%.1f%%",100*(recap.Combatant[arg1].OutgoingDetail[effect].Crushes or 0)/i))
			if (miss==i) then
				RecapPanelOutgoingMissPText:SetText(i.." (100%)")
			else
				RecapPanelOutgoingMissPText:SetText(string.format("%d (%.1f%%)",miss,100*miss/i))
			end
		end
	else
		-- crits for heals
		i = (recap.Combatant[arg1].OutgoingDetail[effect].CritsEvents or 0)
		if i>0 then
			RecapPanelOutgoingCritPText:SetText(((i==recap.Combatant[arg1].OutgoingDetail[effect].Crits) and "100%") or string.format("%.1f%%",100*(recap.Combatant[arg1].OutgoingDetail[effect].Crits or 0)/i))
		end
		-- Hawksy: change of calculation since effect numbers are full heals, not actual heals
		local heal, overheal
		heal = (recap.Combatant[arg1].OutgoingDetail[effect].HitsDmg or 0)+(recap.Combatant[arg1].OutgoingDetail[effect].CritsDmg or 0)+(recap.Combatant[arg1].OutgoingDetail[effect].CrushDmg or 0)+(recap.Combatant[arg1].OutgoingDetail[effect].TicksDmg or 0)
		overheal = (recap.Combatant[arg1].OutgoingDetail[effect].Missed or 0)
		if heal>0 then
			RecapPanelOutgoingMissPText:SetText(string.format("%d (%d%%)",recap.Combatant[arg1].OutgoingDetail[effect].Missed or 0,((100*overheal/heal)+0.5)))
		end
	end

	if string.sub(effect,1,1) < "3" then
		-- non-healing misses
		totalmiss = (recap.Combatant[arg1].OutgoingDetail[effect].Missed or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Dodged or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Parried or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Blocked or 0) +
					(recap.Combatant[arg1].OutgoingDetail[effect].Deflected or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Resisted or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Reflected or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Absorbed or 0) +
					(recap.Combatant[arg1].OutgoingDetail[effect].Immune or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Evaded or 0)
		i = totalmiss + (recap.Combatant[arg1].OutgoingDetail[effect].Hits or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Crits or 0) + (recap.Combatant[arg1].OutgoingDetail[effect].Crushes or 0)
		if i>0 then
			if recap.Combatant[arg1].OutgoingDetail[effect].Missed and (recap.Combatant[arg1].OutgoingDetail[effect].Missed > 0) then
				RecapPanelOutgoingMissMissedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].OutgoingDetail[effect].Missed, 100*recap.Combatant[arg1].OutgoingDetail[effect].Missed/i))
			end
			if recap.Combatant[arg1].OutgoingDetail[effect].Dodged and (recap.Combatant[arg1].OutgoingDetail[effect].Dodged > 0) then
				RecapPanelOutgoingMissDodgedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].OutgoingDetail[effect].Dodged, 100*recap.Combatant[arg1].OutgoingDetail[effect].Dodged/i))
			end
			if recap.Combatant[arg1].OutgoingDetail[effect].Parried and (recap.Combatant[arg1].OutgoingDetail[effect].Parried > 0) then
				RecapPanelOutgoingMissParriedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].OutgoingDetail[effect].Parried, 100*recap.Combatant[arg1].OutgoingDetail[effect].Parried/i))
			end
			if recap.Combatant[arg1].OutgoingDetail[effect].Blocked and (recap.Combatant[arg1].OutgoingDetail[effect].Blocked > 0) then
				RecapPanelOutgoingMissBlockedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].OutgoingDetail[effect].Blocked, 100*recap.Combatant[arg1].OutgoingDetail[effect].Blocked/i))
			end
			if recap.Combatant[arg1].OutgoingDetail[effect].Deflected and (recap.Combatant[arg1].OutgoingDetail[effect].Deflected > 0) then
				RecapPanelOutgoingMissDeflectedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].OutgoingDetail[effect].Deflected, 100*recap.Combatant[arg1].OutgoingDetail[effect].Deflected/i))
			end
			if recap.Combatant[arg1].OutgoingDetail[effect].Resisted and (recap.Combatant[arg1].OutgoingDetail[effect].Resisted > 0) then
				RecapPanelOutgoingMissResistedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].OutgoingDetail[effect].Resisted, 100*recap.Combatant[arg1].OutgoingDetail[effect].Resisted/i))
			end
			if recap.Combatant[arg1].OutgoingDetail[effect].Reflected and (recap.Combatant[arg1].OutgoingDetail[effect].Reflected > 0) then
				RecapPanelOutgoingMissReflectedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].OutgoingDetail[effect].Reflected, 100*recap.Combatant[arg1].OutgoingDetail[effect].Reflected/i))
			end
			if recap.Combatant[arg1].OutgoingDetail[effect].Absorbed and (recap.Combatant[arg1].OutgoingDetail[effect].Absorbed > 0) then
				RecapPanelOutgoingMissAbsorbedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].OutgoingDetail[effect].Absorbed, 100*recap.Combatant[arg1].OutgoingDetail[effect].Absorbed/i))
			end
			if recap.Combatant[arg1].OutgoingDetail[effect].Immune and (recap.Combatant[arg1].OutgoingDetail[effect].Immune > 0) then
				RecapPanelOutgoingMissImmuneText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].OutgoingDetail[effect].Immune, 100*recap.Combatant[arg1].OutgoingDetail[effect].Immune/i))
			end
			if recap.Combatant[arg1].OutgoingDetail[effect].Evaded and (recap.Combatant[arg1].OutgoingDetail[effect].Evaded > 0) then
				RecapPanelOutgoingMissEvadedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].OutgoingDetail[effect].Evaded, 100*recap.Combatant[arg1].OutgoingDetail[effect].Evaded/i))
			end
		end
	end

end

function RecapPanel_PopulateIncomingDetails(arg1,sel)

	local effect, i, totalmiss

	-- Hawksy: guard code
	if not arg1 then
		return
	end
	if (not sel) or (sel < 0) then
		return
	end
	if not recap_temp.IncomingDetailsList then
		return
	end
	if not recap.Combatant then
		return
	end
	if not recap.Combatant[arg1] then
		return
	end
	if not recap.Combatant[arg1].IncomingDetail then
		return
	end

	RecapPanelIncomingEffectText:SetText("--")
	RecapPanelIncomingElementText:SetText("--")
	RecapPanelIncomingDamageText:SetText("--")
	RecapPanelIncomingHitsText:SetText("--")
	RecapPanelIncomingCritsText:SetText("--")
	RecapPanelIncomingCrushesText:SetText("--")
	RecapPanelIncomingTicksText:SetText("--")
	RecapPanelIncomingHitsAvgText:SetText("--")
	RecapPanelIncomingCritsAvgText:SetText("--")
	RecapPanelIncomingCrushAvgText:SetText("--")
	RecapPanelIncomingTicksAvgText:SetText("--")
	RecapPanelIncomingHitsMaxText:SetText("--")
	RecapPanelIncomingCritsMaxText:SetText("--")
	RecapPanelIncomingCrushMaxText:SetText("--")
	RecapPanelIncomingTicksMaxText:SetText("--")
	RecapPanelIncomingMissMissedText:SetText("--")
	RecapPanelIncomingMissDodgedText:SetText("--")
	RecapPanelIncomingMissParriedText:SetText("--")
	RecapPanelIncomingMissBlockedText:SetText("--")
	RecapPanelIncomingMissAbsorbedText:SetText("--")
	RecapPanelIncomingMissEvadedText:SetText("--")
	RecapPanelIncomingMissDeflectedText:SetText("--")
	RecapPanelIncomingMissResistedText:SetText("--")
	RecapPanelIncomingMissReflectedText:SetText("--")
	RecapPanelIncomingMissImmuneText:SetText("--")
	RecapPanelIncomingMissPText:SetText("--")
	RecapPanelIncomingCritPText:SetText("--")
	RecapPanelIncomingCrushPText:SetText("--")

	if sel == 0 then
		return
	end

	if not recap_temp.IncomingDetailsList[sel] then
		return
	end

	effect = recap_temp.IncomingDetailsList[sel].Effect

	-- Hawksy: guard code
	if not effect then
		return
	end

	if (string.sub(effect,1,1)=="3") or (string.sub(effect,1,1)=="4") then
		RecapPanelIncomingDamageLabel:SetText(RECAP_HEAL)
		RecapPanelIncomingMissPLabel:SetText(RECAP_OVERHEALING)
	else
		RecapPanelIncomingDamageLabel:SetText(RECAP_DAMAGE)
		RecapPanelIncomingMissPLabel:SetText(RECAP_MISSES)
	end

	-- Hawksy: guard code
	if not recap.Combatant[arg1].IncomingDetail[effect] then
		return
	end

	-- effect (repeated so we know which effect we are viewing)
	RecapPanelIncomingEffectText:SetText(string.sub(effect,2))
	if string.sub(effect,1,1) < "3" then
		RecapPanelIncomingEffectText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	else
		RecapPanelIncomingEffectText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	end

	-- element
	RecapPanelIncomingElementText:SetText(recap.Combatant[arg1].IncomingDetail[effect].Element or "?")

	-- damage
	RecapPanelIncomingDamageText:SetText((recap.Combatant[arg1].IncomingDetail[effect].HitsDmg or 0)+(recap.Combatant[arg1].IncomingDetail[effect].CritsDmg or 0)+(recap.Combatant[arg1].IncomingDetail[effect].CrushDmg or 0)+(recap.Combatant[arg1].IncomingDetail[effect].TicksDmg or 0))

	-- hits
	i = (recap.Combatant[arg1].IncomingDetail[effect].Hits or 0)
	if i>0 then
		RecapPanelIncomingHitsText:SetText(i)
		RecapPanelIncomingHitsMaxText:SetText(recap.Combatant[arg1].IncomingDetail[effect].HitsMax or 0)
		RecapPanelIncomingHitsAvgText:SetText(string.format("%d",(((recap.Combatant[arg1].IncomingDetail[effect].HitsDmg or 0)/i)+0.5)))
	elseif recap.Combatant[arg1].IncomingDetail[effect].CritsEvents then
		RecapPanelIncomingHitsText:SetText("0")
	end

	-- crits
	i = (recap.Combatant[arg1].IncomingDetail[effect].Crits or 0)
	if i>0 then
		RecapPanelIncomingCritsText:SetText(i)
		RecapPanelIncomingCritsMaxText:SetText(recap.Combatant[arg1].IncomingDetail[effect].CritsMax or 0)
		RecapPanelIncomingCritsAvgText:SetText(string.format("%d",(((recap.Combatant[arg1].IncomingDetail[effect].CritsDmg or 0)/i)+0.5)))
	elseif recap.Combatant[arg1].IncomingDetail[effect].CritsEvents then
		RecapPanelIncomingCritsText:SetText("0")
	end

	-- crushes
	i = (recap.Combatant[arg1].IncomingDetail[effect].Crushes or 0)
	if i>0 then
		RecapPanelIncomingCrushesText:SetText(i)
		RecapPanelIncomingCrushMaxText:SetText(recap.Combatant[arg1].IncomingDetail[effect].CrushMax or 0)
		RecapPanelIncomingCrushAvgText:SetText(string.format("%d",(((recap.Combatant[arg1].IncomingDetail[effect].CrushDmg or 0)/i)+0.5)))
	elseif recap.Combatant[arg1].IncomingDetail[effect].CritsEvents then
		RecapPanelIncomingCrushesText:SetText("0")
	end

	-- ticks
	i = (recap.Combatant[arg1].IncomingDetail[effect].Ticks or 0)
	if i>0 then
		RecapPanelIncomingTicksText:SetText(i)
		RecapPanelIncomingTicksMaxText:SetText(recap.Combatant[arg1].IncomingDetail[effect].TicksMax or 0)
		RecapPanelIncomingTicksAvgText:SetText(string.format("%d",(((recap.Combatant[arg1].IncomingDetail[effect].TicksDmg or 0)/i)+0.5)))
	elseif recap.Combatant[arg1].IncomingDetail[effect].TicksMax then
		RecapPanelIncomingTicksText:SetText("0")
	end

	-- miss/overheal % (and crit % and crush % calcs)
	if string.sub(effect,1,1) < "3" then
		-- miss
		miss = (recap.Combatant[arg1].IncomingDetail[effect].Missed or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Dodged or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Parried or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Blocked or 0) +
				(recap.Combatant[arg1].IncomingDetail[effect].Deflected or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Resisted or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Reflected or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Absorbed or 0) +
				(recap.Combatant[arg1].IncomingDetail[effect].Immune or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Evaded or 0)
		i = miss + (recap.Combatant[arg1].IncomingDetail[effect].CritsEvents or 0)
		if i>0 then
			RecapPanelIncomingCritPText:SetText(((i==recap.Combatant[arg1].IncomingDetail[effect].Crits) and "100%") or string.format("%.1f%%",100*(recap.Combatant[arg1].IncomingDetail[effect].Crits or 0)/i))
			RecapPanelIncomingCrushPText:SetText(((i==recap.Combatant[arg1].IncomingDetail[effect].Crushes) and "100%") or string.format("%.1f%%",100*(recap.Combatant[arg1].IncomingDetail[effect].Crushes or 0)/i))
			if (miss==i) then
				RecapPanelIncomingMissPText:SetText(i.." (100%)")
			else
				RecapPanelIncomingMissPText:SetText(string.format("%d (%.1f%%)",miss,100*miss/i))
			end
		end
	else
		-- crits for heals
		i = (recap.Combatant[arg1].IncomingDetail[effect].CritsEvents or 0)
		if i>0 then
			RecapPanelIncomingCritPText:SetText(((i==recap.Combatant[arg1].IncomingDetail[effect].Crits) and "100%") or string.format("%.1f%%",100*(recap.Combatant[arg1].IncomingDetail[effect].Crits or 0)/i))
		end
		-- Hawksy: change of calculation since effect numbers are full heals, not actual heals
		local heal, overheal
		heal = (recap.Combatant[arg1].IncomingDetail[effect].HitsDmg or 0)+(recap.Combatant[arg1].IncomingDetail[effect].CritsDmg or 0)+(recap.Combatant[arg1].IncomingDetail[effect].CrushDmg or 0)+(recap.Combatant[arg1].IncomingDetail[effect].TicksDmg or 0)
		overheal = (recap.Combatant[arg1].IncomingDetail[effect].Missed or 0)
		if heal>0 then
			RecapPanelIncomingMissPText:SetText(string.format("%d (%d%%)",recap.Combatant[arg1].IncomingDetail[effect].Missed or 0,((100*overheal/heal)+0.5)))
		end
	end

	if string.sub(effect,1,1) < "3" then
		-- non-healing misses
		totalmiss = (recap.Combatant[arg1].IncomingDetail[effect].Missed or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Dodged or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Parried or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Blocked or 0) +
					(recap.Combatant[arg1].IncomingDetail[effect].Deflected or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Resisted or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Reflected or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Absorbed or 0) +
					(recap.Combatant[arg1].IncomingDetail[effect].Immune or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Evaded or 0)
		i = totalmiss + (recap.Combatant[arg1].IncomingDetail[effect].Hits or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Crits or 0) + (recap.Combatant[arg1].IncomingDetail[effect].Crushes or 0)
		if i>0 then
			if recap.Combatant[arg1].IncomingDetail[effect].Missed and (recap.Combatant[arg1].IncomingDetail[effect].Missed > 0) then
				RecapPanelIncomingMissMissedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].IncomingDetail[effect].Missed, 100*recap.Combatant[arg1].IncomingDetail[effect].Missed/i))
			end
			if recap.Combatant[arg1].IncomingDetail[effect].Dodged and (recap.Combatant[arg1].IncomingDetail[effect].Dodged > 0) then
				RecapPanelIncomingMissDodgedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].IncomingDetail[effect].Dodged, 100*recap.Combatant[arg1].IncomingDetail[effect].Dodged/i))
			end
			if recap.Combatant[arg1].IncomingDetail[effect].Parried and (recap.Combatant[arg1].IncomingDetail[effect].Parried > 0) then
				RecapPanelIncomingMissParriedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].IncomingDetail[effect].Parried, 100*recap.Combatant[arg1].IncomingDetail[effect].Parried/i))
			end
			if recap.Combatant[arg1].IncomingDetail[effect].Blocked and (recap.Combatant[arg1].IncomingDetail[effect].Blocked > 0) then
				RecapPanelIncomingMissBlockedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].IncomingDetail[effect].Blocked, 100*recap.Combatant[arg1].IncomingDetail[effect].Blocked/i))
			end
			if recap.Combatant[arg1].IncomingDetail[effect].Deflected and (recap.Combatant[arg1].IncomingDetail[effect].Deflected > 0) then
				RecapPanelIncomingMissDeflectedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].IncomingDetail[effect].Deflected, 100*recap.Combatant[arg1].IncomingDetail[effect].Deflected/i))
			end
			if recap.Combatant[arg1].IncomingDetail[effect].Resisted and (recap.Combatant[arg1].IncomingDetail[effect].Resisted > 0) then
				RecapPanelIncomingMissResistedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].IncomingDetail[effect].Resisted, 100*recap.Combatant[arg1].IncomingDetail[effect].Resisted/i))
			end
			if recap.Combatant[arg1].IncomingDetail[effect].Reflected and (recap.Combatant[arg1].IncomingDetail[effect].Reflected > 0) then
				RecapPanelIncomingMissReflectedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].IncomingDetail[effect].Reflected, 100*recap.Combatant[arg1].IncomingDetail[effect].Reflected/i))
			end
			if recap.Combatant[arg1].IncomingDetail[effect].Absorbed and (recap.Combatant[arg1].IncomingDetail[effect].Absorbed > 0) then
				RecapPanelIncomingMissAbsorbedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].IncomingDetail[effect].Absorbed, 100*recap.Combatant[arg1].IncomingDetail[effect].Absorbed/i))
			end
			if recap.Combatant[arg1].IncomingDetail[effect].Immune and (recap.Combatant[arg1].IncomingDetail[effect].Immune > 0) then
				RecapPanelIncomingMissImmuneText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].IncomingDetail[effect].Immune, 100*recap.Combatant[arg1].IncomingDetail[effect].Immune/i))
			end
			if recap.Combatant[arg1].IncomingDetail[effect].Evaded and (recap.Combatant[arg1].IncomingDetail[effect].Evaded > 0) then
				RecapPanelIncomingMissEvadedText:SetText(string.format("%d (%.1f%%)", recap.Combatant[arg1].IncomingDetail[effect].Evaded, 100*recap.Combatant[arg1].IncomingDetail[effect].Evaded/i))
			end
		end
	end

end

function RecapPanel_OutgoingDetail_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapPanelOutgoingDetailsScrollBar)

	if (index<recap_temp.OutgoingDetailsListSize) and recap_temp.OutgoingDetailSelected==0 then
		RecapPanel_PopulateOutgoingDetails(RecapPanelName:GetText(),index)
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapPanel_OutgoingDetail_OnLeave()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	if recap_temp.OutgoingDetailSelected==0 then
		RecapPanel_PopulateOutgoingDetails(nil,0)
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

-- clicks on individual effect lines
function RecapPanel_OutgoingDetail_OnClick(arg1)

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapPanelOutgoingDetailsScrollBar)
	local linkformat = recap_temp.Localize.OutgoingDamageDetailLink

	if arg1=="LeftButton" and IsShiftKeyDown() and index<recap_temp.OutgoingDetailsListSize then
		if (string.sub(recap_temp.OutgoingDetailsList[index].Effect,1,1)=="3") or (string.sub(recap_temp.OutgoingDetailsList[index].Effect,1,1)=="4") then
			linkformat = recap_temp.Localize.OutgoingHealDetailLink
		end
		Recap_InsertChat(string.format(linkformat,RecapPanelName:GetText(),
							getglobal("RecapPanelOutgoingDetail"..id.."_Name"):GetText(),
							recap_temp.OutgoingDetailsList[index].Element,
							recap_temp.OutgoingDetailsList[index].Total,
							getglobal("RecapPanelOutgoingDetail"..id.."_TotalP"):GetText(),
							RecapPanelName:GetText() ))

	elseif arg1=="RightButton" and index<recap_temp.OutgoingDetailsListSize and IsControlKeyDown() and not IsShiftKeyDown() then
		-- Hawksy: post Recent Outgoing events for this effect for this combatant when Recent Data Mode is active
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(true, RecapPanelName:GetText(), getglobal("RecapPanelOutgoingDetail"..id.."_Name"):GetText(), nil)
		end

	elseif recap_temp.OutgoingDetailSelected==index then
		recap_temp.OutgoingDetailSelected = 0
		RecapPanelOutgoingDetailsScrollBar_Update()
		RecapPanel_PopulateOutgoingDetails(nil,0)

	else
		recap_temp.OutgoingDetailSelected = index
		RecapPanel_PopulateOutgoingDetails(RecapPanelName:GetText(),index)
		RecapPanelOutgoingDetailsScrollBar_Update()
	end
end

function RecapPanel_IncomingDetail_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapPanelIncomingDetailsScrollBar)

	if (index<recap_temp.IncomingDetailsListSize) and recap_temp.IncomingDetailSelected==0 then
		RecapPanel_PopulateIncomingDetails(RecapPanelName:GetText(),index)
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapPanel_IncomingDetail_OnLeave()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	if recap_temp.IncomingDetailSelected==0 then
		RecapPanel_PopulateIncomingDetails(nil,0)
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

-- clicks on individual effect lines
function RecapPanel_IncomingDetail_OnClick(arg1)

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapPanelIncomingDetailsScrollBar)
	local linkformat = recap_temp.Localize.IncomingDamageDetailLink

	if arg1=="LeftButton" and IsShiftKeyDown() and index<recap_temp.IncomingDetailsListSize then
		if (string.sub(recap_temp.IncomingDetailsList[index].Effect,1,1)=="3") or (string.sub(recap_temp.IncomingDetailsList[index].Effect,1,1)=="4") then
			linkformat = recap_temp.Localize.IncomingHealDetailLink
		end
		Recap_InsertChat(string.format(linkformat, getglobal("RecapPanelIncomingDetail"..id.."_Name"):GetText(),
							recap_temp.IncomingDetailsList[index].Element,
							RecapPanelName:GetText(),
							recap_temp.IncomingDetailsList[index].Total,
							getglobal("RecapPanelIncomingDetail"..id.."_TotalP"):GetText(),
							RecapPanelName:GetText() ))

	elseif arg1=="RightButton" and index<recap_temp.IncomingDetailsListSize and IsControlKeyDown() and not IsShiftKeyDown() then
		-- Hawksy: post Recent Incoming events for this effect for this combatant when Recent Data Mode is active
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(true, RecapPanelName:GetText(), getglobal("RecapPanelIncomingDetail"..id.."_Name"):GetText(), nil)
		end

	elseif recap_temp.IncomingDetailSelected==index then
		recap_temp.IncomingDetailSelected = 0
		RecapPanelIncomingDetailsScrollBar_Update()
		RecapPanel_PopulateIncomingDetails(nil,0)

	else
		recap_temp.IncomingDetailSelected = index
		RecapPanel_PopulateIncomingDetails(RecapPanelName:GetText(),index)
		RecapPanelIncomingDetailsScrollBar_Update()
	end
end

function RecapPanel_OnMouseDown(arg1)

	if recap_temp.Loaded and arg1=="LeftButton" then
		RecapPanel:StartMoving()
	end
end

function RecapPanel_OnMouseUp(arg1)

	if recap_temp.Loaded and arg1=="LeftButton" then
		RecapPanel:StopMovingOrSizing()

		-- check for docking
		recap.Opt.PanelAnchor.value = false

		if Recap_Near(RecapFrame:GetRight(),RecapPanel:GetLeft()) then
			if Recap_Near(RecapFrame:GetTop(),RecapPanel:GetTop()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="TOPRIGHT", Panel="TOPLEFT" }
			elseif Recap_Near(RecapFrame:GetBottom(),RecapPanel:GetBottom()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="BOTTOMRIGHT", Panel="BOTTOMLEFT"}
			end
		elseif Recap_Near(RecapFrame:GetLeft(),RecapPanel:GetRight()) then
			if Recap_Near(RecapFrame:GetTop(),RecapPanel:GetTop()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="TOPLEFT", Panel="TOPRIGHT" }
			elseif Recap_Near(RecapFrame:GetBottom(),RecapPanel:GetBottom()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="BOTTOMLEFT", Panel="BOTTOMRIGHT" }
			end
		elseif Recap_Near(RecapFrame:GetRight(),RecapPanel:GetRight()) then
			if Recap_Near(RecapFrame:GetTop(),RecapPanel:GetBottom()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="TOPRIGHT", Panel="BOTTOMRIGHT" }
			elseif Recap_Near(RecapFrame:GetBottom(),RecapPanel:GetTop()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="BOTTOMRIGHT", Panel="TOPRIGHT" }
			end
		elseif Recap_Near(RecapFrame:GetLeft(),RecapPanel:GetLeft()) then
			if Recap_Near(RecapFrame:GetTop(),RecapPanel:GetBottom()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="TOPLEFT", Panel="BOTTOMLEFT" }
			elseif Recap_Near(RecapFrame:GetBottom(),RecapPanel:GetTop()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="BOTTOMLEFT", Panel="TOPLEFT" }
			end
		end

		if recap.Opt.PanelAnchor.value then
			RecapPanel:ClearAllPoints()
			RecapPanel:SetPoint(recap.Opt.PanelAnchor.Panel,"RecapFrame",recap.Opt.PanelAnchor.Main,Recap_PanelOffset("x"),Recap_PanelOffset("y"))
		end

	end
end

function Recap_PanelOffset(arg1)

	local anchor

	anchor = recap.Opt.PanelAnchor.Main..recap.Opt.PanelAnchor.Panel
	if dockoffset[anchor] and arg1 then
		return dockoffset[anchor][arg1]
	else
		return 0
	end
end

-- returns true if the two values are close to each other
function Recap_Near(arg1,arg2)

	local isnear = false

	if (math.max(arg1,arg2)-math.min(arg1,arg2)) < 15 then
	isnear = true
	end

	return isnear
end

function RecapOutgoingDetailHeader_OnMouseUp(arg1)

	if arg1=="RightButton" then
		RecapCreateMenu(recap_temp.Localize.DetailMenu)
	end
end

function RecapIncomingDetailHeader_OnMouseUp(arg1)

	if arg1=="RightButton" then
		RecapCreateMenu(recap_temp.Localize.DetailMenu)
	end
end

-- Hawksy: changed the behaviour of this function -- it now outputs rather than leaving
--   stuff in the buffer -- I know that wasn't the original design, but I think that
--   I prefer sending rather than concatenating -- if someone wants to switch it back
--   replace the "print_chat" just after "<255" with "ChatFrameEditBox:Insert" again,
--   removing the ", chatchan, nil, chatnum".
--   However, if you do this be aware that posting to "Self" (the console) will behave
--   strangely, since the "Self" chat edit box isn't visible so you can't see that it is
--   perhaps half full of something.  You'll post to "Self" and nothing will appear to
--   happen until the hidden chat edit box fills up.  Then you see stuff you might have
--   already forgotten you put there.
-- this substitutes ChatFrameEditBox:Insert.  If 'msg' will fit in the current editbox,
-- it will :Insert.  If not, it will automatically send 'msg' over the current chatType
-- across as many lines as needed, breaking lines by 'delimiter', a single-character
-- string.  (if no delimiter is given it will use "]").  If a delimeter isn't found, it
-- will hard-break every 255 characters.
function Recap_InsertChat(msg,delimiter)

	local cx, i, m_start, m_end, done
	local print_chat = SendChatMessage
	local chatchan, chatnum

	delimiter = string.sub(delimiter or "]",1,1)

	chatchan, chatnum = Recap_GetChannelInfo(nil, nil)
	if chatchan=="Self" then
		print_chat = Recap_Print
	end

	-- If the Clip box on the Options/Reports panel is open, post there instead of to chat or console
	if RecapClipEditBox:IsVisible() then
		print_chat = Recap_LineToClipboard
	end

	-- Hawksy: this will presumably usually return zero
	cx = string.len(ChatFrameEditBox:GetText())

	if cx+string.len(msg)<255 then
		-- Hawksy: modified to force output, not leave it sitting in the chat edit box
		print_chat(msg, chatchan, nil, chatnum)
	else
		if cx>0 then
			print_chat(ChatFrameEditBox:GetText(), chatchan, nil, chatnum)
			ChatFrameEditBox:SetText("")
		end

		cx = string.len(msg)
		m_start = 1

		while not done do
			m_end = false
			for i=m_start,cx do
				if ((string.sub(msg,i,i)==delimiter and (i-m_start)<255)) or ((i-m_start<255) and i==cx) then
					m_end = i
				end
				if (i-m_start)>254 and not m_end then
					m_end = i
				end
			end
			print_chat(string.sub(msg,m_start,m_end), chatchan, nil, chatnum)
			m_start = m_end+1
			if m_start>cx then
				done = true
			end
		end
	end

end

-- click on header on detail panel, dump that header info for all effects
function RecapOutgoingDetailHeader_OnClick()

	local i, lineno
	local text, alltext = "",""
	local print_chat = SendChatMessage
	local chatchan, chatnum

	if arg1=="LeftButton" and IsShiftKeyDown() and recap_temp.Selected~=0 and recap_temp.OutgoingDetailsListSize>1 then

		chatchan, chatnum = Recap_GetChannelInfo(nil, nil)
		if chatchan=="Self" then
			print_chat = Recap_Print
		end

		-- If the Clip box on the Options/Reports panel is open, post there instead of to chat or console
		if RecapClipEditBox:IsVisible() then
			print_chat = Recap_LineToClipboard
		end

		alltext = (recap.Opt.SpamRows.value and "__ Recap: " or "Recap: ")..recap_temp.List[recap_temp.Selected].Name..": "..Recap_GetTooltip(recap.Opt.OutgoingPanelDetail.value)..(recap.Opt.SpamRows.value and " __" or ":")
		lineno = 0
		if recap.Opt.SpamRows.value then
			print_chat(alltext, chatchan, nil, chatnum)
		end
		for i=1,recap_temp.OutgoingDetailsListSize-1 do
			if recap_temp.OutgoingDetailsList[i][recap.Opt.OutgoingPanelDetail.value]~="0.0%" and recap_temp.OutgoingDetailsList[i][recap.Opt.OutgoingPanelDetail.value]~="--" then
				text = string.sub(recap_temp.OutgoingDetailsList[i].Effect,2)..(((string.sub(recap_temp.OutgoingDetailsList[i].Effect,1,1)=="3") or (string.sub(recap_temp.OutgoingDetailsList[i].Effect,1,1)=="4")) and "*" or "")..(recap.Opt.SpamRows.value and ": " or " ")..recap_temp.OutgoingDetailsList[i][recap.Opt.OutgoingPanelDetail.value]..(recap.Opt.OutgoingPanelDetail.value=="Total" and (" ("..recap_temp.OutgoingDetailsList[i].TotalP.."%)") or "")
				if not recap.Opt.SpamRows.value then
					alltext = alltext.." ["..text.."]"
				else
					lineno = lineno + 1
					text = tostring(lineno)..". "..text
					print_chat(text, chatchan, nil, chatnum)
				end
			end
		end
		if not recap.Opt.SpamRows.value then
			Recap_InsertChat(alltext)
		end
	end
end

-- click on header on detail panel, dump that header info for all effects
function RecapIncomingDetailHeader_OnClick()

	local i, lineno
	local text, alltext = "",""
	local print_chat = SendChatMessage
	local chatchan, chatnum

	if arg1=="LeftButton" and IsShiftKeyDown() and recap_temp.Selected~=0 and recap_temp.IncomingDetailsListSize>1 then

		chatchan, chatnum = Recap_GetChannelInfo(nil, nil)
		if chatchan=="Self" then
			print_chat = Recap_Print
		end

		-- If the Clip box on the Options/Reports panel is open, post there instead of to chat or console
		if RecapClipEditBox:IsVisible() then
			print_chat = Recap_LineToClipboard
		end

		alltext = (recap.Opt.SpamRows.value and "__ Recap: " or "Recap: ")..recap_temp.List[recap_temp.Selected].Name..": "..RECAP_INCOMING.." "..Recap_GetTooltip(recap.Opt.IncomingPanelDetail.value)..(recap.Opt.SpamRows.value and " __" or ":")
		lineno = 0
		if recap.Opt.SpamRows.value then
			print_chat(alltext, chatchan, nil, chatnum)
		end
		for i=1,recap_temp.IncomingDetailsListSize-1 do
			if recap_temp.IncomingDetailsList[i][recap.Opt.IncomingPanelDetail.value]~="0.0%" and recap_temp.IncomingDetailsList[i][recap.Opt.IncomingPanelDetail.value]~="--" then
				text = string.sub(recap_temp.IncomingDetailsList[i].Effect,2)..(((string.sub(recap_temp.IncomingDetailsList[i].Effect,1,1)=="3") or (string.sub(recap_temp.IncomingDetailsList[i].Effect,1,1)=="4")) and "*" or "")..(recap.Opt.SpamRows.value and ": " or " ")..recap_temp.IncomingDetailsList[i][recap.Opt.IncomingPanelDetail.value]..(recap.Opt.IncomingPanelDetail.value=="Total" and (" ("..recap_temp.IncomingDetailsList[i].TotalP.."%)") or "")
				if not recap.Opt.SpamRows.value then
					alltext = alltext.." ["..text.."]"
				else
					lineno = lineno + 1
					text = tostring(lineno)..". "..text
					print_chat(text, chatchan, nil, chatnum)
				end
			end
		end
		if not recap.Opt.SpamRows.value then
			Recap_InsertChat(alltext)
		end
	end
end

RecapPanel_lua_367 = true
